if(NOT Boost_CONFIG_INCLUDED)
set(Boost_CONFIG_INCLUDED 1)

set(BOOST_VERSION 1.75.0)
set(Boost_VERSION 107500) # BOOST_VERSION value from boost/version.hpp needed by cmake FindBoost.cmake

include(Common)

external_project_directories(boost ${BOOST_VERSION} BOOST_TMP_DIR BOOST_STAMP_DIR BOOST_DOWNLOAD_DIR BOOST_SOURCE_DIR BOOST_BINARY_DIR BOOST_INSTALL_DIR)
append_target_architecture_directory(${BOOST_INSTALL_DIR} BOOST_INSTALL_DIR)

if(NOT BOOST_ROOT)
	set(BOOST_ROOT ${BOOST_INSTALL_DIR})
endif()	

function(with_boost_libraries RESULT)
	foreach(LIBRARY ${ARGN})
		set(WITH_LIBRARIES ${WITH_LIBRARIES} --with-${LIBRARY})
	endforeach()
	set(${RESULT} ${WITH_LIBRARIES} PARENT_SCOPE)
endfunction()

function(with_boost_prefix_libraries RESULT)
	foreach(LIBRARY ${ARGN})
		set(BOOST_LIBRARIES ${BOOST_LIBRARIES} boost_${LIBRARY})
	endforeach()
	set(${RESULT} ${BOOST_LIBRARIES} PARENT_SCOPE)
endfunction()

function(boost_address_model RESULT)
    if(TARGET_32_BITS)
    	set(${RESULT} 32 PARENT_SCOPE)
    elseif(TARGET_64_BITS)
    	set(${RESULT} 64 PARENT_SCOPE)
    endif()
endfunction()

function(boost_bootstrap_command RESULT)
    set(BOOST_BOOTSTRAP_COMMAND ./bootstrap.sh)
    if(BOOST_VERSION VERSION_LESS 1.61)
        # see https://stackoverrun.com/fr/q/4788765
        set(BOOST_BOOTSTRAP_COMMAND ${BOOST_BOOTSTRAP_COMMAND} --with-toolset=mingw)
     endif()    
     set(${RESULT} ${BOOST_BOOTSTRAP_COMMAND} PARENT_SCOPE)
endfunction()

function(boost_install_command RESULT)
    set(BOOST_INSTALL_COMMAND ${CMAKE_COMMAND} -E copy_directory "${BOOST_SOURCE_DIR}/boost" "${BOOST_INSTALL_DIR}/include/boost")
    if(BOOST_VERSION VERSION_LESS 1.61)
        # see https://stackoverflow.com/questions/5012429/building-boost-under-msys-cant-find-mingw-jam
        # replace 'mingw' by 'gcc' in file project-config.jam
        set(BOOST_POST_INSTALL_COMMAND sed -i "s/mingw/gcc/g" ${BOOST_SOURCE_DIR}/project-config.jam)
        set(BOOST_INSTALL_COMMAND ${BOOST_INSTALL_COMMAND} COMMAND ${BOOST_POST_INSTALL_COMMAND})
    endif()    
    set(${RESULT} ${BOOST_INSTALL_COMMAND} PARENT_SCOPE)
endfunction()

function(boost_build_command RESULT)
    set(${RESULT} b2 PARENT_SCOPE)
endfunction()

set(BOOST_EXTERNAL boost_external)

function(install_boost)
    if(NOT BOOST_INSTALL)
    	message(STATUS "Install Boost Libraries ${BOOST_VERSION}")
    
    	underscore_version(${BOOST_VERSION} BOOST_UNDERSCORE_VERSION)
    	set(BOOST_UNDERSCORE boost_${BOOST_UNDERSCORE_VERSION})
    	if(UNIX)
        	set(BOOST_URL http://downloads.sourceforge.net/project/boost/boost/${BOOST_VERSION}/${BOOST_UNDERSCORE}.tar.gz)
        	set(BOOST_MD5 319c6ffbbeccc366f14bb68767a6db79)
        else()
            if(BOOST_VERSION VERSION_LESS 1.71.0)
       	        set(BOOST_URL http://downloads.sourceforge.net/project/boost/boost/${BOOST_VERSION}/${BOOST_UNDERSCORE}.zip)
            else()
       	        set(BOOST_URL https://dl.bintray.com/boostorg/release/${BOOST_VERSION}/source/${BOOST_UNDERSCORE}.zip)
       	        # SHA256 HASH in file https://dl.bintray.com/boostorg/release/${BOOST_VERSION}/source/${BOOST_UNDERSCORE}.zip.json
            endif()       	        
       	    if(BOOST_VERSION VERSION_EQUAL 1.58.0)
       	        set(BOOST_MD5 b0605a9323f1e960f7434dbbd95a7a5c)
       	    elseif(BOOST_VERSION VERSION_EQUAL 1.59.0)
        	    set(BOOST_MD5 08d29a2d85db3ebc8c6fdfa3a1f2b83c)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.60.0)
        	    set(BOOST_MD5 0cc5b9cf9ccdf26945b225c7338b4288)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.61.0)
        	    set(BOOST_MD5 015ae4afa6f3e597232bfe1dab949ace)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.62.0)
        	    set(BOOST_MD5 9495bfde25f46698edb8b8eb02e55eab)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.63.0)
        	    set(BOOST_MD5 3c706b3fc749884ea5510c39474fd732)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.64.0)
        	    set(BOOST_MD5 36093e4018aecd5b0e31e80457ac5fc1)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.65.1)
        	    set(BOOST_MD5 9824a7a3e25c9d4fdf2def07bce8651c)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.66.0)
        	    set(BOOST_MD5 5d9857fc54171c450f9061cedda9a654)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.67.0)
        	    set(BOOST_MD5 6da1ba65f8d33b1d306616e5acd87f67)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.68.0)
        	    set(BOOST_MD5 f4096c4583947b0eb103c8539f1623a3)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.69.0)
        	    set(BOOST_MD5 aec39b2e85552077e7f5c4e8cf9240cd)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.70.0)
        	    set(BOOST_MD5 a110ebd91a3d2c34c72ace09c92ae50b)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.71.0)
        	    set(BOOST_HASH SHA256=85a94ac71c28e59cf97a96714e4c58a18550c227ac8b0388c260d6c717e47a69)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.72.0)
        	    set(BOOST_HASH SHA256=8c20440aaba21dd963c0f7149517445f50c62ce4eb689df2b5544cc89e6e621e)
      	    elseif(BOOST_VERSION VERSION_EQUAL 1.73.0)
        	    set(BOOST_HASH SHA256=0909a79524f857ef54570ceef8f397cc0629202532cc997785479c7c08bbc2a4)
       	    elseif(BOOST_VERSION VERSION_EQUAL 1.74.0)
        	    set(BOOST_HASH SHA256=a0e7ce67c52d816708fdeccdd8c9725626ba61254c13c18770498cacd514710a)
       	    elseif(BOOST_VERSION VERSION_EQUAL 1.75.0)
        	    set(BOOST_HASH SHA256=caf36d7c13b3d8ce62282a64a695113945a13b0f1796a45160726d04295f95ed)
            endif()        	    
    	endif()
    	
    	boost_bootstrap_command(BOOST_BOOTSTRAP_COMMAND)
    	boost_install_command(BOOST_INSTALL_COMMAND)
    
    	ExternalProject_Add(${BOOST_EXTERNAL}
    		TMP_DIR ${BOOST_TMP_DIR}
    		STAMP_DIR ${BOOST_STAMP_DIR}
    		DOWNLOAD_DIR ${BOOST_DOWNLOAD_DIR}
    		SOURCE_DIR ${BOOST_SOURCE_DIR}
    		INSTALL_DIR ${BOOST_INSTALL_DIR}
    		
    		URL ${BOOST_URL}
    		URL_MD5 ${BOOST_MD5}
    		URL_HASH ${BOOST_HASH}
    		BUILD_IN_SOURCE 1
    		CONFIGURE_COMMAND ""
    		BUILD_COMMAND ${BOOST_BOOTSTRAP_COMMAND}
    	    INSTALL_COMMAND ${BOOST_INSTALL_COMMAND})
    	    
        set(BOOST_INSTALL 1 PARENT_SCOPE)	    
    endif()
endfunction()
	    
macro(find_boost)
    find_package(Boost ${BOOST_VERSION} EXACT)
    if(NOT Boost_FOUND)
    	install_boost()
      	include_directories(${BOOST_INSTALL_DIR}/include)
    else()
      	include_directories(${Boost_INCLUDE_DIRS})
    endif()
endmacro()

macro(require_boost_components)
    find_package(Boost ${BOOST_VERSION} REQUIRED EXACT COMPONENTS ${ARGN})
    include_directories(${Boost_INCLUDE_DIRS})
endmacro()

macro(find_boost_components)
    find_package(Boost ${BOOST_VERSION} EXACT COMPONENTS ${ARGN})
    if(NOT Boost_FOUND)
    	install_boost()
      	include_directories(${BOOST_INSTALL_DIR}/include)
    else()
      	include_directories(${Boost_INCLUDE_DIRS})
    endif()
endmacro()

remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_boost
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${BOOST_SOURCE_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${BOOST_INSTALL_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${BOOST_TMP_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${BOOST_STAMP_DIR})	

endif()
