if(NOT CGAL_CONFIG_INCLUDED)
set(CGAL_CONFIG_INCLUDED 1)

set(CGAL_VERSION 5.2)

include(Common)
include(CGALBoost)
include(MPFR)
	
external_project_directories(cgal ${CGAL_VERSION} CGAL_TMP_DIR CGAL_STAMP_DIR CGAL_DOWNLOAD_DIR CGAL_SOURCE_DIR CGAL_BINARY_DIR CGAL_INSTALL_DIR)
if(CGAL_VERSION VERSION_LESS 5) # CGAL version 5.x is header only by default
    append_target_architecture_directory(${CGAL_TMP_DIR} CGAL_TMP_DIR)
    append_target_architecture_directory(${CGAL_STAMP_DIR} CGAL_STAMP_DIR)
    append_target_architecture_directory(${CGAL_BINARY_DIR} CGAL_BINARY_DIR)
    append_target_architecture_directory(${CGAL_INSTALL_DIR} CGAL_INSTALL_DIR)
endif()

if(NOT CGAL_ROOT)
	set(CGAL_ROOT ${CGAL_INSTALL_DIR})
endif()

function(cgal_cmake_args RESULT)
    set(CXX_FLAGS "${CXX_ARCH}")
    if(CGAL_VERSION VERSION_LESS 5)
        set(CXX_FLAGS "${CXX_FLAGS} -Wno-attributes")
        set(WITH_CGAL_CORE ON)
    else()
        set(WITH_CGAL_CORE OFF)
    endif()
    if((CGAL_VERSION VERSION_GREATER 5) OR (CGAL_VERSION VERSION_EQUAL 5))
        set(CXX_FLAGS "${CXX_FLAGS} -I${GMP_INCLUDE_DIR}")
    endif()
	set(${RESULT} 
	    -DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE}
		-DCMAKE_CXX_COMPILER=${CMAKE_CXX_COMPILER} 
		-DCMAKE_CXX_FLAGS=${CXX_FLAGS}
		-DCMAKE_INSTALL_PREFIX=${CGAL_INSTALL_DIR} 
		-DBOOST_ROOT=${BOOST_ROOT} 
		-DGMP_LIBRARIES=${GMP_LIBRARY}
		-DGMP_INCLUDE_DIR=${GMP_INCLUDE_DIR}
        -DMPFR_LIBRARIES=${MPFR_LIBRARY}
 		-DMPFR_INCLUDE_DIR=${MPFR_INCLUDE_DIR}
		-DWITH_CGAL_Core=${WITH_CGAL_CORE}
		-DWITH_CGAL_ImageIO=OFF
		-DWITH_CGAL_Qt5=OFF 
		${CGAL_SOURCE_DIR} 
		PARENT_SCOPE)
endfunction()

set(CGAL_EXTERNAL cgal_external)

function(install_cgal)
	message(STATUS "Install CGAL Library ${CGAL_VERSION}")
	
	if(CGAL_VERSION VERSION_LESS 5.1)
	    set(CGAL_URL "https://github.com/CGAL/cgal/releases/download/releases/CGAL-${CGAL_VERSION}/CGAL-${CGAL_VERSION}.zip")
	else()
	    set(CGAL_URL "https://github.com/CGAL/cgal/releases/download/v${CGAL_VERSION}/CGAL-${CGAL_VERSION}.zip")
	endif()    
    if(CGAL_VERSION VERSION_EQUAL 4.7)
	    set(CGAL_MD5 ebd5ae3dc7e62b2bcf03169c62275a42)
    elseif(CGAL_VERSION VERSION_EQUAL 4.8.2)
	    set(CGAL_MD5 3d5a42d7214cde56100f2fba1de7738d)
    elseif(CGAL_VERSION VERSION_EQUAL 4.9.1)
	    set(CGAL_MD5 cd2c9be3ad0e44a37c9c0df57bcaf306)
    elseif(CGAL_VERSION VERSION_EQUAL 4.10.2)
	    set(CGAL_MD5 640fde604c1f16473874f657ee6c2568)
    elseif(CGAL_VERSION VERSION_EQUAL 4.11.3)
	    set(CGAL_MD5 e08eeeb3d66a03f35199a1d31f0fcf4f)
    elseif(CGAL_VERSION VERSION_EQUAL 4.12.2)
	    set(CGAL_MD5 5a44f25f09973b3f3a71fed10beb89de)
    elseif(CGAL_VERSION VERSION_EQUAL 4.13.2)
	    set(CGAL_MD5 6ae9ac9b5279d3aa0bc2779d669bb3dd)
    elseif(CGAL_VERSION VERSION_EQUAL 4.14.3)
	    set(CGAL_MD5 3095c197ef925ec6e1a747a79aa04705)
    elseif(CGAL_VERSION VERSION_EQUAL 5.0.3)
	    set(CGAL_MD5 c3decc8791f1cb2a629f95a0874ab7c4)
    elseif(CGAL_VERSION VERSION_EQUAL 5.1)
	    set(CGAL_MD5 0902ed8aa65552fd32564ae2073cb3e9)
    elseif(CGAL_VERSION VERSION_EQUAL 5.2)
	    set(CGAL_MD5 396f1977f3c365fa7f05d88757877994)
	endif()
	
	cgal_cmake_args(CGAL_CMAKE_ARGS)
	
	ExternalProject_Add(${CGAL_EXTERNAL}
		TMP_DIR ${CGAL_TMP_DIR}
		STAMP_DIR ${CGAL_STAMP_DIR}
		DOWNLOAD_DIR ${CGAL_DOWNLOAD_DIR}
		SOURCE_DIR ${CGAL_SOURCE_DIR}
		BINARY_DIR ${CGAL_BINARY_DIR}
		INSTALL_DIR ${CGAL_INSTALL_DIR}
        LOG_INSTALL 1
		
		URL ${CGAL_URL}
		URL_MD5 ${CGAL_MD5}
		CMAKE_ARGS ${CGAL_CMAKE_ARGS}
		BUILD_COMMAND $(MAKE)
		INSTALL_COMMAND $(MAKE) install)
		
	add_dependencies(${CGAL_EXTERNAL} ${MPFR_EXTERNAL} ${CGAL_BOOST_EXTERNAL})
endfunction()

macro(require_cgal)
    require_cgal_boost()
    require_mpfr()
    find_package(CGAL REQUIRED)
	include_directories(${CGAL_INCLUDE_DIR})
	link_directories(${CGAL_LIBRARY_DIR})
endmacro()

macro(find_cgal)
    find_cgal_boost()
    if(MINGW)
        find_mpfr()
    else()
        require_mpfr()
    endif()            
    find_package(CGAL)
    if(NOT CGAL_FOUND)
    	install_cgal()
	  	include_directories(${CGAL_INSTALL_DIR}/include)
		link_directories(${CGAL_INSTALL_DIR}/lib)
		if(MINGW)
		    set(CGAL_RUNTIME_DIR ${CGAL_INSTALL_DIR}/bin)
		else()
		    set(CGAL_RUNTIME_DIR ${CGAL_INSTALL_DIR}/lib)
		endif()
    else()
	  	include_directories(${CGAL_INCLUDE_DIR})
		link_directories(${CGAL_LIBRARY_DIR})
    endif()
endmacro()
	
remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_cgal
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CGAL_SOURCE_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CGAL_BINARY_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CGAL_INSTALL_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CGAL_TMP_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CGAL_STAMP_DIR})	

endif()
   