if(NOT CGALBoost_CONFIG_INCLUDED)
set(CGALBoost_CONFIG_INCLUDED 1)

include(Common)
include(Boost)

if(CGAL_VERSION VERSION_LESS 5.1)
    # The CGAL_Core library no longer requires Boost.Thread, even if the g++ compiler is used
    # see https://www.cgal.org/2020/09/08/cgal51/
    set(CGAL_BOOST_LIBRARIES thread)
endif()

with_boost_prefix_libraries(CGAL_BOOST_PREFIX_LIBRARIES ${CGAL_BOOST_LIBRARIES})

append_target_architecture_directory(${BOOST_TMP_DIR} CGAL_BOOST_TMP_DIR)
append_target_architecture_directory(${BOOST_STAMP_DIR} CGAL_BOOST_STAMP_DIR)

set(CGAL_BOOST_EXTERNAL cgal_boost_external)

function(cgal_boost_build_command LIBRARY THREADING RESULT)
	boost_build_command(BOOST_BUILD_COMMAND)
	boost_address_model(ADDRESS_MODEL)
    with_boost_libraries(WITH_LIBRARY ${LIBRARY})
    set(CXXFLAGS "${CXX_ARCH}")
    if(BOOST_VERSION VERSION_LESS 1.60)
        set(CXXFLAGS "${CXXFLAGS} -Wno-deprecated-declarations -Wno-cast-function-type")
    elseif(BOOST_VERSION VERSION_LESS 1.62)
         set(CXXFLAGS "${CXXFLAGS} -Wno-cast-function-type")
    endif()
    message(STATUS "GEOFIS_BOOST_CXXFLAGS: ${CXXFLAGS}")
    set(${RESULT} ${BOOST_SOURCE_DIR}/${BOOST_BUILD_COMMAND} --prefix=${BOOST_INSTALL_DIR} --libdir=${BOOST_INSTALL_DIR}/lib --layout=system ${WITH_LIBRARY} address-model=${ADDRESS_MODEL} cxxflags=${CXXFLAGS} link=shared threading=${THREADING} variant=release toolset=gcc -j${PROCESSOR_COUNT} install PARENT_SCOPE) 
endfunction()

function(install_cgal_boost_libraries)
    cgal_boost_build_command(system single SYSTEM_BUILD_COMMAND)
    cgal_boost_build_command(thread multi THREAD_BUILD_COMMAND)
    
	ExternalProject_Add(${CGAL_BOOST_EXTERNAL}
		TMP_DIR ${CGAL_BOOST_TMP_DIR}
		STAMP_DIR ${CGAL_BOOST_STAMP_DIR}
		SOURCE_DIR ${BOOST_SOURCE_DIR}
		INSTALL_DIR ${BOOST_INSTALL_DIR}
		
		DOWNLOAD_COMMAND ""
		BUILD_IN_SOURCE 1
		CONFIGURE_COMMAND ""
		BUILD_COMMAND ${SYSTEM_BUILD_COMMAND}
		COMMAND       ${THREAD_BUILD_COMMAND}
	    INSTALL_COMMAND "")

	ExternalProject_Add_Step(${CGAL_BOOST_EXTERNAL} force-build
        DEPENDERS build
        ALWAYS 1)

    add_dependencies(${CGAL_BOOST_EXTERNAL} ${BOOST_EXTERNAL})
endfunction()
		
macro(require_cgal_boost)
    if(CGAL_BOOST_LIBRARIES)
        require_boost_components(${CGAL_BOOST_LIBRARIES})
    endif()
endmacro()
		
macro(find_cgal_boost)
    if(CGAL_BOOST_LIBRARIES)
        find_boost_components(${CGAL_BOOST_LIBRARIES})
        if(NOT Boost_FOUND)
        	install_cgal_boost_libraries()
    		link_directories(${BOOST_INSTALL_DIR}/lib)
        else()
    		link_directories(${Boost_LIBRARY_DIRS})
        endif()
    endif()
endmacro()
		
endif()
