if(NOT Common_CONFIG_INCLUDED)
set(Common_CONFIG_INCLUDED 1)

include(Path)
include(ProcessorCount)
include(TargetArchitecture)
include(ExternalProject)

ProcessorCount(PROCESSOR_COUNT)

get_filename_component(CXX_RUNTIME_DIR ${CMAKE_CXX_COMPILER} PATH)

function(underscore_version VERSION RESULT)
	string(REPLACE "." "_" UNDERSCORE_VERSION ${VERSION})
	set(${RESULT} ${UNDERSCORE_VERSION} PARENT_SCOPE)
endfunction()

function(required_program PROGRAM)
	find_program(PROGRAM_FOUND ${PROGRAM})
	if(NOT PROGRAM_FOUND)
		message(FATAL_ERROR "${PROGRAM} program not found.")
	endif()
endfunction()
	
function(external_project_directories LIBRARY LIBRARY_VERSION RESULT_TMP_DIR RESULT_STAMP_DIR RESULT_DOWNLOAD_DIR RESULT_SOURCE_DIR RESULT_BINARY_DIR RESULT_INSTALL_DIR)
	set(LIBRARY_SUFFIX ${LIBRARY}/${LIBRARY_VERSION})
	set(${RESULT_TMP_DIR} ${CMAKE_EXTERNALS}/tmp/${LIBRARY_SUFFIX} PARENT_SCOPE)
	set(${RESULT_STAMP_DIR} ${CMAKE_EXTERNALS}/Stamp/${LIBRARY_SUFFIX} PARENT_SCOPE)
	set(${RESULT_DOWNLOAD_DIR} ${CMAKE_EXTERNALS}/Download/${LIBRARY} PARENT_SCOPE)
	set(${RESULT_SOURCE_DIR} ${CMAKE_EXTERNALS}/Source/${LIBRARY_SUFFIX} PARENT_SCOPE)
	set(${RESULT_BINARY_DIR} ${CMAKE_EXTERNALS}/Build/${LIBRARY_SUFFIX} PARENT_SCOPE)
	set(${RESULT_INSTALL_DIR} ${CMAKE_EXTERNALS}/Install/${LIBRARY_SUFFIX} PARENT_SCOPE)
endfunction()

function(append_build_type_directory DIRECTORY RESULT)
	set(${RESULT} ${DIRECTORY}/${CMAKE_BUILD_TYPE} PARENT_SCOPE)
endfunction()

function(print_include_directories)
	get_property(directories DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR} PROPERTY INCLUDE_DIRECTORIES)
	foreach(directory ${directories})
	  message(STATUS "include directory='${directory}'")
	endforeach()
endfunction()
		
function(add_tests TEST_RUNNER PATH)
	foreach(TEST ${ARGN})
		get_filename_component(TEST_NAME ${TEST} NAME_WE)
		add_test(${TEST_NAME} ${TEST_RUNNER} ${TEST_NAME})
	    if(WIN32)
		    set_tests_properties(${TEST_NAME} PROPERTIES ENVIRONMENT "PATH=${PATH}")
		else()
		    set_tests_properties(${TEST_NAME} PROPERTIES ENVIRONMENT "LD_LIBRARY_PATH=${PATH}")
		endif()
	endforeach()
endfunction()
		
function(is_snapshot_version VERSION RESULT)
    if("${VERSION}" MATCHES "SNAPSHOT$")
        set(${RESULT} ON PARENT_SCOPE)
    else()
       set(${RESULT} OFF PARENT_SCOPE)
    endif()
endfunction()
		
function(remove_directory_command RESULT)
    if(MSYS)
        set(${RESULT} rm -rf PARENT_SCOPE)
    else()        
        set(${RESULT} ${CMAKE_COMMAND} -E remove_directory PARENT_SCOPE)
    endif()        
endfunction()
		
#function(copy_file _src _dst)
#    message(STATUS "copy_file from ${_src} to ${_dst}")
#    if (CMAKE_VERBOSE_MAKEFILE)
#        message(STATUS "Copy file from ${_src} to ${_dst}")
#    endif (CMAKE_VERBOSE_MAKEFILE)
#    
    # Creates directory if necessary
#    get_filename_component(_path ${_dst} PATH)
#    file(MAKE_DIRECTORY ${_path})
#    
#    exec_program(${CMAKE_COMMAND} ARGS "-E copy_if_different \"${_src}\" \"${_dst}\"" )
#endfunction(copy_file)		
#		
#function(copy_dir_recursive SOURCE DESTINATION)
#    message(STATUS "copy_dir_recursive from ${SOURCE} to ${DESTINATION}")
#    get_filename_component(RELATIVE_PATH ${SOURCE} PATH)
#    file(GLOB_RECURSE FILES ${SOURCE})
#    message(STATUS "FILES: " ${FILES})
#    if(FILES)
#        foreach(FILE ${FILES})
#            file(RELATIVE_PATH path ${RELATIVE_PATH} ${FILE})
#            get_filename_component(path ${path} PATH)
#            copy_file(${FILE} ${DESTINATION}/${path}/.)
#        endforeach()
#    endif()
#endfunction()

set(CMAKE_CXX_FLAGS_RELEASE "-DNDEBUG -Wall -O2")

endif()