set(CPPUNIT_VERSION 1.13.2)
set(CPPUNIT_RUNTIME_LIBRARY_SUFFIX "1-13-0")

include(Common)

external_project_directories(cppunit ${CPPUNIT_VERSION} CPPUNIT_TMP_DIR CPPUNIT_STAMP_DIR CPPUNIT_DOWNLOAD_DIR CPPUNIT_SOURCE_DIR CPPUNIT_BINARY_DIR CPPUNIT_INSTALL_DIR)
append_target_architecture_directory(${CPPUNIT_TMP_DIR} CPPUNIT_TMP_DIR)
append_target_architecture_directory(${CPPUNIT_STAMP_DIR} CPPUNIT_STAMP_DIR)
append_target_architecture_directory(${CPPUNIT_BINARY_DIR} CPPUNIT_BINARY_DIR)
append_target_architecture_directory(${CPPUNIT_INSTALL_DIR} CPPUNIT_INSTALL_DIR)

if(NOT CPPUNIT_ROOT)
	set(CPPUNIT_ROOT ${CPPUNIT_INSTALL_DIR})
endif()

function(cppunit_configure_command RESULT)
    msys_path(${CPPUNIT_SOURCE_DIR} CPPUNIT_MSYS_SOURCE_DIR)
    set(CXXFLAGS "${CXX_ARCH} -Wp,-w")
    set(CONFIGURE_COMMAND ${CPPUNIT_SOURCE_DIR}/configure --prefix=${CPPUNIT_INSTALL_DIR} --srcdir=${CPPUNIT_MSYS_SOURCE_DIR})
    set(${RESULT} ${CONFIGURE_COMMAND} CXXFLAGS=${CXXFLAGS} PARENT_SCOPE)
endfunction()

set(CPPUNIT_EXTERNAL cppunit_external)

function(install_cppunit)
	message(STATUS "Install CppUnit Library ${CPPUNIT_VERSION}")
	
	set(CPPUNIT_URL http://dev-www.libreoffice.org/src/cppunit-${CPPUNIT_VERSION}.tar.gz)
	set(CPPUNIT_MD5 d1c6bdd5a76c66d2c38331e2d287bc01)
	
	cppunit_configure_command(CPPUNIT_CONFIGURE_COMMAND)
	
	ExternalProject_Add(${CPPUNIT_EXTERNAL}
		TMP_DIR ${CPPUNIT_TMP_DIR}
		STAMP_DIR ${CPPUNIT_STAMP_DIR}
		DOWNLOAD_DIR ${CPPUNIT_DOWNLOAD_DIR}
		SOURCE_DIR ${CPPUNIT_SOURCE_DIR}
		BINARY_DIR ${CPPUNIT_BINARY_DIR}
		INSTALL_DIR ${CPPUNIT_INSTALL_DIR}
		
		URL ${CPPUNIT_URL}
		URL_MD5 ${CPPUNIT_MD5}
		CONFIGURE_COMMAND ${CPPUNIT_CONFIGURE_COMMAND}
		BUILD_COMMAND $(MAKE)
		INSTALL_COMMAND $(MAKE) install)
endfunction()

macro(require_cppunit)
	find_package(CppUnit REQUIRED)
  	include_directories(${CppUnit_INCLUDE_DIR})
	link_directories(${CppUnit_LIBRARY_DIR})
endmacro()
	
macro(find_cppunit)
	find_package(CppUnit)
	if(NOT CppUnit_FOUND)
		install_cppunit()
	  	include_directories(${CPPUNIT_INSTALL_DIR}/include)
		link_directories(${CPPUNIT_INSTALL_DIR}/lib)
		if(MINGW)
		    set(CppUnit_RUNTIME_DIR ${CPPUNIT_INSTALL_DIR}/bin)
		else() 
		    set(CppUnit_RUNTIME_DIR ${CPPUNIT_INSTALL_DIR}/lib)
		endif()   
	else()
	  	include_directories(${CppUnit_INCLUDE_DIR})
		link_directories(${CppUnit_LIBRARY_DIR})
	endif()
endmacro()
	
remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_cppunit
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CPPUNIT_SOURCE_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CPPUNIT_BINARY_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CPPUNIT_INSTALL_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CPPUNIT_TMP_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${CPPUNIT_STAMP_DIR})	
	