if(NOT FISPro_CONFIG_INCLUDED)
set(FISPro_CONFIG_INCLUDED 1)

include(FISProVersion)
include(Common)

external_project_directories(fispro ${FISPRO_VERSION} FISPRO_TMP_DIR FISPRO_STAMP_DIR FISPRO_DOWNLOAD_DIR FISPRO_SOURCE_DIR FISPRO_BINARY_DIR FISPRO_INSTALL_DIR)
append_target_architecture_directory(${FISPRO_TMP_DIR} FISPRO_TMP_DIR)
append_target_architecture_directory(${FISPRO_STAMP_DIR} FISPRO_STAMP_DIR)
append_target_architecture_directory(${FISPRO_BINARY_DIR} FISPRO_BINARY_DIR)
append_target_architecture_directory(${FISPRO_INSTALL_DIR} FISPRO_INSTALL_DIR)

if(NOT FISPRO_ROOT)
	set(FISPRO_ROOT ${FISPRO_INSTALL_DIR})
endif()

function(fispro_cmake_args RESULT)
    set(CXX_FLAGS "${CXX_ARCH} -Wall -Wno-unused-but-set-variable -Wno-unused-parameter -Wno-unused-result")
	set(${RESULT} 
		-DCMAKE_CXX_COMPILER=${CMAKE_CXX_COMPILER} 
		-DCMAKE_CXX_FLAGS=${CXX_FLAGS}
		-DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE} 
		-DCMAKE_MODULE_PATH=${CMAKE_MODULE_PATH} 
		-DCMAKE_EXTERNALS=${CMAKE_EXTERNALS} 
		-DCMAKE_INSTALL_PREFIX=${FISPRO_INSTALL_DIR} 
		${FISPRO_SOURCE_DIR}/base 
		PARENT_SCOPE)
endfunction()

function(fispro_svn_dir RESULT) 
    is_snapshot_version(${FISPRO_VERSION} FISPRO_SNAPSHOT_VERSION)
	if(FISPRO_SNAPSHOT_VERSION)
        set(${RESULT} "https://subversion.renater.fr/anonscm/svn/fispro/trunk/FisPro/cpp/base" PARENT_SCOPE)
	else()
        set(${RESULT} "https://subversion.renater.fr/anonscm/svn/fispro/tags/release-fispro-${FISPRO_VERSION}/cpp/base" PARENT_SCOPE)
	endif()
endfunction()

set(FISPRO_EXTERNAL fispro_external)

function(external_fispro)
	fispro_cmake_args(FISPRO_CMAKE_ARGS)
	fispro_svn_dir(FISPRO_SVN_DIR)
	message(STATUS "FISPro SVN Directory: ${FISPRO_SVN_DIR}")

	ExternalProject_Add(${FISPRO_EXTERNAL}
		TMP_DIR ${FISPRO_TMP_DIR}
		STAMP_DIR ${FISPRO_STAMP_DIR}
		SOURCE_DIR ${FISPRO_SOURCE_DIR}/base
		BINARY_DIR ${FISPRO_BINARY_DIR}
		INSTALL_DIR ${FISPRO_INSTALL_DIR}
        LOG_INSTALL 1
		
		SVN_REPOSITORY ${FISPRO_SVN_DIR}
        CMAKE_ARGS ${FISPRO_CMAKE_ARGS}
        BUILD_COMMAND $(MAKE)
        INSTALL_COMMAND $(MAKE) install)
endfunction() 

function(install_fispro)
	message(STATUS "Install FISPro Library ${FISPRO_VERSION}")
    external_fispro()
endfunction() 

macro(require_fispro)
	find_package(FISPro REQUIRED)
	include_directories(${FISPro_INCLUDE_DIR})
	link_directories(${FISPro_LIBRARY_DIR})
endmacro()

macro(find_fispro)
	find_package(FISPro)
	if(NOT FISPro_FOUND)
		install_fispro()
	  	include_directories(${FISPRO_INSTALL_DIR}/include)
		link_directories(${FISPRO_INSTALL_DIR}/lib)
		if(MINGW)
		    set(FISPro_RUNTIME_DIR ${FISPRO_INSTALL_DIR}/bin)
        else()		    
		    set(FISPro_RUNTIME_DIR ${FISPRO_INSTALL_DIR}/lib)
        endif()		    
	else()
	  	include_directories(${FISPro_INCLUDE_DIR})
		link_directories(${FISPro_LIBRARY_DIR})
	endif()
endmacro()

remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_fispro
    COMMAND ${REMOVE_DIRECTORY_COMMAND} ${FISPRO_SOURCE_DIR}
    COMMAND ${REMOVE_DIRECTORY_COMMAND} ${FISPRO_BINARY_DIR}
    COMMAND ${REMOVE_DIRECTORY_COMMAND} ${FISPRO_INSTALL_DIR}
    COMMAND ${REMOVE_DIRECTORY_COMMAND} ${FISPRO_TMP_DIR}
    COMMAND ${REMOVE_DIRECTORY_COMMAND} ${FISPRO_STAMP_DIR})	

endif()
