
include(FindPackageHandleStandardArgs)

find_path(CGAL_INCLUDE_DIR CGAL/auto_link/CGAL.h 
	HINTS ${CGAL_ROOT}
	PATH_SUFFIXES include
	NO_DEFAULT_PATH)

macro(find_cgal_library LIBRARY)
    find_library(CGAL_${LIBRARY}_LIBRARY ${LIBRARY}  
	    HINTS ${CGAL_ROOT}
	    PATH_SUFFIXES lib
	    NO_DEFAULT_PATH)
	    
	if(CGAL_${LIBRARY}_LIBRARY)
	    set(CGAL_${LIBRARY}_FOUND ON)
		get_filename_component(CGAL_LIBRARY_DIR ${CGAL_${LIBRARY}_LIBRARY} PATH)
	else()
	    set(CGAL_${LIBRARY}_FOUND OFF)
    endif()	    
endmacro() 
	
macro(find_cgal_libraries)
    set(CGAL_LIBRARIES_FOUND ON)
    foreach(LIBRARY ${ARGN})
        find_cgal_library(${LIBRARY})
        set(CGAL_LIBRARIES ${CGAL_LIBRARIES} ${CGAL_${LIBRARY}_LIBRARY})
        if(NOT CGAL_${LIBRARY}_FOUND)
           set(CGAL_LIBRARIES_FOUND OFF)
        endif()            
    endforeach()
endmacro()
	
macro(find_cgal_runtime RUNTIME)
    find_library(CGAL_${RUNTIME}_RUNTIME ${RUNTIME}  
	    HINTS ${CGAL_ROOT}
	    PATH_SUFFIXES bin
	    NO_DEFAULT_PATH)
	    
	if(CGAL_${RUNTIME}_RUNTIME)
	    set(CGAL_${RUNTIME}_FOUND ON)
		get_filename_component(CGAL_RUNTIME_DIR ${CGAL_${RUNTIME}_RUNTIME} PATH)
	else()
	    set(CGAL_${RUNTIME}_FOUND OFF)
    endif()	    
endmacro() 
	
macro(find_cgal_runtimes)
    set(CGAL_RUNTIMES_FOUND ON)
    foreach(RUNTIME ${ARGN})
        find_cgal_runtime(${RUNTIME})
        set(CGAL_RUNTIMES ${CGAL_RUNTIMES} ${CGAL_${RUNTIME}_RUNTIME})
        if(NOT CGAL_${RUNTIME}_FOUND)
           set(CGAL_RUNTIMES_FOUND OFF)
        endif()            
    endforeach()
endmacro()

if(CGAL_VERSION VERSION_LESS 5) # CGAL version 5.x is header only by default
    set(CGAL_LIBRARIES_NAME CGAL CGAL_Core)	
    
    find_cgal_libraries(${CGAL_LIBRARIES_NAME})
    if(MINGW)
        find_cgal_runtimes(${CGAL_LIBRARIES_NAME})
        
    	find_package_handle_standard_args(CGAL DEFAULT_MSG CGAL_INCLUDE_DIR CGAL_LIBRARIES_FOUND CGAL_RUNTIMES_FOUND)
    else()
    	find_package_handle_standard_args(CGAL DEFAULT_MSG CGAL_INCLUDE_DIR CGAL_LIBRARIES_FOUND)
    	set(CGAL_RUNTIME_DIR ${CGAL_LIBRARY_DIR})
    endif()	
else()
    find_package_handle_standard_args(CGAL DEFAULT_MSG CGAL_INCLUDE_DIR)
endif()
