if(NOT GDAL_CONFIG_INCLUDED)
set(GDAL_CONFIG_INCLUDED 1)

# keep compatibility with versions 1.x and 2.x
#set(GDAL_VERSION 1.11.5)
set(GDAL_VERSION 2.2.1)

include(Common)
include(GEOS)

external_project_directories(gdal ${GDAL_VERSION} GDAL_TMP_DIR GDAL_STAMP_DIR GDAL_DOWNLOAD_DIR GDAL_SOURCE_DIR GDAL_BINARY_DIR GDAL_INSTALL_DIR)
append_target_architecture_directory(${GDAL_TMP_DIR} GDAL_TMP_DIR)
append_target_architecture_directory(${GDAL_STAMP_DIR} GDAL_STAMP_DIR)
append_target_architecture_directory(${GDAL_SOURCE_DIR} GDAL_SOURCE_DIR)
append_target_architecture_directory(${GDAL_INSTALL_DIR} GDAL_INSTALL_DIR)
if(MINGW)
    set(GDAL_SHARE_DIR ${GDAL_INSTALL_DIR}/share)
endif()    

if(NOT GDAL_ROOT)
	set(GDAL_ROOT ${GDAL_INSTALL_DIR})
endif()

function(gdal_configure_command RESULT)
    set(CFLAGS "${CXX_ARCH}")
    set(CPPFLAGS "${CXX_ARCH}")
    set(CXXFLAGS "${CXX_ARCH} -I${GEOS_INCLUDE_DIR}")
    set(LDFLAGS "${LDFLAGS} -L${GEOS_LIBRARY_DIR}")
    if(GDAL_VERSION VERSION_EQUAL 1.11.5)
        set(CONFIGURE_COMMAND ${GDAL_SOURCE_DIR}/configure  
            --prefix=${GDAL_INSTALL_DIR} 
            --with-threads  
            --with-ogr  
            --with-geos=${GEOS_RUNTIME_DIR}/geos-config   
            --without-libtool 
            --without-libz 
            --with-libtiff=internal 
            --with-geotiff=internal 
            --without-gif 
            --without-pg 
            --without-grass 
            --without-libgrass 
            --without-cfitsio 
            --without-pcraster 
            --without-netcdf 
            --without-png 
            --without-jpeg 
            --without-ogdi 
            --without-fme 
            --without-hdf4 
            --without-hdf5 
            --without-jasper 
            --without-ecw 
            --without-kakadu 
            --without-mrsid 
            --without-jp2mrsid 
            --without-bsb 
            --without-grib 
            --without-mysql 
            --without-ingres 
            --without-xerces 
            --without-expat 
            --without-odbc 
            --without-curl 
            --without-sqlite3 
            --without-dwgdirect 
            --without-idb 
            --without-sde 
            --without-perl 
            --without-php 
            --without-ruby 
            --without-python 
            --with-hide-internal-symbols
            CFLAGS=${CFLAGS} 
            CPPFLAGS=${CPPFLAGS} 
            CXXFLAGS=${CXXFLAGS}
            LDFLAGS=${LDFLAGS})
    elseif(GDAL_VERSION VERSION_EQUAL 2.2.1)
        set(CONFIGURE_COMMAND ${GDAL_SOURCE_DIR}/configure  
            --prefix=${GDAL_INSTALL_DIR} 
            --with-threads  
            --with-geos=${GEOS_RUNTIME_DIR}/geos-config   
            --without-libtool 
            --with-libz=internal 
            --with-libtiff=internal 
            --with-geotiff=internal 
            --with-jpeg=internal 
            --with-png=internal 
            --without-gif 
            --without-pg 
            --without-grass 
            --without-libgrass 
            --without-cfitsio 
            --without-pcraster 
            --without-netcdf 
            --without-jpeg12
            --without-ogdi 
            --without-fme 
            --without-hdf4 
            --without-hdf5 
            --without-jasper 
            --without-ecw 
            --without-kakadu 
            --without-mrsid 
            --without-mrf
            --without-jp2mrsid 
            --without-bsb 
            --without-grib 
            --without-mysql 
            --without-ingres 
            --without-xerces 
            --without-expat 
            --without-odbc 
            --without-curl 
            --without-sqlite3 
            --without-idb 
            --without-sde 
            --without-perl 
            --without-php 
            --without-python 
            --with-hide-internal-symbols
            CFLAGS=${CFLAGS} 
            CPPFLAGS=${CPPFLAGS} 
            CXXFLAGS=${CXXFLAGS}
            LDFLAGS=${LDFLAGS})
    endif()	    
    set(${RESULT} ${CONFIGURE_COMMAND} PARENT_SCOPE)
endfunction()

function(gdal_patch_command RESULT)
    set(${RESULT} patch -p0 -i ${CMAKE_MODULE_PATH}/gdal-${GDAL_VERSION}.patch PARENT_SCOPE)
endfunction()

set(GDAL_EXTERNAL gdal_external)

function(install_gdal)
	message(STATUS "Install GDAL Library ${GDAL_VERSION}")
	
	set(GDAL_URL "http://download.osgeo.org/gdal/${GDAL_VERSION}/gdal-${GDAL_VERSION}.tar.gz")
    if(GDAL_VERSION VERSION_EQUAL 1.11.5)
    	set(GDAL_MD5 879fa140f093a2125f71e38502bdf714) 
    elseif(GDAL_VERSION VERSION_EQUAL 2.2.1)
	    set(GDAL_MD5 785acf2b0cbf9d56d37c9044d0ee2505) 
    endif()	    
	
	gdal_configure_command(GDAL_CONFIGURE_COMMAND)
	gdal_patch_command(GDAL_PATCH_COMMAND)

	ExternalProject_Add(${GDAL_EXTERNAL}
		TMP_DIR ${GDAL_TMP_DIR}
		STAMP_DIR ${GDAL_STAMP_DIR}
		DOWNLOAD_DIR ${GDAL_DOWNLOAD_DIR}
		SOURCE_DIR ${GDAL_SOURCE_DIR}
		INSTALL_DIR ${GDAL_INSTALL_DIR}
		
		URL ${GDAL_URL}
		URL_MD5 ${GDAL_MD5}
    	BUILD_IN_SOURCE 1

        PATCH_COMMAND ${GDAL_PATCH_COMMAND}
		CONFIGURE_COMMAND ${GDAL_CONFIGURE_COMMAND}
		BUILD_COMMAND $(MAKE) 
		INSTALL_COMMAND $(MAKE) install)	
	add_dependencies(${GDAL_EXTERNAL} ${GEOS_EXTERNAL})
endfunction()

macro(require_gdal)
    require_geos()
    find_package(GDAL REQUIRED)
  	include_directories(${GDAL_INCLUDE_DIR})
	link_directories(${GDAL_LIBRARY_DIR})
endmacro()

macro(find_gdal)
    find_geos()
    find_package(GDAL)
    if(NOT GDAL_FOUND)
    	install_gdal()
    	set(GDAL_INCLUDE_DIR ${GDAL_INSTALL_DIR}/include)
   	    set(GDAL_LIBRARY_DIR ${GDAL_INSTALL_DIR}/lib)
   	    set(GDAL_RUNTIME_DIR ${GDAL_INSTALL_DIR}/lib)
    endif()
  	include_directories(${GDAL_INCLUDE_DIR})
	link_directories(${GDAL_LIBRARY_DIR})
endmacro()

endif()
