set(GTEST_VERSION 1.8.0)

include(Common)

external_project_directories(gtest ${GTEST_VERSION} GTEST_TMP_DIR GTEST_STAMP_DIR GTEST_DOWNLOAD_DIR GTEST_SOURCE_DIR GTEST_BINARY_DIR GTEST_INSTALL_DIR)
append_target_architecture_directory(${GTEST_TMP_DIR} GTEST_TMP_DIR)
append_target_architecture_directory(${GTEST_STAMP_DIR} GTEST_STAMP_DIR)
append_target_architecture_directory(${GTEST_BINARY_DIR} GTEST_BINARY_DIR)
append_target_architecture_directory(${GTEST_INSTALL_DIR} GTEST_INSTALL_DIR)

if(NOT GTEST_ROOT)
	set(GTEST_ROOT ${GTEST_INSTALL_DIR})
endif()

function(gtest_cmake_args RESULT)
if(MINGW)
 	set(${RESULT} 
		-DCMAKE_INSTALL_PREFIX=${GTEST_INSTALL_DIR} 
		-Dgtest_disable_pthreads=ON
		${GTEST_SOURCE_DIR} 
		PARENT_SCOPE)
else()
 	set(${RESULT} 
		-DCMAKE_INSTALL_PREFIX=${GTEST_INSTALL_DIR} 
		${GTEST_SOURCE_DIR} 
		PARENT_SCOPE)
endif()
endfunction()

set(GTEST_EXTERNAL gtest_external)

function(install_gtest)
	message(STATUS "Install GoogleTest Library ${GTEST_VERSION}")
	
	set(GTEST_URL https://github.com/google/googletest/archive/release-1.8.0.zip)
	
	gtest_cmake_args(GTEST_CMAKE_ARGS)
	
	ExternalProject_Add(${GTEST_EXTERNAL}
		TMP_DIR ${GTEST_TMP_DIR}
		STAMP_DIR ${GTEST_STAMP_DIR}
		DOWNLOAD_DIR ${GTEST_DOWNLOAD_DIR}
		SOURCE_DIR ${GTEST_SOURCE_DIR}
		BINARY_DIR ${GTEST_BINARY_DIR}
		INSTALL_DIR ${GTEST_INSTALL_DIR}
		
		URL ${GTEST_URL}
		CMAKE_ARGS ${GTEST_CMAKE_ARGS}
		BUILD_COMMAND $(MAKE)
		INSTALL_COMMAND $(MAKE) install)
endfunction()

macro(require_gtest)
    if(UNIX)
        enable_language(C) # find_package Threads needs C language
        find_package(Threads REQUIRED)
    endif()    
	find_package(GTest REQUIRED)
  	include_directories(${GTest_INCLUDE_DIR})
	link_directories(${GTest_LIBRARY_DIR})
endmacro()
	
macro(find_gtest)
    if(UNIX)
        enable_language(C) # find_package Threads needs C language
        find_package(Threads REQUIRED)
    endif()    
	find_package(GTest)
	if(NOT GTest_FOUND)
		install_gtest()
	  	include_directories(${GTEST_INSTALL_DIR}/include)
		link_directories(${GTEST_INSTALL_DIR}/lib)
	else()
	  	include_directories(${GTest_INCLUDE_DIR})
		link_directories(${GTest_LIBRARY_DIR})
	endif()
endmacro()

remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_gtest
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GTEST_SOURCE_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GTEST_BINARY_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GTEST_INSTALL_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GTEST_TMP_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GTEST_STAMP_DIR})	
