if(NOT GeoFIS_CONFIG_INCLUDED)
set(GeoFIS_CONFIG_INCLUDED 1)

include(GeoFISVersion)
include(Common)
include(Util)
include(CGAL)
include(GDAL)
include(GeoFISBoost)

is_snapshot_version(${GEOFIS_VERSION} GEOFIS_SNAPSHOT_VERSION)
	
external_project_directories(geofis ${GEOFIS_VERSION} GEOFIS_TMP_DIR GEOFIS_STAMP_DIR GEOFIS_DOWNLOAD_DIR GEOFIS_SOURCE_DIR GEOFIS_BINARY_DIR GEOFIS_INSTALL_DIR)
append_target_architecture_directory(${GEOFIS_TMP_DIR} GEOFIS_TMP_DIR)
append_target_architecture_directory(${GEOFIS_STAMP_DIR} GEOFIS_STAMP_DIR)
append_target_architecture_directory(${GEOFIS_BINARY_DIR} GEOFIS_BINARY_DIR)
append_target_architecture_directory(${GEOFIS_INSTALL_DIR} GEOFIS_INSTALL_DIR)

if(NOT GEOFIS_ROOT)
	set(GEOFIS_ROOT ${GEOFIS_INSTALL_DIR})
endif()

function(geofis_cmake_args GEOFIS_SOURCE_DIR RESULT)
    set(CXX_FLAGS "${CXX_ARCH} -Wall")
	set(${RESULT} 
		-DCMAKE_CXX_COMPILER=${CMAKE_CXX_COMPILER} 
		-DCMAKE_CXX_FLAGS=${CXX_FLAGS}
		-DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE} 
		-DCMAKE_MODULE_PATH=${CMAKE_MODULE_PATH} 
		-DCMAKE_EXTERNALS=${CMAKE_EXTERNALS} 
		-DCMAKE_INSTALL_PREFIX=${GEOFIS_INSTALL_DIR} 
		-DUTIL_NATIVE_DIR=${UTIL_NATIVE_DIR}
		-DGEOFIS_NATIVE_DIR=${GEOFIS_NATIVE_DIR}
 		--no-warn-unused-cli
		${GEOFIS_SOURCE_DIR} 
		PARENT_SCOPE)
endfunction()

function(require_geofis_native_dir RESULT) 
    if(NOT DEFINED GEOFIS_NATIVE_DIR)
        set(GEOFIS_NATIVE_DIR "../../../geofis-native/src/main") # default GEOFIS_NATIVE_DIR
    endif()
    get_filename_component(GEOFIS_NATIVE_DIR ${GEOFIS_NATIVE_DIR} ABSOLUTE)
    if(NOT EXISTS ${GEOFIS_NATIVE_DIR})
        message(FATAL_ERROR "GEOFIS_NATIVE_DIR '${GEOFIS_NATIVE_DIR}' not exists")
    endif()        
    set(${RESULT} ${GEOFIS_NATIVE_DIR} PARENT_SCOPE)
endfunction()

set(GEOFIS_EXTERNAL geofis_external)

function(external_geofis)
	
	if(GEOFIS_SNAPSHOT_VERSION)
 	    require_geofis_native_dir(GEOFIS_NATIVE_DIR)
	    geofis_cmake_args(${GEOFIS_NATIVE_DIR}, GEOFIS_CMAKE_ARGS)
 
    	ExternalProject_Add(${GEOFIS_EXTERNAL}
    		TMP_DIR ${GEOFIS_TMP_DIR}
    		STAMP_DIR ${GEOFIS_STAMP_DIR}
    		SOURCE_DIR ${GEOFIS_NATIVE_DIR}
    		BINARY_DIR ${GEOFIS_BINARY_DIR}
    		INSTALL_DIR ${GEOFIS_INSTALL_DIR}
            LOG_INSTALL 1
            		
    		CMAKE_ARGS ${GEOFIS_CMAKE_ARGS}
            BUILD_COMMAND $(MAKE) 
            INSTALL_COMMAND $(MAKE) install)
    else()
 	    geofis_cmake_args(${GEOFIS_SOURCE_DIR}, GEOFIS_CMAKE_ARGS)
 
    	ExternalProject_Add(${GEOFIS_EXTERNAL}
    		TMP_DIR ${GEOFIS_TMP_DIR}
    		STAMP_DIR ${GEOFIS_STAMP_DIR}
    		SOURCE_DIR ${GEOFIS_SOURCE_DIR}
    		BINARY_DIR ${GEOFIS_BINARY_DIR}
    		INSTALL_DIR ${GEOFIS_INSTALL_DIR}
            LOG_INSTALL 1
            		
    		SVN_REPOSITORY "https://subversion.renater.fr/anonscm/svn/geofis/tags/release-geofis-${GEOFIS_VERSION}/geofis-native/src/main"
    		CMAKE_ARGS ${GEOFIS_CMAKE_ARGS}
            BUILD_COMMAND $(MAKE) 
            INSTALL_COMMAND $(MAKE) install)
    endif()            
        
    add_dependencies(${GEOFIS_EXTERNAL} ${GEOFIS_BOOST_EXTERNAL} ${UTIL_EXTERNAL} ${CGAL_EXTERNAL} ${GDAL_EXTERNAL} ${GEOS_EXTERNAL})    
endfunction()

function(install_geofis)
	message(STATUS "Install GeoFIS Library ${GEOFIS_VERSION}")
    external_geofis()
endfunction()

function(update_geofis)
	message(STATUS "Update GeoFIS Library ${GEOFIS_VERSION}")
    external_geofis()
endfunction()

macro(require_geofis)
    require_geofis_boost()
    require_cgal()
    if(BUILD_ZONING)
        require_gdal()
    endif()
	find_package(GeoFIS REQUIRED)
  	include_directories(${GeoFIS_INCLUDE_DIR})
	link_directories(${GeoFIS_LIBRARY_DIR})
endmacro()

macro(find_geofis)
    find_geofis_boost()
    find_cgal()
    if(MINGW)
        find_gdal()
    else()
        require_gdal()
    endif()        
	find_package(GeoFIS)
	if(NOT GeoFIS_FOUND)
		install_geofis()
	  	include_directories(${GEOFIS_INSTALL_DIR}/include)
		link_directories(${GEOFIS_INSTALL_DIR}/lib)
		if(MINGW)
		    set(GeoFIS_RUNTIME_DIR ${GEOFIS_INSTALL_DIR}/bin)
		else()    
		    set(GeoFIS_RUNTIME_DIR ${GEOFIS_INSTALL_DIR}/lib)
        endif()		    
	else()
        if(GEOFIS_SNAPSHOT_VERSION)
	        update_geofis()
        endif()
	  	include_directories(${GeoFIS_INCLUDE_DIR})
		link_directories(${GeoFIS_LIBRARY_DIR})
	endif()
endmacro()

remove_directory_command(REMOVE_DIRECTORY_COMMAND)

if(GEOFIS_SNAPSHOT_VERSION)
    add_custom_target(clean_geofis
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_BINARY_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_INSTALL_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_TMP_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_STAMP_DIR})	
else()
    add_custom_target(clean_geofis
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_SOURCE_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_BINARY_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_INSTALL_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_TMP_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${GEOFIS_STAMP_DIR})	
endif()

endif()
   