if(NOT GeoFISBoost_CONFIG_INCLUDED)
set(GeoFISBoost_CONFIG_INCLUDED 1)

include(Common)
include(Boost)

set(GEOFIS_BOOST_LIBRARIES program_options)

with_boost_prefix_libraries(GEOFIS_BOOST_PREFIX_LIBRARIES ${GEOFIS_BOOST_LIBRARIES})

append_target_architecture_directory(${BOOST_TMP_DIR} GEOFIS_BOOST_TMP_DIR)
append_target_architecture_directory(${BOOST_STAMP_DIR} GEOFIS_BOOST_STAMP_DIR)

set(GEOFIS_BOOST_EXTERNAL geofis_boost_external)

function(install_geofis_boost_libraries)
	boost_address_model(ADDRESS_MODEL)
	boost_build_command(BOOST_BUILD_COMMAND)
    with_boost_libraries(GEOFIS_WITH_BOOST_LIBRARIES ${GEOFIS_BOOST_LIBRARIES})
    set(CXXFLAGS "${CXX_ARCH}")
    if(BOOST_VERSION VERSION_LESS 1.60)
        set(CXXFLAGS "${CXXFLAGS} -Wno-parentheses -Wno-deprecated-declarations -Wno-placement-new")
    elseif(BOOST_VERSION VERSION_LESS 1.62)    
         set(CXXFLAGS "${CXXFLAGS} -Wno-parentheses -Wno-placement-new")
    elseif(BOOST_VERSION VERSION_LESS 1.68)    
         set(CXXFLAGS "${CXXFLAGS} -Wno-parentheses")
    endif()
    message(STATUS "GEOFIS_BOOST_CXXFLAGS: ${CXXFLAGS}")
    
	ExternalProject_Add(${GEOFIS_BOOST_EXTERNAL}
		TMP_DIR ${GEOFIS_BOOST_TMP_DIR}
		STAMP_DIR ${GEOFIS_BOOST_STAMP_DIR}
		SOURCE_DIR ${BOOST_SOURCE_DIR}
		INSTALL_DIR ${BOOST_INSTALL_DIR}
		
		DOWNLOAD_COMMAND ""
		BUILD_IN_SOURCE 1
		CONFIGURE_COMMAND ""
		BUILD_COMMAND ${BOOST_SOURCE_DIR}/${BOOST_BUILD_COMMAND} --prefix=${BOOST_INSTALL_DIR} --libdir=${BOOST_INSTALL_DIR}/lib --layout=system ${GEOFIS_WITH_BOOST_LIBRARIES} address-model=${ADDRESS_MODEL} cxxflags=${CXXFLAGS} link=shared threading=single variant=release toolset=gcc -j${PROCESSOR_COUNT} install
	    INSTALL_COMMAND "")

	ExternalProject_Add_Step(${GEOFIS_BOOST_EXTERNAL} force-build
        DEPENDERS build
        ALWAYS 1)

    add_dependencies(${GEOFIS_BOOST_EXTERNAL} ${BOOST_EXTERNAL})
endfunction()
	
macro(require_geofis_boost)
    require_boost_components(${GEOFIS_BOOST_LIBRARIES})
endmacro()
		
macro(find_geofis_boost)
    find_boost_components(${GEOFIS_BOOST_LIBRARIES})
    if(NOT Boost_FOUND)
    	install_geofis_boost_libraries()
		link_directories(${BOOST_INSTALL_DIR}/lib)
    else()
		link_directories(${Boost_LIBRARY_DIRS})
    endif()
endmacro()

endif()		
