if(NOT Libiconv_CONFIG_INCLUDED)
set(Libiconv_CONFIG_INCLUDED 1)

set(LIBICONV_VERSION 1.14)

include(Common)

external_project_directories(libiconv ${LIBICONV_VERSION} LIBICONV_TMP_DIR LIBICONV_STAMP_DIR LIBICONV_DOWNLOAD_DIR LIBICONV_SOURCE_DIR LIBICONV_BINARY_DIR LIBICONV_INSTALL_DIR)
append_target_architecture_directory(${LIBICONV_TMP_DIR} LIBICONV_TMP_DIR)
append_target_architecture_directory(${LIBICONV_STAMP_DIR} LIBICONV_STAMP_DIR)
append_target_architecture_directory(${LIBICONV_BINARY_DIR} LIBICONV_BINARY_DIR)
append_target_architecture_directory(${LIBICONV_INSTALL_DIR} LIBICONV_INSTALL_DIR)

if(NOT LIBICONV_ROOT)
	set(LIBICONV_ROOT ${LIBICONV_INSTALL_DIR})
endif()

function(libiconv_configure_command RESULT)
    msys_path(${LIBICONV_SOURCE_DIR} LIBICONV_MSYS_SOURCE_DIR)
    set(CPPFLAGS "${CXX_ARCH} -Wno-attributes")
    if((CMAKE_CXX_COMPILER_VERSION VERSION_GREATER 7) OR (CMAKE_CXX_COMPILER_VERSION VERSION_EQUAL 7))
        # libiconv bug #47953, libiconv aliases2_lookup undefined, see https://savannah.gnu.org/bugs/?47953
        set(CPPFLAGS "${CPPFLAGS} -fgnu89-inline")
    endif()
    if(TARGET_64_BITS)
    	set(CPPFLAGS "${CPPFLAGS} -Wno-pointer-to-int-cast")
    endif()	
    set(CONFIGURE_COMMAND ${LIBICONV_SOURCE_DIR}/configure --prefix=${LIBICONV_INSTALL_DIR} --srcdir=${LIBICONV_MSYS_SOURCE_DIR})
    set(${RESULT} ${CONFIGURE_COMMAND} CPPFLAGS=${CPPFLAGS} PARENT_SCOPE)
endfunction()

set(LIBICONV_EXTERNAL libiconv_external)

function(install_libiconv)
    if(NOT LIBICONV_INSTALL)
    	message(STATUS "Install Libiconv Library ${LIBICONV_VERSION}")
    	
    	set(LIBICONV_URL "http://ftp.gnu.org/pub/gnu/libiconv/libiconv-${LIBICONV_VERSION}.tar.gz")
    	set(LIBICONV_MD5 e34509b1623cec449dfeb73d7ce9c6c6) 
    	
    	libiconv_configure_command(LIBICONV_CONFIGURE_COMMAND)
    	
    	ExternalProject_Add(${LIBICONV_EXTERNAL}
    		TMP_DIR ${LIBICONV_TMP_DIR}
    		STAMP_DIR ${LIBICONV_STAMP_DIR}
    		DOWNLOAD_DIR ${LIBICONV_DOWNLOAD_DIR}
    		SOURCE_DIR ${LIBICONV_SOURCE_DIR}
    		BINARY_DIR ${LIBICONV_BINARY_DIR}
    		INSTALL_DIR ${LIBICONV_INSTALL_DIR}
    		
    		URL ${LIBICONV_URL}
    		URL_MD5 ${LIBICONV_MD5}
    		# see http://lists.gnu.org/archive/html/bug-gnu-libiconv/2014-05/msg00001.html for libiconv patching
    		PATCH_COMMAND patch -p1 -tuN -i ${CMAKE_MODULE_PATH}/libiconv-1.14_srclib_stdio.in.h-remove-gets-declarations.patch
    		CONFIGURE_COMMAND ${LIBICONV_CONFIGURE_COMMAND}
    		BUILD_COMMAND $(MAKE)
    		INSTALL_COMMAND $(MAKE) install)
    		
        set(LIBICONV_INSTALL 1 PARENT_SCOPE)	    
    endif()
endfunction()

macro(require_libiconv)
	find_package(Libiconv REQUIRED)
	include_directories(${Libiconv_INCLUDE_DIR})
	link_directories(${Libiconv_LIBRARY_DIR})
endmacro()

macro(find_libiconv)
	find_package(Libiconv)
	if(NOT Libiconv_FOUND)
		install_libiconv()
	  	include_directories(${LIBICONV_INSTALL_DIR}/include)
		link_directories(${LIBICONV_INSTALL_DIR}/lib)
	    if(MINGW)
		    set(Libiconv_RUNTIME_DIR ${LIBICONV_INSTALL_DIR}/bin)
		 else()
 		    set(Libiconv_RUNTIME_DIR ${LIBICONV_INSTALL_DIR}/lib)
		 endif()
	else()
	  	include_directories(${Libiconv_INCLUDE_DIR})
		link_directories(${Libiconv_LIBRARY_DIR})
	endif()
endmacro()

remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_libiconv
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${LIBICONV_SOURCE_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${LIBICONV_BINARY_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${LIBICONV_INSTALL_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${LIBICONV_TMP_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${LIBICONV_STAMP_DIR})	

endif()