if(NOT MPFR_CONFIG_INCLUDED)
set(MPFR_CONFIG_INCLUDED 1)

set(MPFR_VERSION 4.1.0)

include(GMP)

external_project_directories(mpfr ${MPFR_VERSION} MPFR_TMP_DIR MPFR_STAMP_DIR MPFR_DOWNLOAD_DIR MPFR_SOURCE_DIR MPFR_BINARY_DIR MPFR_INSTALL_DIR)
append_target_architecture_directory(${MPFR_TMP_DIR} MPFR_TMP_DIR)
append_target_architecture_directory(${MPFR_STAMP_DIR} MPFR_STAMP_DIR)
append_target_architecture_directory(${MPFR_BINARY_DIR} MPFR_BINARY_DIR)
append_target_architecture_directory(${MPFR_INSTALL_DIR} MPFR_INSTALL_DIR)

if(NOT MPFR_ROOT)
	set(MPFR_ROOT ${MPFR_INSTALL_DIR})
endif()

function(mpfr_configure_command RESULT)
    msys_path(${MPFR_SOURCE_DIR} MPFR_MSYS_SOURCE_DIR)
    set(CFLAGS "${CXX_ARCH}")
    if(TARGET_64_BITS)
        set(CFLAGS "${CFLAGS} -Wno-pointer-to-int-cast")
#        set(BUILD "x86_64-w64-mingw32")
#        set(HOST "x86_64-w64-mingw32")
#        set(LDFLAGS "-no-fast-install")
    endif()
 #    set(CONFIGURE_COMMAND ${MPFR_SOURCE_DIR}/configure --prefix=${MPFR_INSTALL_DIR} --srcdir=${MPFR_MSYS_SOURCE_DIR} --with-gmp=${GMP_ROOT} --build=${BUILD} --host=${HOST})
    set(CONFIGURE_COMMAND ${MPFR_SOURCE_DIR}/configure --prefix=${MPFR_INSTALL_DIR} --srcdir=${MPFR_MSYS_SOURCE_DIR} --with-gmp=${GMP_ROOT} --build=${TARGET_ARCHITECTURE} --enable-static --disable-shared)
    set(${RESULT} ${CONFIGURE_COMMAND} CFLAGS=${CFLAGS} PARENT_SCOPE)
endfunction()

set(MPFR_EXTERNAL mpfr_external)

function(install_mpfr)
	message(STATUS "Install MPFR Library ${MPFR_VERSION}")
	
	set(MPFR_URL "ftp://ftp.gnu.org/gnu/mpfr/mpfr-${MPFR_VERSION}.zip")
	if(MPFR_VERSION VERSION_EQUAL 3.1.2)
		set(MPFR_MD5 a25a48ed1b776f0cdc480338f1034617)
	endif()
	
	mpfr_configure_command(MPFR_CONFIGURE_COMMAND)
	message(STATUS "MPFR_CONFIGURE_COMMAND: ${MPFR_CONFIGURE_COMMAND}")
	
	ExternalProject_Add(${MPFR_EXTERNAL}
		TMP_DIR ${MPFR_TMP_DIR}
		STAMP_DIR ${MPFR_STAMP_DIR}
		DOWNLOAD_DIR ${MPFR_DOWNLOAD_DIR}
		SOURCE_DIR ${MPFR_SOURCE_DIR}
		BINARY_DIR ${MPFR_BINARY_DIR}
		INSTALL_DIR ${MPFR_INSTALL_DIR}
		
		URL ${MPFR_URL}
		URL_MD5 ${MPFR_MD5}
		CONFIGURE_COMMAND ${MPFR_CONFIGURE_COMMAND}
		BUILD_COMMAND $(MAKE)
		INSTALL_COMMAND $(MAKE) install
		TEST_COMMAND $(MAKE) check)
		
	add_dependencies(${MPFR_EXTERNAL} ${GMP_EXTERNAL})
endfunction()

macro(require_mpfr)
    require_gmp()
    find_package(MPFR REQUIRED)
	include_directories(${MPFR_INCLUDE_DIR})
endmacro()

macro(find_mpfr)
    find_gmp()
    find_package(MPFR)
    if(NOT MPFR_FOUND)
    	install_mpfr()
    	set(MPFR_INCLUDE_DIR ${MPFR_INSTALL_DIR}/include)
    	set(MPFR_LIBRARY ${MPFR_INSTALL_DIR}/lib/libmpfr.a)
    endif()
	include_directories(${MPFR_INCLUDE_DIR})
endmacro()
	
remove_directory_command(REMOVE_DIRECTORY_COMMAND)

add_custom_target(clean_mpfr
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${MPFR_SOURCE_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${MPFR_BINARY_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${MPFR_INSTALL_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${MPFR_TMP_DIR}
   COMMAND ${REMOVE_DIRECTORY_COMMAND} ${MPFR_STAMP_DIR})	

endif()
   