if(NOT Util_CONFIG_INCLUDED)
set(Util_CONFIG_INCLUDED 1)

include(UtilVersion)
include(Common)
include(UtilBoost)
include(Libiconv)

is_snapshot_version(${UTIL_VERSION} UTIL_SNAPSHOT_VERSION)

external_project_directories(util ${UTIL_VERSION} UTIL_TMP_DIR UTIL_STAMP_DIR UTIL_DOWNLOAD_DIR UTIL_SOURCE_DIR UTIL_BINARY_DIR UTIL_INSTALL_DIR)
append_target_architecture_directory(${UTIL_TMP_DIR} UTIL_TMP_DIR)
append_target_architecture_directory(${UTIL_STAMP_DIR} UTIL_STAMP_DIR)
append_target_architecture_directory(${UTIL_BINARY_DIR} UTIL_BINARY_DIR)
append_target_architecture_directory(${UTIL_INSTALL_DIR} UTIL_INSTALL_DIR)

if(NOT UTIL_ROOT)
	set(UTIL_ROOT ${UTIL_INSTALL_DIR})
endif()

function(util_cmake_args UTIL_SOURCE_DIR RESULT)
    set(CXX_FLAGS "${CXX_ARCH} -Wall")
	set(${RESULT} 
		-DCMAKE_CXX_COMPILER=${CMAKE_CXX_COMPILER} 
		-DCMAKE_CXX_FLAGS=${CXX_FLAGS}
		-DCMAKE_BUILD_TYPE=${CMAKE_BUILD_TYPE} 
		-DCMAKE_MODULE_PATH=${CMAKE_MODULE_PATH} 
		-DCMAKE_EXTERNALS=${CMAKE_EXTERNALS} 
		-DCMAKE_INSTALL_PREFIX=${UTIL_INSTALL_DIR} 
 		-DUTIL_NATIVE_DIR=${UTIL_NATIVE_DIR}
 		--no-warn-unused-cli
		${UTIL_SOURCE_DIR} 
		PARENT_SCOPE)
endfunction()

set(UTIL_EXTERNAL util_external)

function(external_util)

	if(UTIL_SNAPSHOT_VERSION)
        get_filename_component(UTIL_NATIVE_DIR ${UTIL_NATIVE_DIR} ABSOLUTE)
        if(EXISTS ${UTIL_NATIVE_DIR})
            set(UTIL_SOURCE_DIR ${UTIL_NATIVE_DIR}) 
        else()
            set(UTIL_SVN_REPOSITORY "https://subversion.renater.fr/anonscm/svn/geofis/util/trunk/util-native/src/main")
        endif()                
    else()
        set(UTIL_SVN_REPOSITORY "https://subversion.renater.fr/anonscm/svn/geofis/util/tags/release-util-${UTIL_VERSION}/util-native/src/main")
    endif()            
 
    util_cmake_args(${UTIL_SOURCE_DIR}, UTIL_CMAKE_ARGS)
     
 	ExternalProject_Add(${UTIL_EXTERNAL}
		TMP_DIR ${UTIL_TMP_DIR}
		STAMP_DIR ${UTIL_STAMP_DIR}
		SOURCE_DIR ${UTIL_SOURCE_DIR}
		BINARY_DIR ${UTIL_BINARY_DIR}
		INSTALL_DIR ${UTIL_INSTALL_DIR}
        LOG_INSTALL 1
        		
		SVN_REPOSITORY ${UTIL_SVN_REPOSITORY}
		CMAKE_ARGS ${UTIL_CMAKE_ARGS}
        BUILD_COMMAND $(MAKE) 
        INSTALL_COMMAND $(MAKE) install)
        
    add_dependencies(${UTIL_EXTERNAL} ${LIBICONV_EXTERNAL} ${CPPUNIT_EXTERNAL} ${UTIL_BOOST_EXTERNAL})
endfunction()

function(install_util)
	message(STATUS "Install Util Library ${UTIL_VERSION}")
    external_util()
endfunction()

function(update_util)
	message(STATUS "Update Util Library ${UTIL_VERSION}")
    external_util()
 endfunction()

macro(require_util)
    require_util_boost()
    require_libiconv()
	find_package(Util REQUIRED)
	include_directories(${Util_INCLUDE_DIR})
    link_directories(${Util_LIBRARY_DIR})
endmacro()

macro(find_util)
    find_util_boost()
    find_libiconv()
	find_package(Util)
	if(NOT Util_FOUND)
		install_util()
	  	include_directories(${UTIL_INSTALL_DIR}/include)
		link_directories(${UTIL_INSTALL_DIR}/lib)
		if(MINGW)
		    set(Util_RUNTIME_DIR ${UTIL_INSTALL_DIR}/bin)
        else()
		    set(Util_RUNTIME_DIR ${UTIL_INSTALL_DIR}/lib)
        endif()        		    
	else()
        if(UTIL_SNAPSHOT_VERSION)
	        update_util()
        endif()
 	  	include_directories(${Util_INCLUDE_DIR})
		link_directories(${Util_LIBRARY_DIR})
	endif()
endmacro()

remove_directory_command(REMOVE_DIRECTORY_COMMAND)

if(UTIL_SNAPSHOT_VERSION)
    add_custom_target(clean_util
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_BINARY_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_INSTALL_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_TMP_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_STAMP_DIR})	
else()
    add_custom_target(clean_util
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_SOURCE_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_BINARY_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_INSTALL_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_TMP_DIR}
       COMMAND ${REMOVE_DIRECTORY_COMMAND} ${UTIL_STAMP_DIR})	
endif()
       
endif()		