/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/ogr/ogr_data_source.hpp>
#include <boost/filesystem/operations.hpp>
#include <util/assert.hpp>

using namespace boost::filesystem;

namespace geofis {

#if GDAL_VERSION_MAJOR >= 2

ogr_data_source::ogr_data_source(const boost::filesystem::path &path, GDALDriver *driver, int update, char **options) : data_source(NULL), update(update) {
	UTIL_REQUIRE(driver);
	if(exists(path)) {
		data_source = (GDALDataset *)GDALOpenEx(path.string().c_str(), update == TRUE ? GDAL_OF_VECTOR | GDAL_OF_UPDATE: GDAL_OF_VECTOR, NULL, NULL, NULL);
		OGR_ENSURE_THROW(data_source, ogr_open_data_source_error, driver, path);
	}
	else {
		data_source = driver->Create(path.string().c_str(), 0, 0, 0, GDT_Unknown, options);
		OGR_ENSURE_THROW(data_source, ogr_create_data_source_error, driver, path);
	}
}

#else

ogr_data_source::ogr_data_source(const boost::filesystem::path &path, OGRSFDriver *driver, int update, char **options) : data_source(NULL), update(update) {
	UTIL_REQUIRE(driver);
	if(exists(path)) {
		data_source = OGRSFDriverRegistrar::Open(path.string().c_str(), update, NULL);
		OGR_ENSURE_THROW(data_source, ogr_open_data_source_error, driver, path);
	}
	else {
		data_source = driver->CreateDataSource(path.string().c_str(), options);
		OGR_ENSURE_THROW(data_source, ogr_create_data_source_error, driver, path);
	}
}

#endif

void ogr_data_source::delete_layers() const {
	if(update) {
		int layer_count = data_source->GetLayerCount();
		for(int i = 0 ; i < layer_count ; i++) {
			std::string layer_name = data_source->GetLayer(0)->GetName();
			OGRErr ogr_err = data_source->DeleteLayer(0);
			OGR_ENSURE_THROW(ogr_err == OGRERR_NONE, ogr_delete_layer_error, data_source, layer_name, ogr_err);
		}
	}
}

void ogr_data_source::close() {
#if GDAL_VERSION_MAJOR >= 2
	GDALClose(data_source);
#else
	OGRDataSource::DestroyDataSource(data_source);
#endif
	data_source = NULL;
}

ogr_data_source::~ogr_data_source() {
	close();
}

#if GDAL_VERSION_MAJOR >= 2

ogr_data_source::operator GDALDataset*() const {
	return data_source;
}

GDALDataset* ogr_data_source::operator->() const {
	return data_source;
}

#else

ogr_data_source::operator OGRDataSource*() const {
	return data_source;
}

OGRDataSource* ogr_data_source::operator->() const {
	return data_source;
}

#endif

} // namespace geofis

