/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/ogr/ogr_exception.hpp>
#include <boost/format.hpp>

namespace geofis {

std::string get_ogr_error_string(OGRErr ogr_err) {
	switch(ogr_err) {
	case OGRERR_NONE:
		return "none";
	case OGRERR_NOT_ENOUGH_DATA:
		return "not enough data";
	case OGRERR_NOT_ENOUGH_MEMORY:
		return "not enough memory";
	case OGRERR_UNSUPPORTED_GEOMETRY_TYPE:
		return "unsupported geometry type";
	case OGRERR_UNSUPPORTED_OPERATION:
		return "unsupported operation";
	case OGRERR_CORRUPT_DATA:
		return "corrupt data";
	case OGRERR_FAILURE:
		return "failure";
	case OGRERR_UNSUPPORTED_SRS:
		return "unsupported srs";
	case OGRERR_INVALID_HANDLE:
		return "invalid handle";
	default:
		return boost::str(boost::format("unsupported error %1%") % ogr_err);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

std::string get_ogr_geometry_string(OGRwkbGeometryType ogr_geometry_type) {
	switch(ogr_geometry_type) {
	case wkbPoint:
		return "point";
	case wkbLineString:
		return "line";
	case wkbPolygon:
		return "polygon";
	case wkbMultiPoint:
		return "multi point";
	case wkbMultiLineString:
		return "multi line";
	case wkbMultiPolygon:
		return "multi polygon";
	default:
		return boost::str(boost::format("unsupported geometry %1%") % ogr_geometry_type);
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#if GDAL_VERSION_MAJOR >= 2

template <> std::runtime_error make_ogr_exception<ogr_create_data_source_error>(GDALDriver *driver, const boost::filesystem::path &path) {
	return std::runtime_error(boost::str(boost::format("Cannot create %1% %2%") % driver->GetDescription() % path));
}

template <> std::runtime_error make_ogr_exception<ogr_open_data_source_error>(GDALDriver *driver, const boost::filesystem::path &path) {
	return std::runtime_error(boost::str(boost::format("Cannot open %1% %2%") % driver->GetDescription() % path));
}

template <> std::runtime_error make_ogr_exception<ogr_create_layer_error>(GDALDataset *dataset, const std::string &layer_name) {
	return std::runtime_error(boost::str(boost::format("Cannot create layer '%1%' in data source '%2%'") % layer_name % dataset->GetLayer(0)->GetName()));
}

template <> std::runtime_error make_ogr_exception<ogr_delete_layer_error>(GDALDataset *dataset, const std::string &layer_name, OGRErr ogr_err) {
	return std::runtime_error(boost::str(boost::format("Cannot delete layer '%1%' from data source '%2%' (%3%)") % layer_name % dataset->GetLayer(0)->GetName() % get_ogr_error_string(ogr_err)));
}

#else

template <> std::runtime_error make_ogr_exception<ogr_create_data_source_error>(OGRSFDriver *driver, const boost::filesystem::path &path) {
	return std::runtime_error(boost::str(boost::format("Cannot create %1% %2%") % driver->GetName() % path));
}

template <> std::runtime_error make_ogr_exception<ogr_open_data_source_error>(OGRSFDriver *driver, const boost::filesystem::path &path) {
	return std::runtime_error(boost::str(boost::format("Cannot open %1% %2%") % driver->GetName() % path));
}

template <> std::runtime_error make_ogr_exception<ogr_delete_layer_error>(OGRDataSource *data_source, const std::string &layer_name, OGRErr ogr_err) {
	return std::runtime_error(boost::str(boost::format("Cannot delete layer '%1%' from data source '%2%' (%3%)") % layer_name % data_source->GetName() % get_ogr_error_string(ogr_err)));
}

template <> std::runtime_error make_ogr_exception<ogr_create_layer_error>(OGRDataSource *data_source, const std::string &layer_name) {
	return std::runtime_error(boost::str(boost::format("Cannot create layer '%1%' in data source '%2%'") % layer_name % data_source->GetName()));
}

#endif

template <> std::runtime_error make_ogr_exception<ogr_create_field_error>(OGRLayer *layer, OGRFieldDefn &ogr_field_defn, OGRErr ogr_err) {
	return std::runtime_error(boost::str(boost::format("Cannot create field '%1%' in layer '%2%' (%3%)") % ogr_field_defn.GetNameRef() % layer->GetName() % get_ogr_error_string(ogr_err)));
}

template <> std::runtime_error make_ogr_exception<ogr_create_feature_error>(OGRLayer *layer) {
	return std::runtime_error(boost::str(boost::format("Cannot create feature in layer '%1%'") % layer->GetName()));
}

template <> std::runtime_error make_ogr_exception<ogr_set_geometry_error>(OGRErr ogr_err) {
	return std::runtime_error(boost::str(boost::format("Cannot set geometry (%1%)") % get_ogr_error_string(ogr_err)));
}

template <> std::runtime_error make_ogr_exception<ogr_create_feature_error>(OGRLayer *layer, OGRErr ogr_err) {
	return std::runtime_error(boost::str(boost::format("Cannot create feature in layer '%1%' (%2%)") % layer->GetName() % get_ogr_error_string(ogr_err)));
}

template <> std::runtime_error make_ogr_exception<ogr_geometry_type_error>(OGRwkbGeometryType ogr_geometry_type) {
	return std::runtime_error(boost::str(boost::format("Geometry is not '%1%'") % get_ogr_geometry_string(ogr_geometry_type)));
}

} // namespace geofis
