/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#ifndef ATTRIBUTE_RANGE_HPP_
#define ATTRIBUTE_RANGE_HPP_

#include <boost/range/adaptor/transformed.hpp>
#include <geofis/data/is_monodimensional_feature.hpp>

namespace geofis {

namespace detail {

template <class Feature, class IsMonodimensionalFeature = typename is_monodimensional_feature<Feature>::type> struct attribute_function;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// specialization of attribute_function for monodimensional feature

template <class Feature> struct attribute_function<Feature, boost::true_type> {

	typedef typename Feature::attribute_type result_type;

	result_type operator() (const Feature &feature) const { return feature.get_attribute(); }
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// specialization of attribute_function for multidimensional feature

template <class Feature> struct attribute_function<Feature, boost::false_type> {

	typedef typename Feature::const_attribute_range_type result_type;

	result_type operator() (const Feature &feature) const { return feature.get_attribute_range(); }
};

} // namespace detail

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class FeatureRange> struct attribute_range_traits {

	typedef typename boost::range_value<FeatureRange>::type feature_type;
	typedef boost::transformed_range<detail::attribute_function<feature_type>, FeatureRange> base_type;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class FeatureRange> class attribute_range : public attribute_range_traits<FeatureRange>::base_type {

	typedef typename attribute_range_traits<FeatureRange>::base_type base_type;
	typedef typename attribute_range_traits<FeatureRange>::feature_type feature_type;

public:
	attribute_range(FeatureRange &features) : base_type(detail::attribute_function<feature_type>(), features) {}
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class FeatureRange> attribute_range<const FeatureRange> make_attribute_range(const FeatureRange &features) {
	return attribute_range<const FeatureRange>(features);
}

} // namespace geofis

#endif /* ATTRIBUTE_RANGE_HPP_ */
