/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#ifndef H720EA5AD_5BBC_4A16_93D4_0DD4296C2DB7
#define H720EA5AD_5BBC_4A16_93D4_0DD4296C2DB7

#include <ogrsf_frmts.h>
#include <stdexcept>
#include <CGAL/Polygon_2.h>
#include <boost/format.hpp>
#include <boost/filesystem/path.hpp>
#include <geofis/ogr/ogr_data_source.hpp>
#include <geofis/ogr/ogr_layer.hpp>
#include <geofis/ogr/ogr_feature.hpp>
#include <geofis/ogr/ogr_spatial_reference.hpp>

namespace geofis {

#if GDAL_VERSION_MAJOR >= 2
#define OGR_CONTOUR_THROW(condition, path, driver) if(!(condition)) throw std::runtime_error(boost::str(boost::format("%1% is not a valid %2% contour") % path % driver->GetDescription()));
#else
#define OGR_CONTOUR_THROW(condition, path, driver) if(!(condition)) throw std::runtime_error(boost::str(boost::format("%1% is not a valid %2% contour") % path % driver->GetName()));
#endif

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Kernel> class ogr_contour {

	typedef CGAL::Polygon_2<Kernel> polygon_type;

	ogr_data_source data_source;
	OGRSpatialReference *spatial_reference;
	polygon_type contour;

public:
#if GDAL_VERSION_MAJOR >= 2

	ogr_contour(GDALDriver *driver, const boost::filesystem::path &path) : data_source(path, driver), spatial_reference(NULL) {
		OGR_CONTOUR_THROW(data_source->GetLayerCount() == 1, path, driver);
		ogr_layer layer(data_source->GetLayer(0));
		if(layer->GetSpatialRef())
			spatial_reference = layer->GetSpatialRef()->Clone();
		OGR_CONTOUR_THROW(layer->GetFeatureCount() == 1, path, driver);
		ogr_feature feature(layer->GetNextFeature());
		OGR_CONTOUR_THROW(wkbFlatten(feature->GetGeometryRef()->getGeometryType()) == wkbPolygon, path, driver);
		contour = feature.get_geometry<polygon_type>();
	}

#else

	ogr_contour(OGRSFDriver *driver, const boost::filesystem::path &path) : data_source(path, driver), spatial_reference(NULL) {
		OGR_CONTOUR_THROW(data_source->GetLayerCount() == 1, path, driver);
		ogr_layer layer(data_source->GetLayer(0));
		if(layer->GetSpatialRef())
			spatial_reference = layer->GetSpatialRef()->Clone();
		OGR_CONTOUR_THROW(layer->GetFeatureCount() == 1, path, driver);
		ogr_feature feature(layer->GetNextFeature());
		OGR_CONTOUR_THROW(wkbFlatten(feature->GetGeometryRef()->getGeometryType()) == wkbPolygon, path, driver);
		contour = feature.get_geometry<polygon_type>();
	}

#endif

	virtual ~ogr_contour() {
		CPLFree(spatial_reference);
	}

	OGRSpatialReference *get_spatial_reference() const {
		return spatial_reference;
	}

	polygon_type get_contour() const {
		return contour;
	}
};

} // namespace geofis

#endif // H720EA5AD_5BBC_4A16_93D4_0DD4296C2DB7 
