/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#ifndef INCLUDE_GEOFIS_OGR_OGR_EXCEPTION_HPP_
#define INCLUDE_GEOFIS_OGR_OGR_EXCEPTION_HPP_

#include <ogrsf_frmts.h>
#include <boost/filesystem/path.hpp>

namespace geofis {

enum ogr_exception_error { ogr_create_data_source_error, ogr_open_data_source_error, ogr_create_layer_error, ogr_delete_layer_error, ogr_create_field_error, ogr_create_feature_error, ogr_set_geometry_error, ogr_geometry_type_error };

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#if GDAL_VERSION_MAJOR >= 2

template <ogr_exception_error State> std::runtime_error make_ogr_exception(GDALDriver *driver, const boost::filesystem::path &path);
template <> std::runtime_error make_ogr_exception<ogr_create_data_source_error>(GDALDriver *driver, const boost::filesystem::path &path);
template <> std::runtime_error make_ogr_exception<ogr_open_data_source_error>(GDALDriver *driver, const boost::filesystem::path &path);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(GDALDataset *dataset, const std::string &layer_name);
template <> std::runtime_error make_ogr_exception<ogr_create_layer_error>(GDALDataset *dataset, const std::string &layer_name);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(GDALDataset *dataset, const std::string &layer_name, OGRErr ogr_err);
template <> std::runtime_error make_ogr_exception<ogr_delete_layer_error>(GDALDataset *dataset, const std::string &layer_name, OGRErr ogr_err);

#else

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRSFDriver *driver, const boost::filesystem::path &path);
template <> std::runtime_error make_ogr_exception<ogr_create_data_source_error>(OGRSFDriver *driver, const boost::filesystem::path &path);
template <> std::runtime_error make_ogr_exception<ogr_open_data_source_error>(OGRSFDriver *driver, const boost::filesystem::path &path);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRDataSource *data_source, const std::string &layer_name);
template <> std::runtime_error make_ogr_exception<ogr_create_layer_error>(OGRDataSource *data_source, const std::string &layer_name);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRDataSource *data_source, const std::string &layer_name, OGRErr ogr_err);
template <> std::runtime_error make_ogr_exception<ogr_delete_layer_error>(OGRDataSource *data_source, const std::string &layer_name, OGRErr ogr_err);

#endif

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRLayer *layer, OGRFieldDefn &ogr_field_defn, OGRErr ogr_err);
template <> std::runtime_error make_ogr_exception<ogr_create_field_error>(OGRLayer *layer, OGRFieldDefn &ogr_field_defn, OGRErr ogr_err);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRLayer *layer);
template <> std::runtime_error make_ogr_exception<ogr_create_feature_error>(OGRLayer *layer);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRErr ogr_err);
template <> std::runtime_error make_ogr_exception<ogr_set_geometry_error>(OGRErr ogr_err);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRLayer *layer, OGRErr ogr_err);
template <> std::runtime_error make_ogr_exception<ogr_create_feature_error>(OGRLayer *layer, OGRErr ogr_err);

template <ogr_exception_error State> std::runtime_error make_ogr_exception(OGRwkbGeometryType ogr_geometry_type);
template <> std::runtime_error make_ogr_exception<ogr_geometry_type_error>(OGRwkbGeometryType ogr_geometry_type);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define OGR_ASSERT_THROW(condition, state, ...) if(!(condition)) throw make_ogr_exception<state>(__VA_ARGS__);
#define OGR_REQUIRE_THROW(condition, state, ...) OGR_ASSERT_THROW(condition, state, __VA_ARGS__)
#define OGR_ENSURE_THROW(condition, state, ...) OGR_ASSERT_THROW(condition, state, __VA_ARGS__)

} // namespace geofis

#endif /* INCLUDE_GEOFIS_OGR_OGR_EXCEPTION_HPP_ */
