/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#ifndef INCLUDE_GEOFIS_OGR_OGR_GEOMETRY_GETTER_HPP_
#define INCLUDE_GEOFIS_OGR_OGR_GEOMETRY_GETTER_HPP_

#include <util/assert.hpp>
#include <CGAL/Point_2.h>
#include <boost/range/adaptor/transformed.hpp>
#include <boost/range/adaptor/reversed.hpp>
#include <boost/range/counting_range.hpp>
#include <boost/range/adaptor/sliced.hpp>
#include <geofis/geometry/polygon.hpp>
#include <geofis/geometry/polygon_with_holes.hpp>
#include <geofis/ogr/ogr_exception.hpp>

namespace geofis {

namespace detail  {

template <class Geometry> struct geometry_getter;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Kernel> struct geometry_getter<CGAL::Point_2<Kernel> > {

	typedef CGAL::Point_2<Kernel> point_type;

	static point_type apply(OGRPoint *ogr_point) {
		UTIL_REQUIRE(ogr_point);
		return point_type(ogr_point->getX(), ogr_point->getY());
	}

	static point_type apply(OGRGeometry *ogr_geometry) {
		UTIL_REQUIRE(ogr_geometry);
		OGR_ENSURE_THROW(wkbFlatten(ogr_geometry->getGeometryType()) == wkbPoint, ogr_geometry_type_error, wkbPoint);
		return apply((OGRPoint *) ogr_geometry);
	}
};

template <class Kernel> inline CGAL::Point_2<Kernel> get_point(OGRPoint *ogr_point) {
	return geometry_getter<CGAL::Point_2<Kernel> >::apply(ogr_point);
}

template <class Kernel> struct to_point {

	typedef CGAL::Point_2<Kernel> result_type;

	OGRLinearRing *ogr_linear_ring;

	to_point(OGRLinearRing *ogr_linear_ring) : ogr_linear_ring(ogr_linear_ring) {}

	result_type operator()(int point) const {
		UTIL_REQUIRE(ogr_linear_ring);
		OGRPoint ogr_point;
		ogr_linear_ring->getPoint(point, &ogr_point);
		return  get_point<Kernel>(&ogr_point);
	}
};

template <class Kernel> struct point_range_traits {

	typedef boost::iterator_range<boost::counting_iterator<int> > counting_range_type;
	typedef boost::transformed_range<to_point<Kernel>, const counting_range_type> range_type;
};

template <class Kernel> inline typename point_range_traits<Kernel>::range_type make_point_range(OGRLinearRing *ogr_linear_ring) {
	return boost::adaptors::transform(boost::counting_range(0, ogr_linear_ring->getNumPoints()), to_point<Kernel>(ogr_linear_ring));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Kernel> struct geometry_getter<CGAL::Polygon_2<Kernel> > {

	typedef CGAL::Polygon_2<Kernel> polygon_type;

	static polygon_type apply(OGRLinearRing *ogr_linear_ring) {
		UTIL_REQUIRE(ogr_linear_ring);
		UTIL_REQUIRE(ogr_linear_ring->get_IsClosed());
		polygon_type polygon;
		auto point_range = boost::adaptors::slice(make_point_range<Kernel>(ogr_linear_ring), 0, ogr_linear_ring->getNumPoints() - 1);
		if(ogr_linear_ring->isClockwise())
			polygon = make_polygon<Kernel>(boost::adaptors::reverse(point_range));
		else
			polygon = make_polygon<Kernel>(point_range);
		UTIL_ENSURE(is_valid_polygon(polygon));
		return polygon;
	}

	static polygon_type apply(OGRPolygon *ogr_polygon) {
		UTIL_REQUIRE(ogr_polygon);
		OGR_ENSURE_THROW(ogr_polygon->getNumInteriorRings() == 0, ogr_geometry_type_error, wkbPolygon);
		UTIL_REQUIRE(ogr_polygon->IsValid());
		return apply(ogr_polygon->getExteriorRing());
	}

	static polygon_type apply(OGRGeometry *ogr_geometry) {
		UTIL_REQUIRE(ogr_geometry);
		OGR_ENSURE_THROW(wkbFlatten(ogr_geometry->getGeometryType()) == wkbPolygon, ogr_geometry_type_error, wkbPolygon);
		return apply((OGRPolygon *) ogr_geometry);
	}
};

template <class Kernel> inline CGAL::Polygon_2<Kernel> get_polygon(OGRLinearRing *ogr_linear_ring) {
	return geometry_getter<CGAL::Polygon_2<Kernel> >::apply(ogr_linear_ring);
}

template <class Kernel> struct to_polygon {

	typedef CGAL::Polygon_2<Kernel> result_type;

	OGRPolygon *ogr_polygon;

	to_polygon(OGRPolygon *ogr_polygon) : ogr_polygon(ogr_polygon) {}

	result_type operator()(int polygon) const {
		UTIL_REQUIRE(ogr_polygon);
		return get_polygon<Kernel>(ogr_polygon->getInteriorRing(polygon));
	}
};

template <class Kernel> struct hole_range_traits {

	typedef boost::iterator_range<boost::counting_iterator<int> > counting_range_type;
	typedef boost::transformed_range<to_polygon<Kernel>, const counting_range_type> range_type;
};

template <class Kernel> inline typename hole_range_traits<Kernel>::range_type make_hole_range(OGRPolygon *ogr_polygon) {
	return boost::adaptors::transform(boost::counting_range(0, ogr_polygon->getNumInteriorRings()), to_polygon<Kernel>(ogr_polygon));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Kernel> struct geometry_getter<CGAL::Polygon_with_holes_2<Kernel> > {

	typedef CGAL::Polygon_2<Kernel> polygon_type;
	typedef CGAL::Polygon_with_holes_2<Kernel> polygon_with_holes_type;

	static polygon_with_holes_type apply(OGRPolygon *ogr_polygon) {
		UTIL_REQUIRE(ogr_polygon);
		UTIL_REQUIRE(ogr_polygon->IsValid());
		polygon_with_holes_type polygon_with_holes = make_polygon_with_holes(get_polygon<Kernel>(ogr_polygon->getExteriorRing()), make_hole_range<Kernel>(ogr_polygon));
		UTIL_ENSURE(is_valid_polygon_with_holes(polygon_with_holes));
		return polygon_with_holes;
	}

	static polygon_with_holes_type apply(OGRGeometry *ogr_geometry) {
		UTIL_REQUIRE(ogr_geometry);
		OGR_ENSURE_THROW(wkbFlatten(ogr_geometry->getGeometryType()) == wkbPolygon, ogr_geometry_type_error, wkbPolygon);
		return apply((OGRPolygon *) ogr_geometry);
	}
};

} // namespace detail

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Geometry> inline Geometry get_geometry(OGRGeometry *ogr_geometry) {
	UTIL_REQUIRE(ogr_geometry);
    return detail::geometry_getter<Geometry>::apply(ogr_geometry);
}

} // namespace geofis

#endif /* INCLUDE_GEOFIS_OGR_OGR_GEOMETRY_GETTER_HPP_ */
