/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#ifndef INCLUDE_GEOFIS_OGR_OGR_GEOMETRY_SETTER_HPP_
#define INCLUDE_GEOFIS_OGR_OGR_GEOMETRY_SETTER_HPP_

#include <algorithm>
#include <ogr_geometry.h>
#include <CGAL/Point_2.h>
#include <geofis/geometry/polygon.hpp>
#include <geofis/geometry/polygon_with_holes.hpp>
#include <geofis/ogr/ogr_geometry_traits.hpp>

namespace geofis {

namespace detail {

template <class OGRGeometry, class Geometry> struct ogr_geometry_setter;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Kernel> struct ogr_geometry_setter<OGRPoint, CGAL::Point_2<Kernel> > {

	typedef CGAL::Point_2<Kernel> point_type;

	static void apply(OGRPoint &ogr_point, const point_type &point) {
		ogr_point.setX(CGAL::to_double(point.x()));
		ogr_point.setY(CGAL::to_double(point.y()));
		UTIL_ENSURE(ogr_point.IsValid());
	}
};

template <class Kernel> inline void initialize_ogr_point(OGRPoint &ogr_point, const CGAL::Point_2<Kernel> &point) {
    ogr_geometry_setter<OGRPoint, CGAL::Point_2<Kernel> >::apply(ogr_point, point);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct add_point_to_ogr_linear_ring {

	OGRLinearRing &ogr_linear_ring;

	add_point_to_ogr_linear_ring(OGRLinearRing &ogr_linear_ring) : ogr_linear_ring(ogr_linear_ring) {}

	template <class Kernel> void operator()(const CGAL::Point_2<Kernel> &point) {
		OGRPoint ogr_point;
		initialize_ogr_point(ogr_point, point);
		ogr_linear_ring.addPoint(&ogr_point);
	}
};

template <class Kernel> struct ogr_geometry_setter<OGRLinearRing, CGAL::Polygon_2<Kernel> > {

	typedef CGAL::Polygon_2<Kernel> polygon_type;

	static void apply(OGRLinearRing &ogr_linear_ring, const polygon_type &polygon) {
		UTIL_REQUIRE(is_valid_polygon(polygon));
		std::for_each(polygon.vertices_begin(), polygon.vertices_end(), add_point_to_ogr_linear_ring(ogr_linear_ring));
		ogr_linear_ring.closeRings();
		// UTIL_ENSURE(ogr_linear_ring.IsValid()); IsValid() or IsRing() returns for LinearRing always False. https://trac.osgeo.org/gdal/ticket/5006
	}
};

template <class Kernel> inline void initialize_ogr_linear_ring(OGRLinearRing &ogr_linear_ring, const CGAL::Polygon_2<Kernel> &polygon) {
    ogr_geometry_setter<OGRLinearRing, CGAL::Polygon_2<Kernel> >::apply(ogr_linear_ring, polygon);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Kernel> struct ogr_geometry_setter<OGRPolygon, CGAL::Polygon_2<Kernel> > {

	typedef CGAL::Polygon_2<Kernel> polygon_type;

	static void apply(OGRPolygon &ogr_polygon, const polygon_type &polygon) {
		UTIL_REQUIRE(is_valid_polygon(polygon));
		OGRLinearRing ogr_linear_ring;
		initialize_ogr_linear_ring(ogr_linear_ring, polygon);
		ogr_polygon.addRing(&ogr_linear_ring);
		UTIL_ENSURE(ogr_polygon.IsValid());
	}
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct add_hole_to_ogr_polygon {

	OGRPolygon &ogr_polygon;

	add_hole_to_ogr_polygon(OGRPolygon &ogr_polygon) : ogr_polygon(ogr_polygon) {}

	template <class Kernel> void operator()(CGAL::Polygon_2<Kernel> polygon) {
		OGRLinearRing ogr_linear_ring;
		polygon.reverse_orientation();
		initialize_ogr_linear_ring(ogr_linear_ring, polygon);
		ogr_polygon.addRing(&ogr_linear_ring);
	}
};

template <class Kernel> struct ogr_geometry_setter<OGRPolygon, CGAL::Polygon_with_holes_2<Kernel> > {

	typedef CGAL::Polygon_with_holes_2<Kernel> polygon_with_holes_type;

	static void apply(OGRPolygon &ogr_polygon, const polygon_with_holes_type &polygon_with_holes) {
		UTIL_REQUIRE(is_valid_polygon_with_holes(polygon_with_holes));
		OGRLinearRing ogr_linear_ring;
		initialize_ogr_linear_ring(ogr_linear_ring, polygon_with_holes.outer_boundary());
		ogr_polygon.addRing(&ogr_linear_ring);
		std::for_each(polygon_with_holes.holes_begin(), polygon_with_holes.holes_end(), add_hole_to_ogr_polygon(ogr_polygon));
		UTIL_ENSURE(ogr_polygon.IsValid());
	}
};

} // namespace detail

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class OGRGeometry, class Geometry> inline void set_ogr_geometry(OGRGeometry &ogr_geometry, const Geometry &geometry) {
    detail::ogr_geometry_setter<OGRGeometry, Geometry>::apply(ogr_geometry, geometry);
}

} // namespace geofis

#endif /* INCLUDE_GEOFIS_OGR_OGR_GEOMETRY_SETTER_HPP_ */
