/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/algorithm/zoning/fusion/aggregation/minimum_aggregation.hpp>
#include <gmock/gmock.h>
#include <util/gmock/matchers.hpp>
#include <boost/assign/list_of.hpp>
#include <list>

using namespace std;
using namespace geofis;
using namespace testing;
using namespace boost;
using namespace boost::assign;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace {

struct zone_pair_mock {

	MOCK_CONST_METHOD0( get_distance, double () );
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct zone_pair {

	zone_pair(zone_pair_mock &mock) : mock(mock) {}
	zone_pair(const zone_pair &other) : mock(other.mock) {}

	double get_distance() const {
		return mock.get_distance();
	}

	zone_pair_mock &mock;
};

} // namespace

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef list<zone_pair> zone_pair_container_type;
typedef typename zone_pair_container_type::iterator zone_pair_iterator_type;
typedef list<zone_pair_iterator_type> zone_pair_iterator_container_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(minimum_aggregation_tester, test_minimum_zone_pairs_distances) {

	zone_pair_mock zone_pair_mock1;
	EXPECT_CALL( zone_pair_mock1, get_distance() ).WillRepeatedly(Return(4));

	zone_pair_mock zone_pair_mock2;
	EXPECT_CALL( zone_pair_mock2, get_distance() ).WillRepeatedly(Return(2));

	zone_pair_mock zone_pair_mock3;
	EXPECT_CALL( zone_pair_mock3, get_distance() ).WillRepeatedly(Return(1));

	zone_pair_mock zone_pair_mock4;
	EXPECT_CALL( zone_pair_mock4, get_distance() ).WillRepeatedly(Return(3));

	zone_pair_container_type zone_pairs = list_of(zone_pair(zone_pair_mock1))(zone_pair(zone_pair_mock2))(zone_pair(zone_pair_mock3))(zone_pair(zone_pair_mock4));
	zone_pair_iterator_container_type zone_pairs_to_merge;

	minimum_aggregation()(zone_pairs, back_inserter(zone_pairs_to_merge));
	ASSERT_THAT( zone_pairs, IsSorted(zone_pair_distance_less()));
	ASSERT_THAT( zone_pairs_to_merge.size(), Eq((size_t) 1));
	ASSERT_THAT( zone_pairs_to_merge, EqRange(list_of(boost::begin(zone_pairs))));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(minimum_aggregation_tester, test_when_many_zone_pairs_distances_are_equals) {

	zone_pair_mock zone_pair_mock1;
	EXPECT_CALL( zone_pair_mock1, get_distance() ).WillRepeatedly(Return(3));

	zone_pair_mock zone_pair_mock2;
	EXPECT_CALL( zone_pair_mock2, get_distance() ).WillRepeatedly(Return(1));

	zone_pair_mock zone_pair_mock3;
	EXPECT_CALL( zone_pair_mock3, get_distance() ).WillRepeatedly(Return(2));

	zone_pair_mock zone_pair_mock4;
	EXPECT_CALL( zone_pair_mock4, get_distance() ).WillRepeatedly(Return(1));

	zone_pair_container_type zone_pairs = list_of(zone_pair(zone_pair_mock1))(zone_pair(zone_pair_mock2))(zone_pair(zone_pair_mock3))(zone_pair(zone_pair_mock4));
	zone_pair_iterator_container_type zone_pairs_to_merge;

	minimum_aggregation()(zone_pairs, back_inserter(zone_pairs_to_merge));
	ASSERT_THAT( zone_pairs, IsSorted(zone_pair_distance_less()));
	ASSERT_THAT( zone_pairs_to_merge.size(), Eq((size_t) 2));
	ASSERT_THAT( zone_pairs_to_merge, EqRange(list_of(boost::begin(zone_pairs))(boost::next(boost::begin(zone_pairs), 1))));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(minimum_aggregation_tester, test_when_all_zone_pairs_distances_are_equals) {

	zone_pair_mock zone_pair_mock1;
	EXPECT_CALL( zone_pair_mock1, get_distance() ).WillRepeatedly(Return(1));

	zone_pair_mock zone_pair_mock2;
	EXPECT_CALL( zone_pair_mock2, get_distance() ).WillRepeatedly(Return(1));

	zone_pair_mock zone_pair_mock3;
	EXPECT_CALL( zone_pair_mock3, get_distance() ).WillRepeatedly(Return(1));

	zone_pair_mock zone_pair_mock4;
	EXPECT_CALL( zone_pair_mock4, get_distance() ).WillRepeatedly(Return(1));

	zone_pair_container_type zone_pairs = list_of(zone_pair(zone_pair_mock1))(zone_pair(zone_pair_mock2))(zone_pair(zone_pair_mock3))(zone_pair(zone_pair_mock4));
	zone_pair_iterator_container_type zone_pairs_to_merge;

	minimum_aggregation()(zone_pairs, back_inserter(zone_pairs_to_merge));
	ASSERT_THAT( zone_pairs, IsSorted(zone_pair_distance_less()));
	ASSERT_THAT( zone_pairs_to_merge.size(), Eq((size_t) 4));
	ASSERT_THAT( zone_pairs_to_merge, EqRange(list_of(boost::begin(zone_pairs))(boost::next(boost::begin(zone_pairs), 1))(boost::next(boost::begin(zone_pairs), 2))(boost::next(boost::begin(zone_pairs), 3))));
}
