/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/algorithm/zoning/fusion/distance/feature_distance.hpp>
#include <util/functional/distance/euclidean_distance.hpp>
#include <boost/range/any_range.hpp>
#include <boost/assign/list_of.hpp>
#include <gmock/gmock.h>

using namespace std;
using namespace util;
using namespace geofis;
using namespace boost;
using namespace boost::assign;
using namespace testing;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace {

struct monodimensional_feature_mock {

	MOCK_CONST_METHOD0( get_normalized_attribute, double () );
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct multidimensional_feature_mock {

	typedef any_range<double, single_pass_traversal_tag, const double> const_attribute_range_type;

	MOCK_CONST_METHOD0( get_normalized_attribute_range, const_attribute_range_type () );
	MOCK_CONST_METHOD0( get_normalized_attribute_size, size_t () );
};

} // namespace

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace geofis {

template <> struct is_monodimensional_feature<monodimensional_feature_mock> : boost::true_type {};

} // namespace geofis

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef euclidean_distance<double> attribute_distance_type;
typedef euclidean_distance<double> multidimensional_distance_type;

typedef feature_distance<multidimensional_distance_type, attribute_distance_type> feature_distance_type;
typedef feature_distance<void, attribute_distance_type> monodimensional_feature_distance_type;
typedef feature_distance<multidimensional_distance_type, void> multidimensional_feature_distance_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(feature_distance_tester, test_feature_distance) {

	multidimensional_feature_mock feature_mock1, feature_mock2;

	EXPECT_CALL( feature_mock1, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.0)(0.1)(0.2)));
	EXPECT_CALL( feature_mock2, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.1)(0.3)(0.5)));
	EXPECT_CALL( feature_mock1, get_normalized_attribute_size() ).WillRepeatedly(Return(3));
	EXPECT_CALL( feature_mock2, get_normalized_attribute_size() ).WillRepeatedly(Return(3));

	double expected_feature_distance = sqrt(pow(0.1, 2) + pow(0.2, 2) + pow(0.3, 2));

	{
		feature_distance_type feature_distance(3);

		ASSERT_DOUBLE_EQ( expected_feature_distance, feature_distance(feature_mock1, feature_mock2) );
	}

	{
		attribute_distance_type attribute_distance;
		feature_distance_type feature_distance(3, attribute_distance);

		ASSERT_DOUBLE_EQ( expected_feature_distance, feature_distance(feature_mock1, feature_mock2) );
	}

	{
		feature_distance_type feature_distance(list_of(attribute_distance_type())(attribute_distance_type())(attribute_distance_type()));

		ASSERT_DOUBLE_EQ( expected_feature_distance, feature_distance(feature_mock1, feature_mock2) );
	}

	{
		multidimensional_distance_type multidimensional_distance;
		feature_distance_type feature_distance(multidimensional_distance, list_of(attribute_distance_type())(attribute_distance_type())(attribute_distance_type()));

		ASSERT_DOUBLE_EQ( expected_feature_distance, feature_distance(feature_mock1, feature_mock2) );
	}

	{
		multidimensional_distance_type multidimensional_distance;
		feature_distance_type feature_distance(multidimensional_distance, 3);

		ASSERT_DOUBLE_EQ( expected_feature_distance, feature_distance(feature_mock1, feature_mock2) );
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(feature_distance_tester, test_monodimensional_feature_distance) {

	{
		monodimensional_feature_mock feature_mock1, feature_mock2;

		EXPECT_CALL( feature_mock1, get_normalized_attribute() ).WillRepeatedly(Return(0.1));
		EXPECT_CALL( feature_mock2, get_normalized_attribute() ).WillRepeatedly(Return(0.3));

		double expected_monodimensional_feature_distance = 0.2;

		monodimensional_feature_distance_type monodimensional_feature_distance;

		ASSERT_DOUBLE_EQ( expected_monodimensional_feature_distance, monodimensional_feature_distance(feature_mock1, feature_mock2) );
	}

	{
		multidimensional_feature_mock feature_mock1, feature_mock2;

		EXPECT_CALL( feature_mock1, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.1)));
		EXPECT_CALL( feature_mock2, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.3)));
		EXPECT_CALL( feature_mock1, get_normalized_attribute_size() ).WillRepeatedly(Return(1));
		EXPECT_CALL( feature_mock2, get_normalized_attribute_size() ).WillRepeatedly(Return(1));

		double expected_monodimensional_feature_distance = 0.2;

		{
			monodimensional_feature_distance_type monodimensional_feature_distance;

			ASSERT_DOUBLE_EQ( expected_monodimensional_feature_distance, monodimensional_feature_distance(feature_mock1, feature_mock2) );
		}

		{
			attribute_distance_type attribute_distance;
			monodimensional_feature_distance_type monodimensional_feature_distance(attribute_distance);

			ASSERT_DOUBLE_EQ( expected_monodimensional_feature_distance, monodimensional_feature_distance(feature_mock1, feature_mock2) );
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(feature_distance_tester, test_multidimensional_feature_distance) {

	multidimensional_feature_mock feature_mock1, feature_mock2;

	EXPECT_CALL( feature_mock1, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.0)(0.1)(0.2)));
	EXPECT_CALL( feature_mock2, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.1)(0.3)(0.5)));
	EXPECT_CALL( feature_mock1, get_normalized_attribute_size() ).WillRepeatedly(Return(3));
	EXPECT_CALL( feature_mock2, get_normalized_attribute_size() ).WillRepeatedly(Return(3));

	double expected_multidimensional_feature_distance = sqrt(pow(0.1, 2) + pow(0.2, 2) + pow(0.3, 2));

	{
		multidimensional_feature_distance_type multidimensional_feature_distance;

		ASSERT_DOUBLE_EQ( expected_multidimensional_feature_distance, multidimensional_feature_distance(feature_mock1, feature_mock2) );
	}

	{
		multidimensional_distance_type multidimensional_distance;
		multidimensional_feature_distance_type multidimensional_feature_distance(multidimensional_distance);

		ASSERT_DOUBLE_EQ( expected_multidimensional_feature_distance, multidimensional_feature_distance(feature_mock1, feature_mock2) );
	}
}
