/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/algorithm/zoning/fusion/distance/zone_distance_adapter.hpp>
#include <geofis/algorithm/zoning/fusion/distance/variant_zone_distance.hpp>
#include <geofis/algorithm/zoning/fusion/distance/variant_multidimensional_distance.hpp>
#include <geofis/algorithm/zoning/fusion/distance/variant_attribute_distance.hpp>
#include <geofis/algorithm/zoning/fusion/distance/variant_feature_distance.hpp>
#include <boost/range/any_range.hpp>
#include <boost/assign/list_of.hpp>
#include <gmock/gmock.h>

using namespace geofis;
using namespace boost;
using namespace boost::assign;
using namespace testing;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace {

struct feature_mock {

	typedef any_range<double, forward_traversal_tag, const double> attribute_range_type;

	MOCK_CONST_METHOD0( get_normalized_attribute_range, attribute_range_type () );
	MOCK_CONST_METHOD0( get_normalized_attribute_size, size_t () );
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct zone_mock {

	typedef any_range<feature_mock, forward_traversal_tag, feature_mock &> feature_range_type;

	MOCK_CONST_METHOD0( get_feature_range, feature_range_type () );
};

} // namespace

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef variant_zone_distance zone_distance_type;
typedef variant_multidimensional_distance multidimensional_distance_type;
typedef variant_attribute_distance attribute_distance_type;
typedef typename variant_feature_distance_traits<multidimensional_distance_type, attribute_distance_type>::variant_feature_distance_type feature_distance_type;

typedef zone_distance_adapter<zone_distance_type, feature_distance_type> zone_distance_adapter_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(zone_distance_adapter_tester, test_zone_distance_adapter) {

	zone_mock zone_mock1, zone_mock2;
	feature_mock feature_mock1, feature_mock2, feature_mock3, feature_mock4;

	EXPECT_CALL( feature_mock1, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.0)(0.1)(0.2)));
	EXPECT_CALL( feature_mock2, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.1)(0.2)(0.3)));
	EXPECT_CALL( feature_mock3, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(0.3)(0.7)(0.9)));
	EXPECT_CALL( feature_mock4, get_normalized_attribute_range() ).WillRepeatedly(Return(list_of(1.0)(1.0)(1.0)));
	// see geofis-native/src/test/resources/geofis/algorithm/zoning/fusion/distancezone_distance_adapter_tester.xls
	// multidimensional distance features(1, 3) = 0.969535971
	// multidimensional distance features(1, 4) = 1.565247584
	// multidimensional distance features(2, 3) = 0.806225775
	// multidimensional distance features(2, 4) = 1.392838828
	EXPECT_CALL( feature_mock1, get_normalized_attribute_size() ).WillRepeatedly(Return(3));
	EXPECT_CALL( feature_mock2, get_normalized_attribute_size() ).WillRepeatedly(Return(3));
	EXPECT_CALL( feature_mock3, get_normalized_attribute_size() ).WillRepeatedly(Return(3));
	EXPECT_CALL( feature_mock4, get_normalized_attribute_size() ).WillRepeatedly(Return(3));

	EXPECT_CALL( zone_mock1, get_feature_range() ).WillRepeatedly(Return(ref_list_of<2>(feature_mock1)(feature_mock2)));
	EXPECT_CALL( zone_mock2, get_feature_range() ).WillRepeatedly(Return(ref_list_of<2>(feature_mock3)(feature_mock4)));

	zone_distance_type zone_distance;
	multidimensional_distance_type multidimensional_distance;
	vector<attribute_distance_type> attribute_distances(3);

	zone_distance_adapter_type zone_distance_adapter(zone_distance, multidimensional_distance, attribute_distances);
	ASSERT_NEAR( 0.806225775, zone_distance_adapter(zone_mock1, zone_mock2), 1e-9 );
}
