/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/algorithm/zoning/pair/zone_pair_distance.hpp>
#include <geofis/algorithm/zoning/fusion/distance/variant_zone_distance.hpp>
#include <boost/assign/list_of.hpp>
#include <cppunit/extensions/HelperMacros.h>

using namespace util;
using namespace geofis;
using namespace boost::assign;

typedef zone_pair_distance<maximum<double> > maximum_zone_pair_distance_type;
typedef zone_pair_distance<minimum<double> > minimum_zone_pair_distance_type;
typedef zone_pair_distance<mean<double> > mean_zone_pair_distance_type;
typedef zone_pair_distance<variant_zone_distance> variant_zone_pair_distance_type;

#define DELTA 1e-6

class zone_pair_distance_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( zone_pair_distance_tester );

	CPPUNIT_TEST( test_maximum_zone_pair_distance );
	CPPUNIT_TEST( test_variant_maximum_zone_pair_distance );

	CPPUNIT_TEST( test_minimum_zone_pair_distance );
	CPPUNIT_TEST( test_variant_minimum_zone_pair_distance );

	CPPUNIT_TEST( test_mean_zone_pair_distance );
	CPPUNIT_TEST( test_variant_mean_zone_pair_distance );

	CPPUNIT_TEST_SUITE_END();

public:

	template <class ZonePairDistance> void test_maximum_zone_pair_distance(ZonePairDistance &zone_pair_distance, const ZonePairDistance &other_zone_pair_distance) {
		CPPUNIT_ASSERT_DOUBLES_EQUAL(3., zone_pair_distance.get_distance(), DELTA);
		zone_pair_distance.update_feature_distances(list_of(4.)(6.)(5.));
		CPPUNIT_ASSERT_DOUBLES_EQUAL(6., zone_pair_distance.get_distance(), DELTA);
		zone_pair_distance.update(other_zone_pair_distance);
		CPPUNIT_ASSERT_DOUBLES_EQUAL(9., zone_pair_distance.get_distance(), DELTA);
	}

	void test_maximum_zone_pair_distance() {
		maximum_zone_pair_distance_type zone_pair_distance(maximum<double>(), list_of(1.)(3.)(2.));
		maximum_zone_pair_distance_type other_zone_pair_distance(maximum<double>(), list_of(7.)(9.)(8.));
		test_maximum_zone_pair_distance(zone_pair_distance, other_zone_pair_distance);
	}

	void test_variant_maximum_zone_pair_distance() {
		variant_zone_pair_distance_type zone_pair_distance(variant_zone_distance(util::maximum<double>()), list_of(1.)(3.)(2.));
		variant_zone_pair_distance_type other_zone_pair_distance(variant_zone_distance(util::maximum<double>()), list_of(7.)(9.)(8.));
		test_maximum_zone_pair_distance(zone_pair_distance, other_zone_pair_distance);
	}

	template <class ZonePairDistance> void test_minimum_zone_pair_distance(ZonePairDistance &zone_pair_distance, const ZonePairDistance &other_zone_pair_distance) {
		CPPUNIT_ASSERT_DOUBLES_EQUAL(7., zone_pair_distance.get_distance(), DELTA);
		zone_pair_distance.update_feature_distances(list_of(5.)(4.)(6.));
		CPPUNIT_ASSERT_DOUBLES_EQUAL(4., zone_pair_distance.get_distance(), DELTA);
		zone_pair_distance.update(other_zone_pair_distance);
		CPPUNIT_ASSERT_DOUBLES_EQUAL(1., zone_pair_distance.get_distance(), DELTA);
	}

	void test_minimum_zone_pair_distance() {
		minimum_zone_pair_distance_type zone_pair_distance(minimum<double>(), list_of(8.)(7.)(9.));
		minimum_zone_pair_distance_type other_zone_pair_distance(minimum<double>(), list_of(2.)(1.)(3.));
		test_minimum_zone_pair_distance(zone_pair_distance, other_zone_pair_distance);
	}

	void test_variant_minimum_zone_pair_distance() {
		variant_zone_pair_distance_type zone_pair_distance(variant_zone_distance(minimum<double>()), list_of(8.)(7.)(9.));
		variant_zone_pair_distance_type other_zone_pair_distance(variant_zone_distance(minimum<double>()), list_of(2.)(1.)(3.));
		test_minimum_zone_pair_distance(zone_pair_distance, other_zone_pair_distance);
	}

	template <class ZonePairDistance> void test_mean_zone_pair_distance(ZonePairDistance &zone_pair_distance, const ZonePairDistance &other_zone_pair_distance) {
		CPPUNIT_ASSERT_DOUBLES_EQUAL(3., zone_pair_distance.get_distance(), DELTA);
		zone_pair_distance.update_feature_distances(list_of(8.)(10.)(3.));
		CPPUNIT_ASSERT_DOUBLES_EQUAL(5.4, zone_pair_distance.get_distance(), DELTA);
		zone_pair_distance.update(other_zone_pair_distance);
		CPPUNIT_ASSERT_DOUBLES_EQUAL(9, zone_pair_distance.get_distance(), DELTA);
	}

	void test_mean_zone_pair_distance() {
		mean_zone_pair_distance_type zone_pair_distance(mean<double>(), list_of(1.)(5.));
		mean_zone_pair_distance_type other_zone_pair_distance(mean<double>(), list_of(5.)(9.)(10.)(30.));
		test_mean_zone_pair_distance(zone_pair_distance, other_zone_pair_distance);
	}

	void test_variant_mean_zone_pair_distance() {
		variant_zone_pair_distance_type zone_pair_distance(variant_zone_distance(mean<double>()), list_of(1.)(5.));
		variant_zone_pair_distance_type other_zone_pair_distance(variant_zone_distance(mean<double>()), list_of(5.)(9.)(10.)(30.));
		test_mean_zone_pair_distance(zone_pair_distance, other_zone_pair_distance);
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( zone_pair_distance_tester );
