/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/algorithm/zoning/triangulation/voronoi/face_to_polygon.hpp>
#include <cppunit/extensions/HelperMacros.h>
//#include <CGAL/Exact_predicates_exact_constructions_kernel.h>
#include <CGAL/Cartesian.h>
#include <CGAL/Polygon_2.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/Delaunay_triangulation_adaptation_traits_2.h>
#include <CGAL/Voronoi_diagram_2.h>
#include <boost/assign/list_of.hpp>
#include <boost/assign/list_inserter.hpp>
#include <boost/range/adaptor/transformed.hpp>
#include <boost/range/algorithm/count.hpp>
#include <boost/range/algorithm/find.hpp>
#include <geofis/geometry/polygon.hpp>

#include <util/type.hpp>

using namespace std;
using namespace geofis;
using namespace CGAL;
using namespace boost;
using namespace boost::assign;
using namespace boost::adaptors;

//typedef Exact_predicates_exact_constructions_kernel kernel_type;
typedef Cartesian<double> kernel_type;
typedef Point_2<kernel_type> point_type;
typedef Polygon_2<kernel_type> polygon_type;

typedef Delaunay_triangulation_2<kernel_type> delaunay_triangulation_type;
typedef Delaunay_triangulation_adaptation_traits_2<delaunay_triangulation_type> delaunay_triangulation_adaptation_type;
typedef Voronoi_diagram_2<delaunay_triangulation_type, delaunay_triangulation_adaptation_type> voronoi_diagram_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class face_to_polygon_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( face_to_polygon_tester );

	CPPUNIT_TEST( test );

	CPPUNIT_TEST_SUITE_END();

public:

	template <class Range> bool equal_all(const polygon_type &face_polygon, const Range &range) {
		count(range, face_polygon);
		find<return_found_end>(range, face_polygon);
		return (count(range, face_polygon) == 1) && !find<return_found_end>(range, face_polygon).empty();
	}

	template <class Iterator, class Range> bool equal_all(Iterator first, Iterator last, const Range &range) {
		while(first != last)
			if(!equal_all(*first++, range))
				return false;
		return true;
	}

	template <class FacePolygon, class Result> bool equal_all(const FacePolygon &face_polygons, const Result &results) {
		CPPUNIT_ASSERT_EQUAL( distance(face_polygons),  distance(results) );
		return equal_all(boost::begin(face_polygons), boost::end(face_polygons), results);
	}

	template <class FacePolygon, class Result> void test_result_face_to_polygon(const FacePolygon &face_polygons, const Result &results) {
		CPPUNIT_ASSERT( equal_all(face_polygons, results) );
	}

	template <class Result> void test_result_face_to_polygon(const voronoi_diagram_type &voronoi, const polygon_type &boundary, const Result &results) {
		test_result_face_to_polygon(transform(make_iterator_range(voronoi.faces_begin(), voronoi.faces_end()), make_face_to_polygon(boundary)), results);
	}

	void test_face_to_polygon1(const voronoi_diagram_type &voronoi) {

		polygon_type boundary_polygon = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(6, 0))(point_type(6, 6))(point_type(0, 6)));

		polygon_type result_polygon1 = make_polygon<kernel_type>(list_of(point_type(3, 1))(point_type(5, 3))(point_type(3, 5))(point_type(1, 3)));
		polygon_type result_polygon2 = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(3, 0))(point_type(3, 1))(point_type(1, 3))(point_type(0, 3)));
		polygon_type result_polygon3 = make_polygon<kernel_type>(list_of(point_type(3, 0))(point_type(6, 0))(point_type(6, 3))(point_type(5, 3))(point_type(3, 1)));
		polygon_type result_polygon4 = make_polygon<kernel_type>(list_of(point_type(6, 3))(point_type(6, 6))(point_type(3, 6))(point_type(3, 5))(point_type(5, 3)));
		polygon_type result_polygon5 = make_polygon<kernel_type>(list_of(point_type(3, 6))(point_type(0, 6))(point_type(0, 3))(point_type(1, 3))(point_type(3, 5)));

		test_result_face_to_polygon(voronoi, boundary_polygon, list_of(result_polygon1)(result_polygon2)(result_polygon3)(result_polygon4)(result_polygon5));
	}

	void test_face_to_polygon2(const voronoi_diagram_type &voronoi) {

		polygon_type boundary_polygon = make_polygon<kernel_type>(list_of(point_type(1, 1))(point_type(5, 1))(point_type(5, 5))(point_type(1, 5)));

		polygon_type result_polygon1 = make_polygon<kernel_type>(list_of(point_type(3, 1))(point_type(5, 3))(point_type(3, 5))(point_type(1, 3)));
		polygon_type result_polygon2 = make_polygon<kernel_type>(list_of(point_type(1, 1))(point_type(3, 1))(point_type(1, 3)));
		polygon_type result_polygon3 = make_polygon<kernel_type>(list_of(point_type(3, 1))(point_type(5, 1))(point_type(5, 3)));
		polygon_type result_polygon4 = make_polygon<kernel_type>(list_of(point_type(5, 3))(point_type(5, 5))(point_type(3, 5)));
		polygon_type result_polygon5 = make_polygon<kernel_type>(list_of(point_type(3, 5))(point_type(1, 5))(point_type(1, 3)));

		test_result_face_to_polygon(voronoi, boundary_polygon, list_of(result_polygon1)(result_polygon2)(result_polygon3)(result_polygon4)(result_polygon5));
	}

	void test_face_to_polygon3(const voronoi_diagram_type &voronoi) {

		polygon_type boundary_polygon = make_polygon<kernel_type>(list_of(point_type(2, 0))(point_type(4, 0))(point_type(6, 2))(point_type(6, 4))(point_type(4, 6))(point_type(2, 6))(point_type(0, 4))(point_type(0, 2)));

		polygon_type result_polygon1 = make_polygon<kernel_type>(list_of(point_type(3, 1))(point_type(5, 3))(point_type(3, 5))(point_type(1, 3)));
		polygon_type result_polygon2 = make_polygon<kernel_type>(list_of(point_type(2, 0))(point_type(3, 0))(point_type(3, 1))(point_type(1, 3))(point_type(0, 3))(point_type(0, 2)));
		polygon_type result_polygon3 = make_polygon<kernel_type>(list_of(point_type(3, 0))(point_type(4, 0))(point_type(6, 2))(point_type(6, 3))(point_type(5, 3))(point_type(3, 1)));
		polygon_type result_polygon4 = make_polygon<kernel_type>(list_of(point_type(6, 3))(point_type(6, 4))(point_type(4, 6))(point_type(3, 6))(point_type(3, 5))(point_type(5, 3)));
		polygon_type result_polygon5 = make_polygon<kernel_type>(list_of(point_type(3, 6))(point_type(2, 6))(point_type(0, 4))(point_type(0, 3))(point_type(1, 3))(point_type(3, 5)));

		test_result_face_to_polygon(voronoi, boundary_polygon, list_of(result_polygon1)(result_polygon2)(result_polygon3)(result_polygon4)(result_polygon5));
	}

	void test_face_to_polygon4(const voronoi_diagram_type &voronoi) {

		polygon_type boundary_polygon = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(2.5, 0))(point_type(2.5, 1.5))(point_type(3.5, 1.5))(point_type(3.5, 0))(point_type(6, 0))(point_type(6, 2.5))(point_type(4.5, 2.5))(point_type(4.5, 3.5))(point_type(6, 3.5))(point_type(6, 6))(point_type(3.5, 6))(point_type(3.5, 4.5))(point_type(2.5, 4.5))(point_type(2.5, 6))(point_type(0, 6))(point_type(0, 3.5))(point_type(1.5, 3.5))(point_type(1.5, 2.5))(point_type(0, 2.5)));

		polygon_type result_polygon1 = make_polygon<kernel_type>(list_of(point_type(2.5, 1.5))(point_type(3.5, 1.5))(point_type(4.5, 2.5))(point_type(4.5, 3.5))(point_type(3.5, 4.5))(point_type(2.5, 4.5))(point_type(1.5, 3.5))(point_type(1.5, 2.5)));
		polygon_type result_polygon2 = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(2.5, 0))(point_type(2.5, 1.5))(point_type(1.5, 2.5))(point_type(0, 2.5)));
		polygon_type result_polygon3 = make_polygon<kernel_type>(list_of(point_type(3.5, 0))(point_type(6, 0))(point_type(6, 2.5))(point_type(4.5, 2.5))(point_type(3.5, 1.5)));
		polygon_type result_polygon4 = make_polygon<kernel_type>(list_of(point_type(6, 3.5))(point_type(6, 6))(point_type(3.5, 6))(point_type(3.5, 4.5))(point_type(4.5, 3.5)));
		polygon_type result_polygon5 = make_polygon<kernel_type>(list_of(point_type(2.5, 6))(point_type(0, 6))(point_type(0, 3.5))(point_type(1.5, 3.5))(point_type(2.5, 4.5)));

		test_result_face_to_polygon(voronoi, boundary_polygon, list_of(result_polygon1)(result_polygon2)(result_polygon3)(result_polygon4)(result_polygon5));
	}

	void test_face_to_polygon5(const voronoi_diagram_type &voronoi) {

		polygon_type boundary_polygon = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(6, 0))(point_type(6, 2))(point_type(0, 2)));

		polygon_type result_polygon1 = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(2, 0))(point_type(2, 2))(point_type(0, 2)));
		polygon_type result_polygon2 = make_polygon<kernel_type>(list_of(point_type(2, 0))(point_type(4, 0))(point_type(4, 2))(point_type(2, 2)));
		polygon_type result_polygon3 = make_polygon<kernel_type>(list_of(point_type(4, 0))(point_type(6, 0))(point_type(6, 2))(point_type(4, 2)));

		test_result_face_to_polygon(voronoi, boundary_polygon, list_of(result_polygon1)(result_polygon2)(result_polygon3));
	}

	void test() {

		try {

			{

				/*point_type p(1, 1);
				std::cout << util::make_type(p.x()).name() << std::endl;
				std::cout << util::make_type(p.x().exact().to_double()).name() << std::endl;
				//double d = (double)p.x();
				std::cout << p.x().exact().to_double() << std::endl;
*/
				vector<point_type> points = list_of(point_type(1, 1))(point_type(5, 1))(point_type(5, 5))(point_type(1, 5))(point_type(3, 3));

				delaunay_triangulation_type delaunay;
				delaunay.insert(points.begin(), points.end());

				voronoi_diagram_type voronoi(delaunay, true);
				CPPUNIT_ASSERT_EQUAL( (size_t)5, voronoi.number_of_faces() );

				test_face_to_polygon1(voronoi);
				//test_face_to_polygon2(voronoi);
				//test_face_to_polygon3(voronoi);
				//test_face_to_polygon4(voronoi);
			}

#ifdef iuyg
			{
				vector<point_type> points = list_of(point_type(1, 1))(point_type(3, 1))(point_type(5, 1));

				delaunay_triangulation_type delaunay;
				delaunay.insert(points.begin(), points.end());

				voronoi_diagram_type voronoi(delaunay, true);
				CPPUNIT_ASSERT_EQUAL( (size_t)3, voronoi.number_of_faces() );

				test_face_to_polygon5(voronoi);
			}

			{

				//std::cout << "--------------------------------------------------------------\n";
				vector<point_type> points = list_of(point_type(763801.18, 194121.76))(point_type(763819.98, 194115.71))(point_type(763838.94, 194109.64))(point_type(763819.6, 194094.93))(point_type(763838.37, 194088.79))(point_type(763776.36, 194103.68))(point_type(763801.94, 194095.32))(point_type(763766.53, 194096.45))(point_type(763785.86, 194084.9))(point_type(763811.39, 194076.6))(point_type(763824.23, 194064.06))(point_type(763755.44, 194068.47))(point_type(763789.42, 194057.37))(point_type(763816.55, 194045.93))(point_type(763759.69, 194059.17))(point_type(763772.92, 194054.9))(point_type(763820.63, 194039.45))(point_type(763768.57, 194040.58))(point_type(763781.82, 194036.17))(point_type(763796.03, 194031.6))(point_type(763738.55, 194045.17))(point_type(763755.6, 194033.91))(point_type(763795.45, 194020.93))(point_type(763817.38, 194013.83))(point_type(763800.4, 194013.94))(point_type(763729.74, 194031.98))(point_type(763770.87, 194013.28))(point_type(763728.54, 194021.66))(point_type(763775.44, 194001.26))(point_type(763805.68, 193991.36));
				//vector<point_type> points = list_of(point_type(763801, 194121))(point_type(763819, 194115))(point_type(763838, 194109))(point_type(763819, 194094))(point_type(763838, 194088))(point_type(763776, 194103))/*(point_type(763801.94, 194095.32))(point_type(763766.53, 194096.45))(point_type(763785.86, 194084.9))(point_type(763811.39, 194076.6))*/;
				//vector<point_type> points = list_of(point_type(101, 121))(point_type(119, 115))(point_type(138, 109))(point_type(119, 94))(point_type(138, 88))(point_type(76,103))/*(point_type(763801.94, 194095.32))(point_type(763766.53, 194096.45))(point_type(763785.86, 194084.9))(point_type(763811.39, 194076.6))*/;

				//std::cout.precision(3);
				//std::cout.setf( std::ios::fixed, std:: ios::floatfield );

				/*double d = 763801.18;
				double d1 = d + 0.5;

				std::cout << d << std::endl;
				std::cout << d1 << std::endl;
				std::cout << points[0] << std::endl;
				std::cout << points[0].x() << std::endl;
				std::cout << points[0].y() << std::endl;
				std::cout << points[0].hx() << std::endl;
				std::cout << points[0].hy() << std::endl;*/

				delaunay_triangulation_type delaunay;
				delaunay.insert(points.begin(), points.end());

				voronoi_diagram_type voronoi(delaunay, true);
				CPPUNIT_ASSERT_EQUAL( points.size(), voronoi.number_of_faces() );

				//polygon_type boundary_polygon = make_polygon<kernel_type>(list_of(point_type(763784, 194146))(point_type(763886, 194113))(point_type(763815, 193972))(point_type(763683, 194016)));
				//double min_x = 763776, max_x = 763838;
				//double min_y = 194088, max_y = 194121;
				//double min_x = 76, max_x = 138;
				//double min_y = 88, max_y = 121;
				double min_x = 70, max_x = 140;
				double min_y = 80, max_y = 130;
				polygon_type boundary_polygon = make_polygon<kernel_type>(list_of(point_type(min_x, min_y))(point_type(max_x, min_y))(point_type(max_x, max_y))(point_type(min_x, max_y)));

				vector<polygon_type> polygons;
				push_back(polygons).range(transform(make_iterator_range(voronoi.faces_begin(), voronoi.faces_end()), make_face_to_polygon(boundary_polygon)));
				CPPUNIT_ASSERT_EQUAL( (size_t)30, polygons.size() );

			}

#endif

		} catch(std::exception &e) {
			CPPUNIT_FAIL( e.what() ); }
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( face_to_polygon_tester );

