/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/geometry/area/geometry_area.hpp>
#include <geofis/geometry/polygon_with_holes.hpp>
#include <boost/assign/assign_reference_iterator_traits.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/ref.hpp>
#include <CGAL/Cartesian.h>
#include <gmock/gmock.h>

using namespace geofis;
using namespace CGAL;
using namespace boost;
using namespace boost::assign;
using namespace testing;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef Cartesian<double> kernel_type;
typedef Point_2<kernel_type> point_type;
typedef Polygon_2<kernel_type> polygon_type;
typedef Polygon_with_holes_2<kernel_type> polygon_with_holes_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace {

struct geometry_mock {

	MOCK_CONST_METHOD0( area, double () );
};

} // namespace

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(geometry_area_tester, test_geometry_area) {

	geometry_mock geometry_mock;
	EXPECT_CALL( geometry_mock, area() ).WillOnce(Return(4.0));

	ASSERT_DOUBLE_EQ( 4.0, get_double_geometry_area(geometry_mock) );
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(geometry_area_tester, test_polygon_area) {

	polygon_type polygon = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(2, 0))(point_type(2, 2))(point_type(0, 2))); // area = 4

	ASSERT_DOUBLE_EQ( 4.0, get_double_geometry_area(polygon) );
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(geometry_area_tester, test_polygon_with_holes_area) {

	polygon_type boundary = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(5, 0))(point_type(5, 3))(point_type(0, 3))); // area = 15

	{
		polygon_with_holes_type polygon_with_holes = make_polygon_with_holes(boundary);

		ASSERT_DOUBLE_EQ( 15.0, get_double_geometry_area(polygon_with_holes) );
	}

	{
		polygon_type hole1 = make_polygon<kernel_type>(list_of(point_type(1, 1))(point_type(2, 1))(point_type(2, 2))(point_type(1, 2))); // area = 1
		polygon_type hole2 = make_polygon<kernel_type>(list_of(point_type(3, 1))(point_type(4, 1))(point_type(4, 2))(point_type(3, 2))); // area = 1

		polygon_with_holes_type polygon_with_holes = make_polygon_with_holes<kernel_type>(boundary, list_of(hole1)(hole2));

		ASSERT_DOUBLE_EQ( 13.0, get_double_geometry_area(polygon_with_holes) );
	}
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(geometry_area_tester, test_geometries_area) {

	geometry_mock geometry_mock1;
	EXPECT_CALL( geometry_mock1, area() ).WillOnce(Return(1.0));

	geometry_mock geometry_mock2;
	EXPECT_CALL( geometry_mock2, area() ).WillOnce(Return(3.0));

	ASSERT_DOUBLE_EQ( 4.0, get_double_geometries_area(cref_list_of<2>(geometry_mock1)(geometry_mock2)) );
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(geometry_area_tester, test_polygons_area) {

	polygon_type polygon1 = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(2, 0))(point_type(2, 2))(point_type(0, 2))); // area = 4
	polygon_type polygon2 = make_polygon<kernel_type>(list_of(point_type(5, 0))(point_type(6, 0))(point_type(6, 1))(point_type(5, 1))); // area = 1

	ASSERT_DOUBLE_EQ( 5.0, get_double_geometries_area(list_of(polygon1)(polygon2)) );
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(geometry_area_tester, test_polygons_with_holes_area) {

	polygon_type boundary1 = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(5, 0))(point_type(5, 3))(point_type(0, 3))); // area = 15
	polygon_type hole1 = make_polygon<kernel_type>(list_of(point_type(1, 1))(point_type(2, 1))(point_type(2, 2))(point_type(1, 2))); // area = 1
	polygon_type hole2 = make_polygon<kernel_type>(list_of(point_type(3, 1))(point_type(4, 1))(point_type(4, 2))(point_type(3, 2))); // area = 1
	polygon_with_holes_type polygon_with_holes1 = make_polygon_with_holes(boundary1, list_of(hole1)(hole2)); // area = 13

	polygon_type boundary2 = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(3, 0))(point_type(3, 3))(point_type(0, 3))); // area = 9
	polygon_type hole3 = make_polygon<kernel_type>(list_of(point_type(1, 1))(point_type(2, 1))(point_type(2, 2))(point_type(1, 2))); // area = 1
	polygon_with_holes_type polygon_with_holes2 = make_polygon_with_holes(boundary2, list_of(hole3)); // area = 8

	ASSERT_DOUBLE_EQ( 21.0, get_double_geometries_area(list_of(polygon_with_holes1)(polygon_with_holes2)) );
}
