/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/rcpp/geometry/polygon_2.hpp>
#include <gmock/gmock.h>
#include <string>
#include <CGAL/Cartesian.h>
#include <boost/assign/list_of.hpp>

using namespace testing;
using namespace std;
using namespace Rcpp;
using namespace geofis;
using namespace CGAL;
using namespace boost::assign;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef Cartesian<double> kernel_type;
typedef Point_2<kernel_type> point_type;
typedef Polygon_2<kernel_type> polygon_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace {

S4 make_wkt_rcpp_polygon(string wkt) {
	Function read_wkt("readWKT");
	S4 spatial_polygons = read_wkt(wkt);
	List polygons_list = spatial_polygons.slot("polygons");
	S4 polygons = polygons_list[0];
	List polygon_list = polygons.slot("Polygons");
	return polygon_list(0);
}

} // namespace

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(polygon_2_tester, test_polygon_2) {

	S4 rcpp_polygon = make_wkt_rcpp_polygon("POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))");
	polygon_type polygon = make_polygon_2<kernel_type>(rcpp_polygon);

	polygon_type expected_polygon = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(1, 0))(point_type(1, 1))(point_type(0, 1)));

	ASSERT_THAT(polygon, Eq(expected_polygon));
}
