/**
 * GeoFIS Native Library
 *
 * Copyright (C) 2013 IRSTEA INRA SupAgro
 *
 * IDDN.FR.001.390016.000.R.P.2013.000.31235
 *
 * Auteurs/Authors:
 *     Jean-luc Lablée - IRSTEA
 *     Serge Guillaume - IRSTEA
 *     Brigitte Charnomordic - INRA
 *     Anne Tireau - INRA
 *     Bruno Tisseyre - SupAgro
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
#include <geofis/rcpp/geometry/polygons.hpp>
#include <gmock/gmock.h>
#include <CGAL/Cartesian.h>
#include <boost/assign/list_of.hpp>
#include <rcpp/rcpp_matchers.hpp>
#include <geofis/geometry/polygon.hpp>
#include <geofis/geometry/polygon_with_holes.hpp>

#include <util/type.hpp>

using namespace testing;
using namespace std;
using namespace Rcpp;
using namespace geofis;
using namespace CGAL;
using namespace boost;
using namespace boost::assign;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef Cartesian<double> kernel_type;
typedef Point_2<kernel_type> point_type;
typedef Polygon_2<kernel_type> polygon_type;
typedef Polygon_with_holes_2<kernel_type> polygon_with_holes_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace {

S4 make_wkt_rcpp_polygons(string wkt, string id) {
	Function read_wkt("readWKT");
	S4 spatial_polygons = read_wkt(wkt);
	List polygons_list = spatial_polygons.slot("polygons");
	S4 polygons = polygons_list[0];
	List polygon_list = polygons.slot("Polygons");
	polygons.slot("ID") = id;
	return polygons;
}

} // namespace

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(polygons_tester, test_polygons) {

	polygon_type polygon = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(1, 0))(point_type(1, 1))(point_type(0, 1)));
	S4 rcpp_polygons = make_rcpp_polygons(polygon, "1");

	S4 expected_rcpp_polygons = make_wkt_rcpp_polygons("POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))", "1");

	ASSERT_THAT(rcpp_polygons, Identical(expected_rcpp_polygons));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(polygons_tester, test_polygons_list) {

	polygon_type polygon1 = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(1, 0))(point_type(1, 1))(point_type(0, 1)));
	polygon_type polygon2 = make_polygon<kernel_type>(list_of(point_type(1, 0))(point_type(2, 0))(point_type(2, 1))(point_type(1, 1)));
	polygon_type polygon3 = make_polygon<kernel_type>(list_of(point_type(2, 0))(point_type(3, 0))(point_type(3, 1))(point_type(2, 1)));
	List rcpp_polygons_list = make_rcpp_polygons_list(list_of(polygon1)(polygon2)(polygon3));

	S4 expected_rcpp_polygons1 = make_wkt_rcpp_polygons("POLYGON((0 0, 0 1, 1 1, 1 0, 0 0))", "1");
	S4 expected_rcpp_polygons2 = make_wkt_rcpp_polygons("POLYGON((1 0, 1 1, 2 1, 2 0, 1 0))", "2");
	S4 expected_rcpp_polygons3 = make_wkt_rcpp_polygons("POLYGON((2 0, 2 1, 3 1, 3 0, 2 0))", "3");
	List expected_rcpp_polygons_list = { expected_rcpp_polygons1, expected_rcpp_polygons2, expected_rcpp_polygons3 };

	Function identical("identical");
	ASSERT_TRUE(as<bool>(identical(rcpp_polygons_list, expected_rcpp_polygons_list)));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

TEST(polygons_tester, test_polygons_with_holes) {

	polygon_type boundary = make_polygon<kernel_type>(list_of(point_type(0, 0))(point_type(5, 0))(point_type(5, 3))(point_type(0, 3)));
	polygon_type hole1 = make_polygon<kernel_type>(list_of(point_type(1, 1))(point_type(2, 1))(point_type(2, 2))(point_type(1, 2)));
	polygon_type hole2 = make_polygon<kernel_type>(list_of(point_type(3, 1))(point_type(4, 1))(point_type(4, 2))(point_type(3, 2)));
	polygon_with_holes_type polygon_with_holes = make_polygon_with_holes(boundary, list_of(hole1)(hole2));
	S4 rcpp_polygons_with_holes = make_rcpp_polygons(polygon_with_holes, "1");

	S4 expected_rcpp_polygons_with_holes = make_wkt_rcpp_polygons("POLYGON((0 0, 5 0, 5 3, 0 3, 0 0), (1 1, 2 1, 2 2, 1 2, 1 1), (3 1, 4 1, 4 2, 3 2, 3 1))", "1");

	ASSERT_THAT(rcpp_polygons_with_holes, Identical(expected_rcpp_polygons_with_holes));
}
