/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/double/double_as_int.h>
#include <cassert>
#include <cstdlib>

#include <iostream>

namespace util {

template <> bool is_nan<float>(float x) {

	assert(sizeof(float) == sizeof(int32_t));
    // A NAN has an exponent of 8 bits (shifted left 23 bits) and a non-zero mantissa.
    int32_t exponent = *(int32_t*)&x & 0x7F800000;
    int32_t mantissa = *(int32_t*)&x & 0x007FFFFF;
    return ((exponent == 0x7F800000) && (mantissa != 0)) ? true : false;
}

template <> bool is_nan<double>(double x) {

	assert(sizeof(double) == sizeof(int64_t));
    // A NAN has an exponent of 11 bits (shifted left 52 bits) and a non-zero mantissa.
	int64_t exponent = *(int64_t*)&x & 0x7FF0000000000000LL;
	int64_t mantissa = *(int64_t*)&x & 0x000FFFFFFFFFFFFFLL;
    return ((exponent == 0x7FF0000000000000LL) && (mantissa != 0)) ? true : false;
}

template <> bool is_nan<long double>(long double x) {

	assert(sizeof(long double) == 12);
	// A NAN has an exponent of 15 bits (shifted left 64 bits) and a non-zero mantissa.
	union{ long double ld; struct{ uint64_t mantissa; int16_t exponent; }parts; } uld = {x};
	return ((uld.parts.exponent == 0x7FFF) && ((uld.parts.mantissa & 0x7FFFFFFFFFFFFFFFLL) != 0)) ? true : false;
}

template <> bool is_infinite<float>(float x) {

	assert(sizeof(float) == sizeof(int32_t));
    // A NAN has an exponent of 8 bits (shifted left 23 bits) and a zero mantissa. There are two infinities - positive and negative.
    return ((*(int32_t*)&x & 0x7FFFFFFF) == 0x7F800000) ? true : false;
}

template <> bool is_infinite<double>(double x) {

	assert(sizeof(double) == sizeof(int64_t));
    // A NAN has an exponent of 11 bits (shifted left 52 bits) and a zero mantissa. There are two infinities - positive and negative.
    return ((*(int64_t*)&x & 0x7FFFFFFFFFFFFFFFLL) == 0x7FF0000000000000LL) ? true : false;
}

template <> bool is_infinite<long double>(long double x) {

	assert(sizeof(long double) == 12);
	// A NAN has an exponent of 15 bits (shifted left 64 bits) and a zero mantissa. There are two infinities - positive and negative.
	union{ long double ld; struct{ uint64_t mantissa; int16_t exponent; }parts; } uld = {x};
	return (((uld.parts.exponent & 0x7FFF) == 0x7FFF) && ((uld.parts.mantissa & 0x7FFFFFFFFFFFFFFFLL) == 0)) ? true : false;
}

template <class T> static bool assert_values(T x, T y) {

	if(is_nan(x) || is_nan(y))
		return false;
    if(is_infinite(x) || is_infinite(y))
        return x == y;
	return true;
}

template <> bool is_equal<float>(float x, float y, int64_t max) {

	assert(sizeof(float) == sizeof(int32_t));

	if(is_nan(x) || is_nan(y))
		return false;

    if(is_infinite(x) || is_infinite(y))
        return x == y;

    int32_t x_int = *(int32_t*)&x;
    if(x_int < 0) // Make x_int lexicographically ordered as a twos-complement int32_t
    	x_int = 0x80000000 - x_int;

    int32_t y_int = *(int32_t*)&y;
    if (y_int < 0) // Make y_int lexicographically ordered as a twos-complement int32_t
    	y_int = 0x80000000 - y_int;

    bool equal = std::abs(x_int - y_int) < max;
    //std::cout << "diff " << std::abs(x_int - y_int) << " max " << max << std::endl;
    //std::cout << "is_equal " << x << " " << y << " " << std::hex << x_int << " " << y_int << " " << equal << std::endl;
    return equal;
}

template <> bool is_equal<double>(double x, double y, int64_t max) {

	assert(sizeof(double) == sizeof(int64_t));

	if(is_nan(x) || is_nan(y))
		return false;

    if(is_infinite(x) || is_infinite(y))
        return x == y;

    int64_t x_int = *(int64_t*)&x;
    if(x_int < 0) // Make x_int lexicographically ordered as a twos-complement int32_t
    	x_int = 0x8000000000000000LL - x_int;

    int64_t y_int = *(int64_t*)&y;
    if (y_int < 0) // Make y_int lexicographically ordered as a twos-complement int32_t
    	y_int = 0x8000000000000000LL - y_int;

    bool equal = std::abs(x_int - y_int) < (int64_t)max;
    //std::cout << "diff " << std::abs(x_int - y_int) << " max " << max << std::endl;
    //std::cout << "is_equal(" << x << ", " << y << ") " << std::hex << x_int << " " << y_int << " " << equal << std::endl;
    return equal;
}

template <> bool is_equal<long double>(long double x, long double y, int64_t max) {

	assert(sizeof(long double) == 12);

	if(is_nan(x) || is_nan(y))
		return false;

    if(is_infinite(x) || is_infinite(y))
        return x == y;

	union{ long double ld; struct{ int64_t mantissa; int16_t exponent; }parts; } x_uld = {x};
	union{ long double ld; struct{ int64_t mantissa; int16_t exponent; }parts; } y_uld = {y};

	//if(x_uld.parts.exponent < 0)
	//	x_uld.parts.exponent = 0x8000 - x_uld.parts.exponent;
	if(x_uld.parts.mantissa < 0)
		x_uld.parts.mantissa = 0x8000000000000000LL - x_uld.parts.mantissa;
	//if(y_uld.parts.exponent < 0)
	//	y_uld.parts.exponent = 0x8000 - y_uld.parts.exponent;
	if(y_uld.parts.mantissa < 0)
		y_uld.parts.mantissa = 0x8000000000000000LL - y_uld.parts.mantissa;

	bool equal = (x_uld.parts.exponent == y_uld.parts.exponent) && (std::abs(x_uld.parts.mantissa - y_uld.parts.mantissa) < max);
    //std::cout << "diff " << std::abs(x_uld.parts.mantissa - y_uld.parts.mantissa) << " max " << max << std::endl;
    //std::cout << "is_equal(" << x << ", " << y << ") \"" << std::hex << x_uld.parts.exponent << " " << x_uld.parts.mantissa << "\" \"" << y_uld.parts.exponent << " " << y_uld.parts.mantissa << "\" " << equal << std::endl;
    return equal;
}

} // namespace util
