/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/file/codecvt/fancier_codecvt.h>

namespace util {

//
// http://www.ddj.com/184403638
// http://www.drdobbs.com/unicode-files/184403638?pgno=4
//

static void set_state(mbstate_t &state, unsigned char value) {
	*reinterpret_cast<unsigned char *>(&state) = value;
}

static bool is_state(const mbstate_t &state, unsigned char value) {
	return reinterpret_cast<const unsigned char &>(state) == value;
}

std::codecvt_base::result fancier_codecvt::do_in(_St& _State, const _To *_F1, const _To *_L1, const _To *& _Mid1, _E *_F2, _E *_L2, _E *& _Mid2) const {

	_Mid1 = _F1, _Mid2 = _F2;
    _St _Mystate = _State;
    result _Ans = ok;
    for (; ; )
        if (_Mid1 == _L1 || _Mid2 == _L2)
            break;
        else if (_Mid1 + 1 == _L1)
            {_Ans = partial;
            break; }
        else if (is_state(_Mystate, 0))
            {*_Mid2 = *_Mid1++ & 0xff;
            *_Mid2 |= *_Mid1++ << 8;
            if (*_Mid2 == 0xfeff)
                _Ans = partial;
            else if (*_Mid2 == 0xfffe)
                {_Ans = partial;
                set_state(_Mystate, 1); }
            else
                {++_Mid2;
                _Ans = ok;
                break; }}
        else
            {*_Mid2 = *_Mid1++ << 8;
            *_Mid2 |= *_Mid1++ & 0xff;
            if (*_Mid2 == 0xfeff)
                _Ans = partial;
            else if (*_Mid2 == 0xfffe)
                {_Ans = partial;
                set_state(_Mystate, 0); }
            else
                {++_Mid2;
                _Ans = ok;
                break; }}
    if (_Ans == partial)
        _Mid1 = _F1, _Mid2 = _F2;
    _State = _Mystate;
    return (_Ans);
}

std::codecvt_base::result fancier_codecvt::do_out(_St& _State, const _E *_F1, const _E *_L1, const _E *& _Mid1, _To *_F2, _To *_L2, _To *& _Mid2) const {

	_Mid1 = _F1, _Mid2 = _F2;
    if (_Mid1 == _L1 || _Mid2 == _L2)
        return (ok);
    else if (_Mid2 + 1 == _L2)
        return (partial);
    else if (*_Mid1 == 0xfeff)
        {set_state(_State, 0);
        ++_Mid1;
        *_Mid2++ = 0xfe;
        *_Mid2++ = 0xff;
        return (ok); }
    else if (*_Mid1 == 0xfffe)
        {set_state(_State, 1);
        ++_Mid1;
        *_Mid2++ = 0xff;
        *_Mid2++ = 0xfe;
        return (ok); }
    else if (is_state(_State, 0))
        {*_Mid2++ = (unsigned char)*_Mid1;
        *_Mid2++ = (unsigned char)(*_Mid1++ >> 8);
        return (ok); }
    else
        {*_Mid2++ = (unsigned char)(*_Mid1 >> 8);
        *_Mid2++ = (unsigned char)*_Mid1++;
        return (ok); }
}

} // namespace util

