/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef COUPLING_LOADER_HPP_
#define COUPLING_LOADER_HPP_

#include <boost/range/combine.hpp>
#include <boost/range/adaptor/transformed.hpp>
#include <util/data/loader_data_getter.hpp>
#include <util/data/detail/function_equality.hpp>

namespace util {

template <class Loader1, class Loader2, class Function> class coupling_loader {

	typedef boost::tuple<typename loader_data_traits<Loader1>::loader_const_iterator_type, typename loader_data_traits<Loader2>::loader_const_iterator_type> tuple_of_datas;
    typedef boost::range::combined_range<tuple_of_datas> combined_range_type;

	struct transform_function {

		typedef typename Function::result_type result_type;

		transform_function(const Function &function) : function(function) {}

		template <class Tuple> inline result_type operator() (const Tuple &tuple) const { return function(tuple.template get<0>(), tuple.template get<1>()); }

		const Function &function;
	};

public:
	typedef boost::transformed_range<transform_function, const combined_range_type> iterator_range;

	coupling_loader(const Loader1 &loader1, const Loader2 &loader2, const Function &function) : loader1(loader1), loader2(loader2), function(function) {}

	iterator_range range() const {
		return boost::adaptors::transform(boost::combine(get_data(loader1), get_data(loader2)), transform_function(function));
	}

	friend bool operator== (const coupling_loader &lhs, const coupling_loader &rhs) { return (lhs.loader1 == rhs.loader1) && (lhs.loader2 == rhs.loader2) && detail::function_equality<Function>::equal(lhs.function, rhs.function); }

private:
	Loader1 loader1;
	Loader2 loader2;
	Function function;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Loader1, class Loader2, class Function> struct loader_data_traits<coupling_loader<Loader1, Loader2, Function> > {

	typedef typename coupling_loader<Loader1, Loader2, Function>::iterator_range type;
};

template <class Loader1, class Loader2, class Function> typename coupling_loader<Loader1, Loader2, Function>::iterator_range get_data(const coupling_loader<Loader1, Loader2, Function> &loader) {
	return loader.range();
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Loader1, class Loader2, class Function> inline coupling_loader<Loader1, Loader2, Function> make_coupling_loader(const Loader1 &loader1, const Loader2 &loader2, const Function &function) {
	return coupling_loader<Loader1, Loader2, Function>(loader1, loader2, function);
}

} // namespace util

#endif /* COUPLING_LOADER_HPP_ */
