/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef DATA_HPP_
#define DATA_HPP_

#include <boost/utility.hpp>
#include <boost/range.hpp>
#include <boost/range/algorithm/equal.hpp>
#include <util/object/object_collection.hpp>

namespace util {

template <class T> class data : public boost::noncopyable {

	typedef object_collection<T> collection_type;

public:
	typedef std::unique_ptr<data<T> > unique_ptr;
	typedef typename collection_type::size_type size_type;
	typedef typename collection_type::value_type value_type;
	typedef typename collection_type::const_iterator iterator;
	typedef typename collection_type::const_iterator const_iterator;
	typedef typename collection_type::reference reference;
	typedef typename collection_type::const_reference const_reference;

	virtual ~data() {}

	data *clone() const;

	//reference operator[] (size_type n) { return datas[n]; }
	const_reference operator[] (size_type n) const { return datas[n]; }

    size_t size() const { return datas.size(); }

    const_iterator begin() const { return datas.begin(); }
    const_iterator end() const { return datas.end(); }

    void set_name(const char *name) { this->name = name; }
    const char *get_name() const { return name.c_str(); }

    friend bool operator== (const data &lhs, const data &rhs) {
    	return (typeid(lhs) == typeid(rhs)) && (lhs.datas == rhs.datas) && lhs.is_equal(rhs);
    }

    template <class Visitor> typename Visitor::result_type apply_visitor(Visitor& visitor) const { return visitor(*this); }

protected:
	data(const data &other) : datas(other.datas) {}
	template <class Range> data(const Range &range) : datas(range) {}
	template <class Range, class Name> data(const Range &range, const Name &name) : datas(range), name(name) {}

	template <class Range> void assign(const Range &range) { datas.assign(range); }

private:
	collection_type datas;
	std::string name;

	virtual data *do_clone() const = 0;
	virtual bool is_equal(const data &other) const = 0;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T> inline data<T> *new_clone(const data<T> &object) { return object.clone(); }

template <class T> data<T> *data<T>::clone() const  {
	data *new_data = do_clone();
	BOOST_ASSERT(typeid(*new_data) == typeid(*this) && "do_clone incorrectly overridden");
	return new_data;
}

} // namespace util

#endif /* DATA_HPP_ */
