/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef DATASET_HPP_
#define DATASET_HPP_

#include <util/data/dataset/raw_storage.hpp>
#include <util/data/dataset/variant_storage.hpp>
#include <boost/range/algorithm/for_each.hpp>
#include <boost/assign/list_inserter.hpp>
#include <boost/range/combine.hpp>

namespace util {

template <class T, template <class U> class StoragePolicy = raw_storage> class dataset {

	typedef StoragePolicy<T> storage_type;

public:
	typedef std::unique_ptr<dataset<T, StoragePolicy> > unique_ptr;
	typedef typename storage_type::size_type size_type;
	typedef typename storage_type::value_type value_type;
	typedef typename storage_type::iterator iterator;
	typedef typename storage_type::const_iterator const_iterator;
	typedef typename storage_type::reference reference;
	typedef typename storage_type::const_reference const_reference;

	dataset() {}
	dataset(const dataset &other) : datas(other.datas), name(other.name) {};
	template <class LoaderRange> dataset(const LoaderRange &loader_range) { push_back_range(loader_range); }
	template <class LoaderRange, class NameRange> dataset(const LoaderRange &loader_range, const NameRange &name_range) { push_back_range(loader_range, name_range); }

	template <class LoaderRange> void push_back_range(const LoaderRange &loader_range) { boost::assign::push_back(datas).range(loader_range); }
	template <class LoaderRange, class NameRange> void push_back_range(const LoaderRange &loader_range, const NameRange &name_range) { boost::assign::push_back(datas).range(boost::combine(loader_range, name_range)); }

	template <class Loader> void push_back(const Loader &loader) { datas.push_back(loader); }
	template <class Loader, class Name> void push_back(const Loader &loader, const Name &name) { datas.push_back(loader, name); }

	iterator begin() { return datas.begin(); }
	iterator end() { return datas.end(); }
	const_iterator begin() const { return datas.begin(); }
	const_iterator end() const { return datas.end(); }

	reference operator[] (size_type n) { return datas[n]; }
	const_reference operator[] (size_type n) const { return datas[n]; }

	size_t size() const { return datas.size(); }
	size_t data_size() const { return datas.data_size(); }
	bool empty() const { return datas.empty(); }

	dataset &operator= (dataset other)  {
		other.swap(*this);
		return *this;
	}

	void swap(dataset &other) {
		datas.swap(other.datas);
		name.swap(other.name);
	}

    void set_name(const char *name) { this->name = name; }
    const char *get_name() const { return name.c_str(); }

	friend bool operator== (const dataset &lhs, const dataset &rhs) { return (lhs.datas == rhs.datas) && (lhs.name == rhs.name); }

    template <class Visitor> Visitor &apply_data_visitor(Visitor &visitor) const { return boost::for_each(datas, make_visitor(visitor)).visitor; }

	template <class OtherT> struct apply {

		typedef dataset<OtherT, StoragePolicy> type;
	};

private:
	storage_type datas;
	std::string name;

	template <class Visitor> struct internal_apply_data_visitor : public boost::static_visitor<void> {

		internal_apply_data_visitor(Visitor &visitor) : visitor(visitor) {}

		template <class U> void operator() (const data<U> &data) { data.template apply_visitor(visitor); }
		template <class U> void operator() (const boost::shared_ptr<data<U> > &data) { data->template apply_visitor(visitor); }
		template <BOOST_VARIANT_ENUM_PARAMS(class U)> void operator() (const boost::variant<BOOST_VARIANT_ENUM_PARAMS(U)> &variant) { boost::apply_visitor(*this, variant); }

		Visitor &visitor;
	};

	template <class Visitor> static inline internal_apply_data_visitor<Visitor> make_visitor(Visitor &visitor) {
		return internal_apply_data_visitor<Visitor>(visitor);
	}
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, template <class U> class StoragePolicy> inline dataset<T, StoragePolicy> *new_clone(const dataset<T, StoragePolicy> &x) { return new dataset<T, StoragePolicy>(x); }

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, template <class U> class StoragePolicy> struct store_as_pointer<dataset<T, StoragePolicy> > : boost::true_type {};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class LoaderRange> struct dataset_traits {

	typedef typename loader_traits<typename boost::range_value<LoaderRange>::type>::value_type value_type;
	typedef typename dataset<value_type>::unique_ptr unique_ptr;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class LoaderRange> typename dataset_traits<LoaderRange>::unique_ptr new_dataset(const LoaderRange &loader_range) {
	return typename dataset_traits<LoaderRange>::unique_ptr(new dataset<typename dataset_traits<LoaderRange>::value_type>(loader_range));
}

template <class LoaderRange, class NameRange> typename dataset_traits<LoaderRange>::unique_ptr new_dataset(const LoaderRange &loader_range, const NameRange &name_range) {
	return typename dataset_traits<LoaderRange>::unique_ptr(new dataset<typename dataset_traits<LoaderRange>::value_type>(loader_range, name_range));
}

template <class T, class LoaderRange> typename dataset<T>::unique_ptr new_dataset_type(const LoaderRange &loader_range) {
	return typename dataset<T>::unique_ptr(new dataset<T>(loader_range));
}

template <class T, class LoaderRange, class NameRange> typename dataset<T>::unique_ptr new_dataset_type(const LoaderRange &loader_range, const NameRange &name_range) {
	return typename dataset<T>::unique_ptr(new dataset<T>(loader_range, name_range));
}

template <class T, template <class U> class StoragePolicy, class LoaderRange> typename dataset<T, StoragePolicy>::unique_ptr new_dataset_storage(const LoaderRange &loader_range) {
	return typename dataset<T, StoragePolicy>::unique_ptr(new dataset<T, StoragePolicy>(loader_range));
}

template <class T, template <class U> class StoragePolicy, class LoaderRange, class NameRange> typename dataset<T, StoragePolicy>::unique_ptr new_dataset_storage(const LoaderRange &loader_range, const NameRange &name_range) {
	return typename dataset<T, StoragePolicy>::unique_ptr(new dataset<T, StoragePolicy>(loader_range, name_range));
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Visitor, class Dataset> inline Visitor &apply_data_visitor(Visitor& visitor, Dataset& dataset) {
    return dataset.apply_data_visitor(visitor);
}

template <class Visitor, class Dataset> inline const Visitor &apply_data_visitor(const Visitor& visitor, Dataset& dataset) {
    return dataset.apply_data_visitor(visitor);
}

} // namespace util

#endif /* DATASET_HPP_ */
