/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef VARIANT_STORAGE_HPP_
#define VARIANT_STORAGE_HPP_

#include <boost/mpl/assert.hpp>
#include <boost/mpl/is_sequence.hpp>
#include <util/object/object_collection.hpp>
#include <util/data/data_loader.hpp>
#include <util/data/dataset/variant_storage_traits.hpp>
#include <util/data/dataset/check_data.hpp>
#include <boost/tuple/tuple.hpp>

namespace util {

template <class T> class variant_storage {

	BOOST_MPL_ASSERT(( boost::mpl::is_sequence<T> ));

	typedef typename variant_storage_traits<T>::data_type data_type;
	typedef object_collection<data_type> storage_type;

public:
	typedef typename storage_type::size_type size_type;
	typedef typename storage_type::value_type value_type;
	typedef typename storage_type::iterator iterator;
	typedef typename storage_type::const_iterator const_iterator;
	typedef typename storage_type::reference reference;
	typedef typename storage_type::const_reference const_reference;

	variant_storage() {}
	variant_storage(const variant_storage &other) : datas(other.datas) {};

	template <class Loader> void push_back(const Loader &loader) { datas.push_back(check_data(*this, new_data(loader))); }
	template <class Loader, class Name> void push_back(const Loader &loader, const Name &name) { datas.push_back(check_data(*this, new_data(loader, name))); }
	template <class Loader, class Name> void push_back(const boost::tuples::cons<Loader, boost::tuples::cons<Name, boost::tuples::null_type> > &tuple) { datas.push_back(check_data(*this, new_data(tuple.template get<0>(), tuple.template get<1>()))); }

	iterator begin() { return datas.begin(); }
	iterator end() { return datas.end(); }
	const_iterator begin() const { return datas.begin(); }
	const_iterator end() const { return datas.end(); }

	reference back() { return datas.back(); }
	const_reference back() const { return datas.back(); }

	reference operator[] (size_type n) { return datas[n]; }
	const_reference operator[] (size_type n) const { return datas[n]; }

	size_t size() const { return datas.size(); }
	size_t data_size() const { return empty() ? 0 : back().size(); }
	bool empty() const { return datas.empty(); }

	variant_storage &operator= (variant_storage other)  {
		other.swap(*this);
		return *this;
	}

	void swap(variant_storage &other) { datas.swap(other.datas); }

	friend bool operator== (const variant_storage &lhs, const variant_storage &rhs) { return lhs.datas == rhs.datas; }

private:
	storage_type datas;

	template <class Loader> static inline typename data_loader<Loader, T>::unique_ptr new_data(const Loader &loader) { return new_data_type<T>(loader); }
	template <class Loader, class Name> static inline typename data_loader<Loader, T>::unique_ptr new_data(const Loader &loader, const Name &name) { return new_data_type<T>(loader, name); }
};

} // namespace util

#endif /* VARIANT_STORAGE_HPP_ */
