/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef DATASET_MATRIX_ADAPTOR_HPP_
#define DATASET_MATRIX_ADAPTOR_HPP_

#include <util/data/dataset.hpp>
#include <util/data/dataset_matrix_adaptor_traits.hpp>
#include <boost/version.hpp>
#include <boost/range/iterator_range.hpp>
#if BOOST_VERSION == 106400
#include <boost/serialization/array_wrapper.hpp>
#endif
#include <boost/numeric/ublas/matrix.hpp>

namespace util {

template <class T, template <class U> class StoragePolicy = raw_storage, class IsSequence = typename boost::mpl::is_sequence<T>::type> class dataset_matrix_adaptor;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, template <class U> class StoragePolicy> class dataset_matrix_adaptor_impl : public boost::numeric::ublas::matrix_expression<dataset_matrix_adaptor<T, StoragePolicy> > {

public:
    typedef dataset_matrix_adaptor<T, StoragePolicy> self_type;
    typedef self_type 		closure_type;
    typedef const self_type const_closure_type;

    typedef size_t		size_type;
    typedef ptrdiff_t	difference_type;

	typedef dataset<T, StoragePolicy> dataset_type;

    typedef typename dataset_matrix_adaptor_traits<T, StoragePolicy>::value_type 		value_type;
    typedef typename dataset_matrix_adaptor_traits<T, StoragePolicy>::reference 		reference;
    typedef typename dataset_matrix_adaptor_traits<T, StoragePolicy>::const_reference 	const_reference;

    typedef boost::numeric::ublas::indexed_const_iterator1<self_type, boost::numeric::ublas::dense_random_access_iterator_tag> const_iterator1;
    typedef boost::numeric::ublas::indexed_const_iterator2<self_type, boost::numeric::ublas::dense_random_access_iterator_tag> const_iterator2;

    typedef boost::iterator_range<const_iterator1> const_range1;
    typedef boost::iterator_range<const_iterator2> const_range2;

    typedef boost::numeric::ublas::column_major 	storage_category;
    typedef boost::numeric::ublas::column_major_tag orientation_category;

	dataset_matrix_adaptor_impl(const dataset_type &datas) : datas(datas) {}

    size_type size1() const { return datas.data_size(); }
    size_type size2() const { return datas.size(); }

    const_iterator1 find1 (int rank, size_type i, size_type j) const { return const_iterator1(static_cast<const self_type &>(*this), i, j); }
    const_iterator2 find2 (int rank, size_type i, size_type j) const { return const_iterator2(static_cast<const self_type &>(*this), i, j); }

    const_iterator1 begin1() const { return find1(0, 0, 0); }
    const_iterator1 end1() const { return find1(0, size1(), 0); }
    const_iterator2 begin2 () const { return find2(0, 0, 0); }
    const_iterator2 end2 () const { return find2(0, 0, size2()); }

    const_range1 range1() const { return const_range1(begin1(), end1()); }
    const_range2 range2() const { return const_range2(begin2(), end2()); }

protected:
    const dataset_type &datas;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T> class dataset_matrix_adaptor<T, raw_storage, boost::mpl::false_> : public dataset_matrix_adaptor_impl<T, raw_storage> {

	typedef dataset_matrix_adaptor_impl<T, raw_storage> base_type;
	typedef typename base_type::dataset_type dataset_type;

public:
	typedef typename base_type::const_reference const_reference;
	typedef typename base_type::size_type size_type;

	dataset_matrix_adaptor(const dataset_type &datas) : base_type(datas) {}

	const_reference operator()(size_type i, size_type j) const { return base_type::datas[j][i]; }
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T> class dataset_matrix_adaptor<T, raw_storage, boost::mpl::true_> : public dataset_matrix_adaptor_impl<T, raw_storage> {

	typedef dataset_matrix_adaptor_impl<T, raw_storage> base_type;
	typedef typename base_type::dataset_type dataset_type;

public:
	typedef typename base_type::value_type value_type;
	typedef typename base_type::size_type size_type;

	dataset_matrix_adaptor(const dataset_type &datas) : base_type(datas) {}

	value_type operator()(size_type i, size_type j) const { return boost::apply_visitor(data_visitor(i), base_type::datas[j]); }

private:
    struct data_visitor : public boost::static_visitor<value_type> {

    	data_visitor(size_type i) : i(i) {}

    	template <class Data> value_type operator() (const boost::shared_ptr<Data> &data) const { return value_type((*data)[i]); }

		size_type i;
    };
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T> class dataset_matrix_adaptor<T, variant_storage, boost::mpl::true_> : public dataset_matrix_adaptor_impl<T, variant_storage> {

	typedef dataset_matrix_adaptor_impl<T, variant_storage> base_type;
	typedef typename base_type::dataset_type dataset_type;

public:
	typedef typename base_type::const_reference const_reference;
	typedef typename base_type::size_type size_type;

	dataset_matrix_adaptor(const dataset_type &datas) : base_type(datas) {}

	const_reference operator()(size_type i, size_type j) const { return base_type::datas[j][i]; }
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, template <class U> class StoragePolicy> inline dataset_matrix_adaptor<T, StoragePolicy> make_dataset_matrix_adaptor(const dataset<T, StoragePolicy> &dataset) {
	return dataset_matrix_adaptor<T, StoragePolicy>(dataset);
}

} // namespace util

#endif /* DATASET_MATRIX_ADAPTOR_HPP_ */
