/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef DATA_NAME_EXTRACTOR_HPP_
#define DATA_NAME_EXTRACTOR_HPP_

#include <boost/range.hpp>
#include <boost/tokenizer.hpp>
#include <boost/logic/tribool.hpp>
#include <boost/algorithm/string/find.hpp>
#include <boost/algorithm/string/compare.hpp>
#include <boost/algorithm/string/classification.hpp>
#include <util/range/tokenizer_range.hpp>
#include <util/data/optional_name.hpp>
#include <util/data/file/char_separator.hpp>

namespace util {

/*template <class Range> typename boost::range_reference<const Range>::type element_at(const Range &range, int index) {

}*/

/*template <class Iterator> typename std::iterator_traits<Iterator>::reference element_at(Iterator iterator, const Iterator &end, unsigned int n, typename std::iterator_traits<Iterator>::reference default_return_value) {
	return iterator == end ? default_return_value : n ? element_at(++iterator, end, n--, default_return_value) : *iterator;
	//return n ? : *iterator;
	//return n && (iterator != boost::end(range) ? toto(range, ++iterator, n--)
}*/


template <class ReturnValue, class Iterator> ReturnValue element_at(Iterator iterator, const Iterator &end, unsigned int n, const ReturnValue &default_return_value) {
	return iterator == end ? default_return_value : n ? element_at(++iterator, end, n--, default_return_value) : *iterator;
}

template <class ReturnValue, class Range> ReturnValue element_at(const Range &range, unsigned int n, const ReturnValue &default_return_value) {
	return element_at<ReturnValue>(boost::begin(range), boost::end(range), n, default_return_value);
	/*typename boost::range_iterator<Range>::type iterator = boost::begin(range);
	while(n--) {
		if(iterator != boost::end(range))
			++iterator;
		else
			return default_return_value;
	}
	return (ReturnValue)*iterator;*/
}

/*template <class Range> typename boost::range_reference<Range>::type element_at(const Range &range, unsigned int n, typename boost::range_reference<Range>::type default_return_value) {
	return element_at<typename boost::range_reference<Range>::type>(boost::begin(range), boost::end(range), n, default_return_value);
}*/

class data_name_extractor {

public:
	template <class TokenizerFunction> data_name_extractor(unsigned int column, const TokenizerFunction &tokenizer_function, boost::tribool check_data_name = boost::indeterminate, optional_name name = optional_name()) : column(column), tokenizer_function(make_tokenizer_function(tokenizer_function)), check_data_name(check_data_name), name(name) {}

	template <class Range> typename boost::range_iterator<Range>::type skip(Range &range) const {
		return (range.empty() || (check_data_name == false)) ? boost::begin(range) : next_iterator(boost::begin(range));
	}

private:
	unsigned int column;
	boost::escaped_list_separator<char> tokenizer_function;
	boost::tribool check_data_name;
	mutable optional_name name;

	template <class Iterator> Iterator next_iterator(Iterator iterator) const {
		return has_data_name(*iterator) ? store_name(iterator) : iterator;
	}

	template <class Iterator> Iterator store_name(Iterator iterator) const {
		name = extract_data_name(*iterator);
		return ++iterator;
	}

	template <class Line> optional_name extract_data_name(const Line &line) const {
		std::string test = element_at(util::make_tokenizer_range(tokenizer_function, line), column, std::string());
		return element_at(util::make_tokenizer_range(tokenizer_function, line), column, optional_name());
	}

	template <class TokenizerFunction> static boost::escaped_list_separator<char> make_tokenizer_function(const TokenizerFunction &tokenizer_function);

	template <class Range> bool has_data_name(const Range &range) const {
		return !name && (check_data_name == true) ? true : (bool)boost::find_token(range, boost::is_alpha() && !boost::is_any_of("eE"));
	}
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <> boost::escaped_list_separator<char> data_name_extractor::make_tokenizer_function(const util::char_separator<char> &tokenizer_function);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class TokenizerFunction> data_name_extractor inline make_data_name_extractor(unsigned int column, const TokenizerFunction &tokenizer_function, boost::tribool check_data_name = boost::indeterminate, optional_name name = optional_name()) {
	return data_name_extractor(column, tokenizer_function, check_data_name, name);
}

} // namespace util

#endif /* DATA_NAME_EXTRACTOR_HPP_ */
