/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef SHARED_FILE_DATA_HPP_
#define SHARED_FILE_DATA_HPP_

#include <boost/unordered_map.hpp>
#include <boost/filesystem.hpp>
#include <util/file/file_range.hpp>
#include <util/data/file/file_data.hpp>

//#include <locale>
//#include <cwchar>

/*namespace boost { namespace filesystem {

std::size_t hash_value(const path &path);

}}*/ // namespace boost::filesystem

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool operator == (const std::pair<boost::filesystem::path, mbstate_t> &lhs, const std::pair<boost::filesystem::path, mbstate_t> &rhs);

std::size_t hash_value(const mbstate_t &state);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace __gnu_cxx {

bool operator == (const encoding_state &lhs, const encoding_state &rhs);

std::size_t hash_value(const encoding_state &state);

} // namespace __gnu_cxx

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace util {

template <class CharType = char, class Traits = std::char_traits<CharType> > class shared_file_data {

public:
	typedef typename Traits::state_type	state_type;

private:
	// TODO necessary to have state_type in key_type ?
	typedef std::pair<boost::filesystem::path, state_type> key_type;
	typedef boost::unordered_map<key_type, typename file_data<CharType>::weak_ptr> file_data_container;

public:
#ifdef SWIG
	#define file_data_ptr boost::shared_ptr<file_data<CharType> >
#else
	typedef typename file_data<CharType>::shared_ptr file_data_ptr;
#endif

	static shared_file_data default_shared_file_data;

	file_data_ptr get(const boost::filesystem::path &path, const state_type &state = state_type()) {

		typename file_data_container::iterator it = files.find(key_type(path, state));
		if(it != files.end())
			return (*it).second.lock();
		else
			return register_file_data(key_type(path, state));
	}

	static file_data_ptr default_get(const boost::filesystem::path &path, const state_type &state = state_type()) { return default_shared_file_data.get(path, state); }

	size_t size() const { return files.size(); }

private:
	file_data_container files;

	file_data_ptr register_file_data(const key_type &key) {

		file_data_ptr ptr(new file_data<CharType>(file_range<CharType, Traits>(key.first, key.second)), deleter(key, *this));
		files[key] = ptr;
		return ptr;
	}

	void unregister_file_data(const key_type &key) { files.erase(key); }

	struct deleter {

		deleter(const key_type &key, shared_file_data &this_shared_buffer) : key(key), this_shared_buffer(this_shared_buffer) {}

        void operator()(file_data<CharType> *file) {

        	this_shared_buffer.unregister_file_data(key);
        	delete file;
        }

        key_type key;
        shared_file_data &this_shared_buffer;
	};
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class CharType, class Traits> shared_file_data<CharType, Traits> shared_file_data<CharType, Traits>::default_shared_file_data;

} // namespace util

#endif /* SHARED_FILE_DATA_HPP_ */
