/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef TOKENIZER_COLUMN_LOADER_HPP_
#define TOKENIZER_COLUMN_LOADER_HPP_

#include <boost/range.hpp>
#include <boost/shared_container_iterator.hpp>
#include <boost/filesystem.hpp>
#include <boost/range/adaptor/transformed.hpp>
#include <util/data/loader_data_getter.hpp>
#include <util/data/file/column_tokenizer.hpp>
#include <util/data/file/line_range.hpp>
//#include <util/data/file/data_name.hpp>
#include <util/data/file/line_is_not_empty.hpp>
#include <util/data/file/line_filter.hpp>
#include <util/data/file/shared_file_data.hpp>
#include <util/data/file/data_name_extractor.hpp>

namespace util {

typedef boost::optional<std::string> optional_name;

struct no_header {

	template <class Range> typename boost::range_iterator<Range>::type skip(Range &range) const {
		return boost::begin(range);
	}
};


template <class T, class TokenizerFunction> class tokenizer_column_loader {

	typedef line_range<boost::iterator_range<boost::shared_container_iterator<file_data<> > > > line_range_type;
	typedef line_filter<line_is_not_empty, const line_range_type> line_filter_type;

public:
	typedef boost::transformed_range<column_tokenizer<T, TokenizerFunction>, const line_filter_type> iterator_range;

	tokenizer_column_loader() : column(0) {}
	tokenizer_column_loader(const boost::filesystem::path &path, unsigned int column, const TokenizerFunction &tokenizer_function) : path(path), column(column), tokenizer_function(tokenizer_function) {}

	iterator_range range() const { return boost::adaptors::transform(make_line_filter(line_is_not_empty(), (line_range_type)make_line_range(boost::make_shared_container_range(shared_file_data<>::default_get(path))), make_data_name_extractor(column, tokenizer_function)), make_column_tokenizer<T>(path, column, tokenizer_function)); }

	friend bool operator== (const tokenizer_column_loader &lhs, const tokenizer_column_loader &rhs) { return true; }

	static const char *default_separator;

private:
	boost::filesystem::path path;
	unsigned int column;
	TokenizerFunction tokenizer_function;
	//boost::tribool check_data_name;
	//mutable optional_name name;
	//mutable data_name name1;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, class TokenizerFunction> struct loader_data_traits<tokenizer_column_loader<T, TokenizerFunction> > {

	typedef typename tokenizer_column_loader<T, TokenizerFunction>::iterator_range type;
	typedef typename type::const_iterator loader_const_iterator_type;
};

template <class T, class TokenizerFunction> typename loader_data_traits<tokenizer_column_loader<T, TokenizerFunction> >::type get_data(const tokenizer_column_loader<T, TokenizerFunction> &loader) {
	return loader.range();
}

} // namespace util

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace std {

//
// tokenizer_column_loader iterator category specializations
//
// normally the category of tokenizer_column_loader iterator_range is of type boost::detail::iterator_category_with_traversal<std::input_iterator_tag, boost::forward_traversal_tag>
// but in the actual version of boost (1.44.0) for use in a zip_iterator the category must be boost pure_traversal_tag to determine the minimum_category of the IteratorTuple
// in the svn version of boost zip_iterator minimum_traversal_category_in_iterator_tuple<IteratorTuple>::tuple_of_traversal_tags has been modified to transform the boost::detail::iterator_category_with_traversal to pure_traversal_tag
// implemented in future version of boost ? these specializations of boost::detail::iterator_traits may be removed
//

//namespace detail {

template <class T, class TokenizerFunction> struct iterator_traits<boost::transform_iterator<util::column_tokenizer<T, TokenizerFunction>, boost::filter_iterator<util::line_is_not_empty, boost::token_iterator<boost::char_separator<char>, boost::shared_container_iterator<util::file_data<> >, std::string> > > > {

	typedef boost::transform_iterator<util::column_tokenizer<T, TokenizerFunction>, boost::filter_iterator<util::line_is_not_empty, boost::token_iterator<boost::char_separator<char>, boost::shared_container_iterator<util::file_data<> >, std::string> > > iterator;

    typedef boost::forward_traversal_tag 								iterator_category;
    typedef typename std::iterator_traits<iterator>::value_type			value_type;
    typedef typename std::iterator_traits<iterator>::difference_type   	difference_type;
    typedef typename std::iterator_traits<iterator>::pointer           	pointer;
    typedef typename std::iterator_traits<iterator>::reference         	reference;
};

//} // namespace detail
} // namespace boost

#endif /* TOKENIZER_COLUMN_LOADER_HPP_ */
