/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef FANCIER_CODECVT_H_
#define FANCIER_CODECVT_H_

#include <locale>
#include <cwchar>
	
namespace util {

//
// http://www.ddj.com/184403638
// http://www.drdobbs.com/unicode-files/184403638?pgno=4
//

class fancier_codecvt : public std::codecvt<wchar_t, char, mbstate_t> {

public:
    typedef wchar_t _E;
    typedef char _To;
    typedef mbstate_t _St;

    explicit fancier_codecvt(size_t _R = 0) : std::codecvt<wchar_t, char, mbstate_t>(_R) {}

protected:
    virtual result do_in(_St& _State, const _To *_F1, const _To *_L1, const _To *& _Mid1, _E *_F2, _E *_L2, _E *& _Mid2) const;

    virtual result do_out(_St& _State, const _E *_F1, const _E *_L1, const _E *& _Mid1, _To *_F2, _To *_L2, _To *& _Mid2) const;

    virtual result do_unshift(_St& _State, _To *_F2, _To *_L2, _To *& _Mid2) const { return (ok); }

    virtual int do_length(_St& _State, const _To *_F1, const _To *_L1, size_t _N2) const throw() { return ((int)(_L1 - _F1) / 2); }

    virtual bool do_always_noconv() const throw() { return (false); }

    virtual int do_max_length() const throw() { return (2); }

    virtual int do_encoding() const throw() { return (0); }
};

} // namespace util

#endif /* FANCIER_CODECVT_H_ */
