/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef FILE_RANGE_HPP_
#define FILE_RANGE_HPP_

#include <boost/format.hpp>
#include <boost/filesystem.hpp>
#include <util/file/basic_filebuf.hpp>

namespace util {

template <class CharType = char, class Traits = std::char_traits<CharType> > class file_range {

	typedef basic_filebuf<CharType, Traits> buffer_type;
    typedef typename Traits::state_type	state_type;

public:
	typedef std::istreambuf_iterator<CharType, Traits> iterator;
	typedef iterator const_iterator;

	file_range(const boost::filesystem::path &path, const std::locale &locale = get_default_locale()) { init(path, locale); }

	file_range(const boost::filesystem::path &path, const state_type &state, const std::locale &locale = get_default_locale()) : buffer(state) { init(path, locale); }

	file_range(const boost::filesystem::path &path, const char *from_charset, const std::locale &locale = get_default_locale());

	file_range(const boost::filesystem::path &path, const char *to_charset, const char *from_charset, const std::locale &locale = get_default_locale());

	const_iterator begin() const { return const_iterator(&buffer); }
	const_iterator end() const { return const_iterator(); }

	void reset() {

		buffer.pubseekoff(0, std::ios_base::beg);
		buffer.skip_bom();
	}

private:
	mutable buffer_type buffer;

	static std::locale get_default_locale() { return std::locale::classic(); }

	void init(const boost::filesystem::path &path, const std::locale &locale) {

		if(!boost::filesystem::exists(path))
			throw std::runtime_error(boost::str(boost::format("the file %1% not exist") % path));
		buffer.open(path, std::ios::in | std::ios::binary);
		if(!buffer.is_open())
			throw boost::filesystem::filesystem_error("error opening file", path, boost::system::error_code(errno, boost::system::generic_category()));
		buffer.pubimbue(locale);
		buffer.skip_bom();
	}
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <> std::locale file_range<wchar_t>::get_default_locale();

template <> std::locale file_range<char, __gnu_cxx::encoding_char_traits<char> >::get_default_locale();

template <> file_range<char, __gnu_cxx::encoding_char_traits<char> >::file_range(const boost::filesystem::path &path, const char *from_charset, const std::locale &locale);

template <> file_range<char, __gnu_cxx::encoding_char_traits<char> >::file_range(const boost::filesystem::path &path, const char *to_charset, const char *from_charset, const std::locale &locale);

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef file_range<char, __gnu_cxx::encoding_char_traits<char> > encoding_file_range;

} // namespace util

#endif /* FILE_RANGE_HPP_ */
