/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef COMBINING_ITERATOR_HPP_
#define COMBINING_ITERATOR_HPP_

#include <vector>
#include <algorithm>
#include <boost/range/iterator_range.hpp>
#include <boost/iterator/indirect_iterator.hpp>

namespace util {

template <class Iterator> class combining_iterator;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Iterator> struct combining_iterator_traits {

	typedef std::vector<Iterator> container;
	typedef typename std::iterator_traits<Iterator>::value_type iterator_value_type;
	//typedef std::vector<iterator_value_type> value_type;
	typedef boost::indirect_iterator<typename container::const_iterator> indirect_iterator;
	typedef boost::iterator_range<indirect_iterator> value_type;

	typedef boost::iterator_facade<combining_iterator<Iterator>, value_type, std::input_iterator_tag, value_type> type;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Iterator> class combining_iterator : public combining_iterator_traits<Iterator>::type {

    typedef typename combining_iterator_traits<Iterator>::type super_type;
	typedef typename combining_iterator_traits<Iterator>::container container;
	typedef typename combining_iterator_traits<Iterator>::indirect_iterator indirect_iterator;

	friend class boost::iterator_core_access;

public:
	typedef typename container::const_iterator const_iterator;

	combining_iterator() {}
    template <class _Iterator> combining_iterator(const _Iterator &begin, const _Iterator &end) : iterators(begin ,end) {}

    const_iterator first_iterator() const { return iterators.begin(); }
    const_iterator last_iterator() const { return iterators.end(); }

private:
	inline typename super_type::reference dereference() const { return typename super_type::value_type(indirect_iterator(iterators.begin()), indirect_iterator(iterators.end())); }
    inline bool equal(const combining_iterator &other) const { return iterators == other.iterators; }

    inline void increment() { std::for_each(iterators.begin(), iterators.end(), increment_functor()); }

    struct increment_functor {

    	template <class T> inline void operator()(T &t) const { ++t; }
    };

	container iterators;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Iterator> combining_iterator<typename std::iterator_traits<Iterator>::value_type> make_combining_iterator(const Iterator &begin, const Iterator &end) {

	return combining_iterator<typename std::iterator_traits<Iterator>::value_type>(begin, end);
}

} // namespace util
#endif /*COMBINING_ITERATOR_HPP_*/
