/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef DYNAMIC_BITSET_ITERATOR_H_
#define DYNAMIC_BITSET_ITERATOR_H_
#include <iterator>
#include <boost/static_assert.hpp>
#include <boost/type_traits.hpp>
#include <boost/dynamic_bitset.hpp>

namespace util {
namespace iterator {

template <class Iterator> class DynamicBitsetIterator {
	
	BOOST_STATIC_ASSERT((boost::is_convertible<typename std::iterator_traits<Iterator>::iterator_category, std::forward_iterator_tag>::value));
	
public:
	typedef std::forward_iterator_tag									iterator_category;
    typedef typename std::iterator_traits<Iterator>::value_type 		value_type;
   	typedef typename std::iterator_traits<Iterator>::difference_type	difference_type;
    typedef typename std::iterator_traits<Iterator>::pointer 			pointer;
    typedef typename std::iterator_traits<Iterator>::reference 			reference;
	
    DynamicBitsetIterator(const Iterator &it, const Iterator &it_end, const boost::dynamic_bitset<>	&bitset, const boost::dynamic_bitset<>::size_type &start_pos) : it(it), it_end(it_end), bitset(bitset), pos(0) {
		gotoPos(start_pos);        	
    }

    DynamicBitsetIterator(const Iterator &it, const Iterator &it_end, const boost::dynamic_bitset<>	&bitset) : it(it), it_end(it_end), bitset(bitset), pos(0) {
		gotoPos(bitset.find_first());        	
    }

    DynamicBitsetIterator(const Iterator &it, const boost::dynamic_bitset<>	&bitset) : it(it), it_end(it), bitset(bitset), pos(bitset.size()) {}
    
    DynamicBitsetIterator() {}

    inline reference operator*()  const { return *it; }
    inline pointer   operator->() const { return it;  }

    DynamicBitsetIterator<Iterator>& operator++() { 
        gotoPos(bitset.find_next(pos));
    	return *this;
    }
    	
    DynamicBitsetIterator<Iterator>  operator++(int ) { 
    	DynamicBitsetIterator<Iterator> temp = *this; 
        gotoPos(bitset.find_next(pos));
    	return temp;
    }

    inline bool operator==(const DynamicBitsetIterator<Iterator>& right) const { return it == right.it; }
    inline bool operator!=(const DynamicBitsetIterator<Iterator>& right) const { return it != right.it; }
            
private:
	Iterator it;
	Iterator it_end;
	boost::dynamic_bitset<>	bitset;
	boost::dynamic_bitset<>::size_type pos;
	
	void gotoPos(boost::dynamic_bitset<>::size_type new_pos) {
        while ((it != it_end) && (pos != new_pos)) {  
        	++it;
        	pos++;
        }
	}
};

template <class Iterator> inline DynamicBitsetIterator<Iterator> makeDynamicBitsetIterator(const Iterator &it, const Iterator &it_end, const boost::dynamic_bitset<> &bitset, const boost::dynamic_bitset<>::size_type &pos) {
	return DynamicBitsetIterator<Iterator>(it, it_end, bitset, pos);
}

template <class Iterator> inline DynamicBitsetIterator<Iterator> makeDynamicBitsetIterator(const Iterator &it, const Iterator &it_end, const boost::dynamic_bitset<> &bitset) {
	return DynamicBitsetIterator<Iterator>(it, it_end, bitset);
}

template <class Iterator> inline DynamicBitsetIterator<Iterator> makeDynamicBitsetIterator(const Iterator &it, const boost::dynamic_bitset<> &bitset) {
	return DynamicBitsetIterator<Iterator>(it, bitset);
}

} // namespace iterator
} // namespace util

#endif /*DYNAMIC_BITSET_ITERATOR_H_*/
