/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef LOOPING_ITERATOR_HPP_
#define LOOPING_ITERATOR_HPP_

#include <boost/mpl/bool.hpp>  
#include <boost/iterator/iterator_adaptor.hpp>

namespace util {

template <class Iterator> class looping_iterator;

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Iterator> struct looping_iterator_traits {
	
	typedef boost::iterator_adaptor<looping_iterator<Iterator>, Iterator, boost::use_default, std::forward_iterator_tag> type;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Iterator> class looping_iterator : public looping_iterator_traits<Iterator>::type {
	
    typedef typename looping_iterator_traits<Iterator>::type super_type;

    friend class boost::iterator_core_access;

public:
	looping_iterator() {}
	looping_iterator(const Iterator &it, const Iterator &end) : super_type(it), begin(it), end(end), _carry(false) {}

	inline bool carry() { return !_carry ? _carry : !(_carry = false); } 

private:
	inline typename super_type::reference dereference() const { return super_type::base() == end ? *begin : *super_type::base(); }
    void increment();
	
	Iterator begin;
	Iterator end;
	bool _carry;
}; 

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Iterator> void looping_iterator<Iterator>::increment() {
	
	Iterator &it = super_type::base_reference();
	if(it == end) 
		it = begin;
	if(++it == end)
		_carry = true;
}
	
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Iterator> inline looping_iterator<Iterator> make_looping_iterator(const Iterator &it, const Iterator &end) {
	
	return looping_iterator<Iterator>(it, end);
}
 
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T> struct is_looping_iterator : public boost::mpl::false_ {};

template <class T> struct is_looping_iterator<looping_iterator<T> > : public boost::mpl::true_ {};

} // namespace util

#endif /*LOOPING_ITERATOR_HPP_*/
