/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef TYPE_CONVERSION_ITERATOR_HPP_
#define TYPE_CONVERSION_ITERATOR_HPP_

#include <boost/range.hpp>
#include <boost/bind/bind.hpp>
#include <boost/iterator/iterator_facade.hpp>
#include <util/iterator/rolling_iterator.hpp>

namespace util {

// forward declaration of type_conversion_iterator
template <class Iterator, class ToType, bool ToTypeGreaterIteratorType = (sizeof(ToType) > sizeof(typename std::iterator_traits<Iterator>::value_type))> struct type_conversion_iterator;

namespace detail {

template <class Iterator, class ToType, bool ToTypeGreaterIteratorType> struct type_conversion_iterator_base {

	typedef boost::iterator_facade<type_conversion_iterator<Iterator, ToType, ToTypeGreaterIteratorType>, ToType, std::input_iterator_tag, ToType, typename std::iterator_traits<Iterator>::difference_type> type;
};

} // namespace detail

template <class Iterator, class ToType> class type_conversion_iterator<Iterator, ToType, true> : public detail::type_conversion_iterator_base<Iterator, ToType, true>::type {

	friend class boost::iterator_core_access;

	typedef typename std::iterator_traits<Iterator>::value_type from_type;
	typedef typename detail::type_conversion_iterator_base<Iterator, ToType, true>::type base_type;

	enum {
		from_type_size = sizeof(from_type),
		to_type_size = sizeof(ToType),
		from_type_to_read = to_type_size / from_type_size,
	};

	BOOST_STATIC_ASSERT( from_type_to_read > 1 ); // otherwise non conversion need
	BOOST_STATIC_ASSERT( to_type_size % from_type_size == 0 ); // assert sizeof(ToType) is multiple of sizeof(from_type)

public:
	type_conversion_iterator(const Iterator &end) : iterator(end), end(end), end_reached(true), value(0) {}
	type_conversion_iterator(const Iterator &iterator, const Iterator &end) : iterator(iterator), end(end), end_reached(false), value(read_value()) {}
	type_conversion_iterator(const type_conversion_iterator &other) : iterator(other.iterator), end(other.end), end_reached(other.end_reached), value(other.value) {}

private:
	Iterator iterator;
	Iterator end;
	bool end_reached;
	union conversion {

		conversion(ToType to) : to(to) {}
		conversion(const conversion &other) : to(other.to) {}

		from_type from[from_type_to_read];
		ToType to;
	} value;

    typename base_type::reference dereference() const { return value.to; }

    void increment() { read_value(); }

    bool equal(const type_conversion_iterator& other) const { return end_reached & (iterator == other.iterator); }

    ToType read_value() {
		for(size_t i = 0; (i < from_type_to_read) && !(end_reached = (iterator == end)) ; ++i, ++iterator)
			value.from[i] = *iterator;
		return value.to;
	}
};

template <class Iterator, class ToType> class type_conversion_iterator<Iterator, ToType, false> : public detail::type_conversion_iterator_base<Iterator, ToType, false>::type {

	friend class boost::iterator_core_access;

	typedef typename std::iterator_traits<Iterator>::value_type from_type;
	typedef typename detail::type_conversion_iterator_base<Iterator, ToType, false>::type base_type;
	typedef rolling_iterator<const ToType *> to_iterator_type;

	enum {
		from_type_size = sizeof(from_type),
		to_type_size = sizeof(ToType),
		to_type_to_read = from_type_size / to_type_size,
	};

	BOOST_STATIC_ASSERT( to_type_to_read > 1 ); // otherwise non conversion need
	BOOST_STATIC_ASSERT( from_type_size % to_type_size == 0 ); // assert sizeof(ToType) is multiple of sizeof(from_type)

public:
	type_conversion_iterator(const Iterator &end) : iterator(end), value(0), to_iterator(boost::end(value.to)) {}
	type_conversion_iterator(const Iterator &iterator, const Iterator &end) : iterator(iterator), value(iterator != end ? *iterator : 0), to_iterator(boost::begin(value.to), boost::end(value.to), boost::bind(&type_conversion_iterator::increment_iterator, this)) {}
	type_conversion_iterator(const type_conversion_iterator &other) : iterator(other.iterator), value(other.value), to_iterator(copy_to_iterator(other.to_iterator)) {}

private:
	Iterator iterator;
	union conversion {

		conversion(from_type from) : from(from) {}
		conversion(const conversion &other) : from(other.from) {}

		from_type from;
		ToType to[to_type_to_read];
	} value;
	to_iterator_type to_iterator;

private:
    typename base_type::reference dereference() const { return *to_iterator; }

    void increment() { ++to_iterator; }

    bool equal(const type_conversion_iterator& other) const { return (iterator == other.iterator); }

    void increment_iterator() { value.from = *++iterator; }

    to_iterator_type copy_to_iterator(const to_iterator_type &other) { return boost::next(to_iterator_type(boost::begin(value.to), boost::end(value.to), boost::bind(&type_conversion_iterator::increment_iterator, this)), std::distance(other.begin(), other.base())); }
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class ToType, class Iterator> type_conversion_iterator<Iterator, ToType> make_type_conversion_iterator(const Iterator &iterator, const Iterator &end) {
	return type_conversion_iterator<Iterator, ToType>(iterator, end);
}

template <class ToType, class Iterator> type_conversion_iterator<Iterator, ToType> make_type_conversion_iterator(const Iterator &end) {
	return type_conversion_iterator<Iterator, ToType>(end);
}

} // namespace util

#endif /* TYPE_CONVERSION_ITERATOR_HPP_ */
