/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef ARRAY_HPP_
#define ARRAY_HPP_

#include <iostream>

#include <jni.h>
#include <boost/range.hpp>
#include <boost/move/move.hpp>

namespace util {

template <class Array> struct array_traits;

template <> struct array_traits<jcharArray> {

	typedef jchar element_type;
};

template <> struct array_traits<jintArray> {

	typedef jint element_type;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class Array> class array {

	typedef typename array_traits<Array>::element_type element_type;

	BOOST_MOVABLE_BUT_NOT_COPYABLE(array);

public:
	array(JNIEnv *jenv, Array array) : jenv(jenv), jni_array(array), elements(get_array_elements()) {}
	array(BOOST_RV_REF(array) other) : jenv(other.jenv), jni_array(other.release_array()), elements(other.release_elements()) {}
	~array() {
		if(jni_array && elements)
			release_array_elements();
	}

	static Array create(JNIEnv *jenv, jsize size);

	static array<Array> create2(JNIEnv *jenv, jsize size) {
		//std::cout << "avant create()" << std::endl;
		Array new_array(create(jenv, size));
		//std::cout << "apres create()" << std::endl;
		return new_array ? array<Array>(jenv, new_array) : throw std::bad_alloc();
	}

	jsize size() const { return jni_array ? jenv->GetArrayLength(jni_array) : 0; }

	element_type *get_elements() const { return elements; }

	boost::iterator_range<element_type *> get_range() { return boost::make_iterator_range(elements, elements + size()); }
	boost::iterator_range<const element_type *> get_range() const { return boost::make_iterator_range(elements, elements + size()); }

private:
	JNIEnv *jenv;
	Array jni_array;
	element_type *elements;

	element_type *get_array_elements();
	void release_array_elements();

	Array release_array() {
		Array tmp = jni_array;
		jni_array = NULL;
		return tmp;
	}

	element_type *release_elements() {
		element_type *tmp = elements;
		elements = NULL;
		return tmp;
	}
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <> jcharArray array<jcharArray>::create(JNIEnv *jenv, jsize size);
template <> array_traits<jcharArray>::element_type *array<jcharArray>::get_array_elements();
template <> void array<jcharArray>::release_array_elements();

template <> jintArray array<jintArray>::create(JNIEnv *jenv, jsize size);
template <> array_traits<jintArray>::element_type *array<jintArray>::get_array_elements();
template <> void array<jintArray>::release_array_elements();

} // namespace util

#endif /* ARRAY_HPP_ */
