/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef SEQUENCE_STORAGE_HPP_
#define SEQUENCE_STORAGE_HPP_

#include <memory>
#include <vector>
#include <boost/variant.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/mpl/assert.hpp>
#include <boost/mpl/is_sequence.hpp>
#include <boost/mpl/transform.hpp>
#include <boost/mpl/contains.hpp>
#include <boost/mpl/find_if.hpp>
#include <boost/mpl/if.hpp>
#include <boost/mpl/not.hpp>
#include <boost/mpl/end.hpp>
#include <boost/mpl/deref.hpp>
#include <boost/range.hpp>
#include <boost/range/algorithm/equal.hpp>
#include <boost/iterator/transform_iterator.hpp>
#include <util/object/collection/store_as_pointer.hpp>

namespace util {

template <class T> class sequence_storage {

	BOOST_MPL_ASSERT(( boost::mpl::is_sequence<T> ));

	typedef typename boost::mpl::transform<T, boost::mpl::if_<store_as_pointer<boost::mpl::_1>, boost::shared_ptr<boost::mpl::_1>, boost::mpl::_1> >::type transformed_type;
	typedef typename boost::make_variant_over<transformed_type>::type variant_type;

	typedef std::vector<variant_type> storage_type;

public:
	typedef typename storage_type::size_type size_type;
	typedef typename storage_type::value_type value_type;
	typedef typename storage_type::iterator iterator;
	typedef typename storage_type::const_iterator const_iterator;
	typedef typename storage_type::reference reference;
	typedef typename storage_type::const_reference const_reference;

	sequence_storage() {}
	sequence_storage(const sequence_storage &other) : objects(boost::make_transform_iterator(boost::begin(other.objects), copy()), boost::make_transform_iterator(boost::end(other.objects), copy())) {}
	template <class Range> sequence_storage(const Range &range) : objects(boost::make_transform_iterator(boost::begin(range), assigner<typename boost::range_value<Range>::type>()), boost::make_transform_iterator(boost::end(range), assigner<typename boost::range_value<Range>::type>())) {};

	template <class Range> void assign(const Range &range) { objects.assign(boost::make_transform_iterator(boost::begin(range), assigner<typename boost::range_value<Range>::type>()), boost::make_transform_iterator(boost::end(range), assigner<typename boost::range_value<Range>::type>())); }

	template <class U> void push_back(std::unique_ptr<U> x) { objects.push_back(convert_to_bounded_type<U>::get(x)); }

	iterator begin() { return objects.begin(); }
	iterator end() { return objects.end(); }
	const_iterator begin() const { return objects.begin(); }
	const_iterator end() const { return objects.end(); }

	reference back() { return objects.back(); }
	const_reference back() const { return objects.back(); }

	reference operator[] (size_type n) { return objects[n]; }
	const_reference operator[] (size_type n) const { return objects[n]; }

	size_t size() const { return objects.size(); }
	bool empty() const { return objects.empty(); }

	sequence_storage &operator= (sequence_storage other)  {
		other.swap(*this);
		return *this;
	}

	void swap(sequence_storage &other) { objects.swap(other.objects); }

	friend bool operator== (const sequence_storage &lhs, const sequence_storage &rhs) { return boost::equal(lhs, rhs, equality()); }

private:
	storage_type objects;

	template <class U, class ContainU = typename boost::mpl::contains<T, U>::type> struct assigner;

	template <class U> struct assigner<U, boost::mpl::true_> : public std::_Identity<U> {};

	template <class U> struct assigner<U, boost::mpl::false_> {

		typedef typename boost::mpl::find_if<T, boost::is_base_of<boost::mpl::_1, U> >::type find_type_iterator;

		BOOST_MPL_ASSERT((boost::mpl::not_<boost::is_same<find_type_iterator , boost::mpl::end<T> > >));

		typedef typename boost::mpl::deref<find_type_iterator>::type find_type;
		typedef boost::shared_ptr<find_type> result_type;

		result_type operator() (const U &x) const { return result_type(new_clone(x)); }
	};

	struct equality {

		struct equality_visitor : public boost::static_visitor<bool> {

			template <class U, class V> bool operator() (const U &, const V &) const { return false; }

			template <class U> bool operator() (const U &lhs, const U &rhs) const { return lhs == rhs; }

			template <class U> bool operator() (const boost::shared_ptr<U> &lhs, const boost::shared_ptr<U> &rhs) const {
				BOOST_ASSERT(lhs != rhs);
				return *lhs == *rhs;
			}
		};

		bool operator() (const variant_type &lhs, const variant_type &rhs) const { return boost::apply_visitor(equality_visitor(), lhs, rhs); }
	};

	struct copy : public std::unary_function<variant_type, variant_type> {

		struct copy_visitor : public boost::static_visitor<variant_type> {

			template <class U> const U &operator() (const U &x) const { return x; }

			template <class U> boost::shared_ptr<U> operator() (const boost::shared_ptr<U> &x) const { return boost::shared_ptr<U>(new_clone(*x)); }
		};

		variant_type operator() (const variant_type &x) const { return boost::apply_visitor(copy_visitor(), x); }
	};

	template <class U, class ContainU = typename boost::mpl::contains<transformed_type, U>::type, class ContainPointerU = typename boost::mpl::contains<transformed_type, boost::shared_ptr<U> >::type> struct convert_to_bounded_type;

	template <class U> struct convert_to_bounded_type<U, boost::mpl::true_, boost::mpl::false_> {

		static const U &get(std::unique_ptr<U> &x) { return *x; }
	};

	template <class U> struct convert_to_bounded_type<U, boost::mpl::false_, boost::mpl::true_> {

		typedef boost::shared_ptr<U> result_type;

		static result_type get(std::unique_ptr<U> &x) { return result_type(x); }
	};

	template <class U> struct convert_to_bounded_type<U, boost::mpl::false_, boost::mpl::false_> {

		typedef typename boost::mpl::find_if<T, boost::is_base_of<boost::mpl::_1, U> >::type find_type_iterator;

		BOOST_MPL_ASSERT((boost::mpl::not_<boost::is_same<find_type_iterator , boost::mpl::end<T> > >));

		typedef typename boost::mpl::deref<find_type_iterator>::type find_type;
		typedef boost::shared_ptr<find_type> result_type;

		static result_type get(std::unique_ptr<U> &x) { return result_type(x); }
	};
};

} // namespace util

#endif /* SEQUENCE_STORAGE_HPP_ */
