/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef OBJECT_COLLECTION_HPP_
#define OBJECT_COLLECTION_HPP_

#include <memory>
#include <util/object/collection/object_collection_traits.hpp>

namespace util {

template <class T, class StoragePolicy = typename object_collection_traits<T>::type> class object_collection {

public:
	typedef typename StoragePolicy::size_type size_type;
	typedef typename StoragePolicy::value_type value_type;
	typedef typename StoragePolicy::iterator iterator;
	typedef typename StoragePolicy::const_iterator const_iterator;
	typedef typename StoragePolicy::reference reference;
	typedef typename StoragePolicy::const_reference const_reference;

	object_collection() {}
	object_collection(const object_collection &other) : objects(other.objects) {};
	template <class Range> object_collection(const Range &range) : objects(range) {};

	template <class Range> void assign(const Range &range) { objects.assign(range); }

	template <class U> void push_back(std::unique_ptr<U> x) { objects.push_back(std::move(x)); }

	iterator begin() { return objects.begin(); }
	iterator end() { return objects.end(); }
	const_iterator begin() const { return objects.begin(); }
	const_iterator end() const { return objects.end(); }

	reference back() { return objects.back(); }
	const_reference back() const { return objects.back(); }

	reference operator[] (size_type n) { return objects[n]; }
	const_reference operator[] (size_type n) const { return objects[n]; }

	size_t size() const { return objects.size(); }
	bool empty() const { return objects.empty(); }

	object_collection &operator= (object_collection other)  {
		other.swap(*this);
		return *this;
	}

	void swap(object_collection &other) { objects.swap(other.objects); }

	friend bool operator== (const object_collection &lhs, const object_collection &rhs) { return lhs.objects == rhs.objects; }

private:
	StoragePolicy objects;
};

} // namespace util

#endif /* OBJECT_COLLECTION_HPP_ */
