/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef ELEMENT_AT_HPP_
#define ELEMENT_AT_HPP_

#include <boost/range.hpp>

#include <iostream>

namespace util {

struct throw_if_out_of_bounds {};

template <class Value> class return_default_value {

public:
	typedef Value & type;

	return_default_value(const Value &value) : value(value) {}

	Value &operator() (unsigned int) { return value; }
	const Value &operator() (unsigned int) const  { return value; }

	Value &operator* () { return value; }
	const Value &operator* () const  { return value; }

private:
	Value value;
};

template <class ReturnValue> struct return_value_traits {

	typedef ReturnValue & type;
};

/*template <class ReturnValue> struct return_value_traits<const ReturnValue> {

	typedef ReturnValue type;
};*/

template <class Value> struct return_value_traits<return_default_value<Value> > {

	typedef typename return_default_value<Value>::type type;
};

template <class Value> struct return_value_traits<const return_default_value<Value> > {

	typedef typename return_default_value<const Value>::type type;
};

template <class ReturnValue, class Iterator> typename return_value_traits<ReturnValue>::type element_at(Iterator iterator, const Iterator &end, unsigned int n, ReturnValue &default_return_value) {
	std::cout << "element_at " << (iterator == end) << " " << n << std::endl;
	return iterator == end ? default_return_value(n) : n ? element_at(++iterator, end, --n, default_return_value) : *iterator;
}

/*template <class ReturnValue, class Iterator> const ReturnValue &element_at(Iterator iterator, const Iterator &end, unsigned int n, const ReturnValue &default_return_value) {
	return iterator == end ? default_return_value : n ? element_at(++iterator, end, n--, default_return_value) : *iterator;
}*/

template <class ReturnValue, class Range> typename return_value_traits<ReturnValue>::type element_at(Range &range, unsigned int n, ReturnValue &default_return_value) {
	return element_at<ReturnValue>(boost::begin(range), boost::end(range), n, default_return_value);
}

template <class ReturnValue, class Range> typename return_value_traits<ReturnValue>::type element_at(Range &range, unsigned int n, const ReturnValue &default_return_value) {
	return element_at<const ReturnValue>(boost::begin(range), boost::end(range), n, default_return_value);
}

template <class ReturnValue, class Range> typename return_value_traits<ReturnValue>::type element_at(const Range &range, unsigned int n, const ReturnValue &default_return_value) {
	return element_at<const ReturnValue>(boost::begin(range), boost::end(range), n, default_return_value);
}

} // namespace util

#endif /* ELEMENT_AT_HPP_ */
