/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/algorithm/normalization.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/specializations.hpp>
#include <boost/assign/list_of.hpp>

using namespace util;
using namespace boost::assign;

class normalization_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( normalization_tester );

	CPPUNIT_TEST( test );

	CPPUNIT_TEST_SUITE_END();

public:

	void test() {

		try {

			std::vector<double> values = list_of<double>(1)(2)(3)(4)(5);
			std::vector<double> expected_normalized = list_of<double>(0)(0.25)(0.5)(0.75)(1);

			normalization<double> normalization(normalization<double>::initialize(values));

			std::vector<double> normalized;
			normalization.normalize(values, std::back_inserter(normalized));
			CPPUNIT_ASSERT_RANGES_EQUAL( expected_normalized, normalized );

			std::vector<double> unnormalized;
			normalization.unnormalize(normalized, std::back_inserter(unnormalized));
			CPPUNIT_ASSERT_RANGES_EQUAL( values, unnormalized );

			std::vector<double> original_values(boost::begin(values), boost::end(values));
			normalization.normalize(values);
			CPPUNIT_ASSERT_RANGES_EQUAL( expected_normalized, values );

			normalization.unnormalize(values);
			CPPUNIT_ASSERT_RANGES_EQUAL( original_values, values );

			std::vector<double> other_values = list_of<double>(2)(3)(4);
			std::vector<double> other_expected_normalized = list_of<double>(0.25)(0.5)(0.75);

			std::vector<double> other_normalized;
			normalization.normalize(other_values, std::back_inserter(other_normalized));
			CPPUNIT_ASSERT_RANGES_EQUAL( other_expected_normalized, other_normalized );

			std::vector<double> other_unnormalized;
			normalization.unnormalize(other_normalized, std::back_inserter(other_unnormalized));
			CPPUNIT_ASSERT_RANGES_EQUAL( other_values, other_unnormalized );

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( normalization_tester );
