/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/data/dataset.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/specializations.hpp>
#include <cppunit/specializations/data/data.hpp>
#include <util/data/transform_loader.hpp>
#include <util/object/numbered_name_range.hpp>
#include <util/object/numbered_name_generator.hpp>
#include <boost/mpl/vector.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/range/counting_range.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include "../object/object.h"
#include "../object/base_object.h"

using namespace util;
using namespace boost;
using namespace boost::assign;
using namespace boost::adaptors;

typedef boost::mpl::vector<int, object, base_object> sequence_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace boost {

struct int_equality : public static_visitor<bool> {

	int_equality(int value) : value(value) {}

	template <class T> bool operator() (const T &t) const { return value == t; }

	template <class T> bool operator() (const shared_ptr<T> &pointer) const { return value == *pointer; }

	int value;
};

template <BOOST_VARIANT_ENUM_PARAMS(typename T)> bool operator== (int lhs, const variant<BOOST_VARIANT_ENUM_PARAMS(T)> &rhs) {
	return apply_visitor(int_equality(lhs), rhs);
}

} // namespace boost

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef iterator_range<counting_iterator<int> > counting_range_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, class Range> struct dataset_maker;

template <class Range> struct dataset_maker<int, Range> {

	typename dataset<int>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset(list_of(range1)(range2)); }
	template <class NameRange> typename dataset<int>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset(list_of(range1)(range2), name_range); }
};

template <class Range> struct dataset_maker<object, Range> {

	typename dataset<object>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset(list_of(make_transform_loader(range1, make_object()))(make_transform_loader(range2, make_object()))); }
	template <class NameRange> typename dataset<object>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset(list_of(make_transform_loader(range1, make_object()))(make_transform_loader(range2, make_object())), name_range); }
};

template <class Range> struct dataset_maker<base_object, Range> {

	typename dataset<base_object>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset_type<base_object>(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object()))); }
	template <class NameRange> typename dataset<base_object>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset_type<base_object>(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object())), name_range); }
};

template <class Range> struct dataset_maker<derived_object, Range> {

	typename dataset<derived_object>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object()))); }
	template <class NameRange> typename dataset<derived_object>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object())), name_range); }
};

template <class T, class Range> typename dataset<T>::unique_ptr make_dataset(const Range &range1, const Range &range2) {
	return dataset_maker<T, Range>()(range1, range2);
}

template <class T, class Range, class NameRange> typename dataset<T>::unique_ptr make_dataset_name(const Range &range1, const Range &range2, const NameRange &name_range) {
	return dataset_maker<T, Range>()(range1, range2, name_range);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, class Range> struct sequence_dataset_maker;

template <class Range> struct sequence_dataset_maker<int, Range> {

	typename dataset<sequence_type>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset_type<sequence_type>(list_of(range1)(range2)); }
	template <class NameRange> typename dataset<sequence_type>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset_type<sequence_type>(list_of(range1)(range2), name_range); }
};

template <class Range> struct sequence_dataset_maker<object, Range> {

	typename dataset<sequence_type>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset_type<sequence_type>(list_of(make_transform_loader(range1, make_object()))(make_transform_loader(range2, make_object()))); }
	template <class NameRange> typename dataset<sequence_type>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset_type<sequence_type>(list_of(make_transform_loader(range1, make_object()))(make_transform_loader(range2, make_object())), name_range); }
};

template <class Range> struct sequence_dataset_maker<base_object, Range> {

	typename dataset<sequence_type>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset_type<sequence_type>(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object()))); }
	template <class NameRange> typename dataset<sequence_type>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset_type<sequence_type>(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object())), name_range); }
};

template <class T, class Range> typename dataset<sequence_type>::unique_ptr make_sequence_dataset(const Range &range1, const Range &range2) {
	return sequence_dataset_maker<T, Range>()(range1, range2);
}

template <class T, class Range, class NameRange> typename dataset<sequence_type>::unique_ptr make_sequence_dataset_name(const Range &range1, const Range &range2, const NameRange &name_range) {
	return sequence_dataset_maker<T, Range>()(range1, range2, name_range);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

template <class T, class Range> struct variant_storage_dataset_maker;

template <class Range> struct variant_storage_dataset_maker<int, Range> {

	typename dataset<sequence_type, variant_storage>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset_storage<sequence_type, variant_storage>(list_of(range1)(range2)); }
	template <class NameRange> typename dataset<sequence_type, variant_storage>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset_storage<sequence_type, variant_storage>(list_of(range1)(range2), name_range); }
};

template <class Range> struct variant_storage_dataset_maker<object, Range> {

	typename dataset<sequence_type, variant_storage>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset_storage<sequence_type, variant_storage>(list_of(make_transform_loader(range1, make_object()))(make_transform_loader(range2, make_object()))); }
	template <class NameRange> typename dataset<sequence_type, variant_storage>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset_storage<sequence_type, variant_storage>(list_of(make_transform_loader(range1, make_object()))(make_transform_loader(range2, make_object())), name_range); }
};

template <class Range> struct variant_storage_dataset_maker<base_object, Range> {

	typename dataset<sequence_type, variant_storage>::unique_ptr operator() (const Range &range1, const Range &range2) { return new_dataset_storage<sequence_type, variant_storage>(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object()))); }
	template <class NameRange> typename dataset<sequence_type, variant_storage>::unique_ptr operator() (const Range &range1, const Range &range2, const NameRange &name_range) { return new_dataset_storage<sequence_type, variant_storage>(list_of(make_transform_loader(range1, make_derived_object()))(make_transform_loader(range2, make_derived_object())), name_range); }
};

template <class T, class Range> typename dataset<sequence_type, variant_storage>::unique_ptr make_variant_storage_dataset(const Range &range1, const Range &range2) {
	return variant_storage_dataset_maker<T, Range>()(range1, range2);
}

template <class T, class Range, class NameRange> typename dataset<sequence_type, variant_storage>::unique_ptr make_variant_storage_dataset_name(const Range &range1, const Range &range2, const NameRange &name_range) {
	return variant_storage_dataset_maker<T, Range>()(range1, range2, name_range);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class data_access {

public:
	template <class T, class U> static data<U> &get_data(data<U> &data) { return data_getter<T, U>::get(data); }
	template <class T, class U> static const data<U> &get_data(const data<U> &data) { return data_getter<T, U>::get(data); }

	template <class T, BOOST_VARIANT_ENUM_PARAMS(class T)> static data<T> &get_data(boost::variant<BOOST_VARIANT_ENUM_PARAMS(T)> &variant) { return *boost::get<boost::shared_ptr<data<T> > >(variant); }
	template <class T, BOOST_VARIANT_ENUM_PARAMS(class T)> static const data<T> &get_data(const boost::variant<BOOST_VARIANT_ENUM_PARAMS(T)> &variant) { return *boost::get<boost::shared_ptr<const data<T> > >(variant); }

private:
	template <class T, class U> struct data_getter {

		static data<U> &get(data<U> &data) { return data; }
		static const data<U> &get(const data<U> &data) { return data; }
	};

	template <class T> struct data_getter<T, T> {

		static data<T> &get(data<T> &data) { return data; }
		static const data<T> &get(const data<T> &data) { return data; }
	};

};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct name_concatenate_visitor : public boost::static_visitor<void> {

	template <class T> void operator()(const data<T> &data) { result += data.get_name(); }

	std::string result;
};

struct mutable_name_concatenate_visitor : public boost::static_visitor<void> {

	template <class T> void operator()(const data<T> &data) const { result += data.get_name(); }

	mutable std::string result;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class dataset_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( dataset_tester );

	CPPUNIT_TEST( test_raw_storage<int> );
	CPPUNIT_TEST( test_raw_storage<object> );
	//CPPUNIT_TEST( test_raw_storage<base_object> ); // "taking address of temporary"
	CPPUNIT_TEST( test_raw_storage<derived_object> );

	//CPPUNIT_TEST( test_raw_storage_sequence<int> ); // "taking address of temporary"
	//CPPUNIT_TEST( test_raw_storage_sequence<object> ); // "taking address of temporary"
	//CPPUNIT_TEST( test_raw_storage_sequence<base_object> ); // "taking address of temporary"

	//CPPUNIT_TEST( test_variant_storage<int> ); // "taking address of temporary"
	//CPPUNIT_TEST( test_variant_storage<object> ); // "taking address of temporary"
	//CPPUNIT_TEST( test_variant_storage<base_object> ); // "taking address of temporary"

	CPPUNIT_TEST_SUITE_END();

	std::vector<int> v1;
	std::vector<int> v2;

public:

	dataset_tester() {

		v1 = {0, 1, 2, 3, 4};
		v2 = {1, 2, 3, 4, 5};
	}

	template <class T, class Dataset> void test_dataset(const std::unique_ptr<Dataset> &datas) {

		CPPUNIT_ASSERT_EQUAL( (size_t)2, datas->size() );
		CPPUNIT_ASSERT_EQUAL( (size_t)5, datas->data_size() );
		CPPUNIT_ASSERT_RANGES_EQUAL( counting_range(0, 5), data_access::get_data<T>((*datas)[0]) );
		CPPUNIT_ASSERT_RANGES_EQUAL( counting_range(1, 6), data_access::get_data<T>((*datas)[1]) );

		// copy
		Dataset datas_copy(*datas);
		CPPUNIT_ASSERT( datas_copy == *datas );
	}

	template <class T, class Dataset> void test_dataset_name(const std::unique_ptr<Dataset> &datas) {

		test_dataset<T, Dataset>(datas);
		CPPUNIT_ASSERT_STRINGS_EQUAL( "name 1", data_access::get_data<T>((*datas)[0]).get_name() );
		CPPUNIT_ASSERT_STRINGS_EQUAL( "name 2", data_access::get_data<T>((*datas)[1]).get_name() );
		test_dataset_visitation(datas);
	}

	template <class Dataset> void test_dataset_visitation(const std::unique_ptr<Dataset> &datas) {

		name_concatenate_visitor v;
		CPPUNIT_ASSERT_STRINGS_EQUAL( "name 1name 2", apply_data_visitor(v, *datas).result );
		v.result.clear();
		CPPUNIT_ASSERT_STRINGS_EQUAL( "name 1name 2", apply_data_visitor(v, static_cast<const Dataset &>(*datas)).result );
		CPPUNIT_ASSERT_STRINGS_EQUAL( "name 1name 2", apply_data_visitor(mutable_name_concatenate_visitor(), *datas).result );
		CPPUNIT_ASSERT_STRINGS_EQUAL( "name 1name 2", apply_data_visitor(mutable_name_concatenate_visitor(), static_cast<const Dataset &>(*datas)).result );
	}

	template <class Dataset> void test_assignment(const std::unique_ptr<Dataset> &datas1, const std::unique_ptr<Dataset> &datas2) {

		CPPUNIT_ASSERT( !(*datas1 == *datas2) );
		*datas1 = *datas2;
		CPPUNIT_ASSERT( *datas1 == *datas2 );
	}

	/*template <class Archive> void register_type(Archive &archive) {

		archive.register_type<data_loader<std::vector<int> > >();
		archive.register_type<data_loader<std::vector<int>, sequence_type> >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_object> > >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_object>, sequence_type> >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_derived_object> > >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_derived_object>, base_object> >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_derived_object>, sequence_type> >();
	}*/

	template <class T> void test_raw_storage() {

		try {

			CPPUNIT_ASSERT_THROW( make_dataset<T>(counting_range(0, 5), counting_range(0, 6)), std::runtime_error );
			test_dataset<T>(make_dataset<T>(counting_range(0, 5), counting_range(1, 6)));
			test_dataset_name<T>(make_dataset_name<T>(counting_range(0, 5), counting_range(1, 6), make_numbered_name_range(numbered_name_generator("name"), 1, 3)));
			test_assignment(make_dataset<T>(counting_range(0, 5), counting_range(1, 6)), make_dataset<T>(counting_range(1, 7), counting_range(2, 8)));

		}
		catch(CppUnit::Exception &e) { throw; }
		catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}

	template <class T> void test_raw_storage_sequence() {

		try {

			CPPUNIT_ASSERT_THROW( make_sequence_dataset<T>(counting_range(0, 5), counting_range(0, 6)), std::runtime_error );
			test_dataset<T>(make_sequence_dataset<T>(counting_range(0, 5), counting_range(1, 6)));
			test_dataset_name<T>(make_sequence_dataset_name<T>(counting_range(0, 5), counting_range(1, 6), make_numbered_name_range(numbered_name_generator("name"), 1, 3)));
			test_assignment(make_sequence_dataset<T>(counting_range(0, 5), counting_range(1, 6)), make_sequence_dataset<T>(counting_range(1, 7), counting_range(2, 8)));

		}
		catch(CppUnit::Exception &e) { throw; }
		catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}

	template <class T> void test_variant_storage() {

		try {

			CPPUNIT_ASSERT_THROW( make_variant_storage_dataset<T>(counting_range(0, 5), counting_range(0, 6)), std::runtime_error );
			test_dataset<T>(make_variant_storage_dataset<T>(counting_range(0, 5), counting_range(1, 6)));
			test_dataset_name<T>(make_variant_storage_dataset_name<T>(counting_range(0, 5), counting_range(1, 6), make_numbered_name_range(numbered_name_generator("name"), 1, 3)));
			test_assignment(make_variant_storage_dataset<T>(counting_range(0, 5), counting_range(1, 6)), make_variant_storage_dataset<T>(counting_range(1, 7), counting_range(2, 8)));

		}
		catch(CppUnit::Exception &e) { throw; }
		catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( dataset_tester );
