/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/data/file/line_range.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/specializations.hpp>
#include <boost/assign/list_of.hpp>

using namespace std;
using namespace util;

class line_range_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( line_range_tester );

	CPPUNIT_TEST( test );
	CPPUNIT_TEST( test_other_range );

	CPPUNIT_TEST_SUITE_END();

public:

	static std::string dos_string;
	static std::string unix_string;
	static std::string macintosh_string;

	void test_string(const std::string &string) {

		typedef line_range<const std::string> lines_type;
		lines_type lines(make_line_range(string));

		CPPUNIT_ASSERT_EQUAL( (size_t)4,  boost::size(lines) );
		CPPUNIT_ASSERT_RANGES_EQUAL( boost::assign::list_of("1")("2")("3")("4"), lines );
	}

	void test() {

		try {

			test_string(dos_string);
			test_string(unix_string);
			test_string(macintosh_string);

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}

	void test_other_range() {

		try {

			typedef boost::iterator_range<std::string::iterator> iterator_range_string;
			typedef line_range<iterator_range_string> string_lines_type;

			CPPUNIT_ASSERT_RANGES_EQUAL( boost::assign::list_of("1")("2")("3")("4"), string_lines_type(make_line_range(boost::make_iterator_range(dos_string))) );
			CPPUNIT_ASSERT_RANGES_EQUAL( boost::assign::list_of("1")("2")("3")("4"), string_lines_type(make_line_range(std::make_pair(boost::begin(dos_string), boost::end(dos_string)))) );

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}
};

std::string line_range_tester::dos_string("1\x0d\x0a""2\x0d\x0a""3\x0d\x0a""4");
std::string line_range_tester::unix_string("1\x0a""2\x0a""3\x0a""4");
std::string line_range_tester::macintosh_string("1\x0d""2\x0d""3\x0d""4");

CPPUNIT_TEST_SUITE_REGISTRATION( line_range_tester );
