/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/data/file/line_filter.hpp>
#include <util/data/file/line_range.hpp>
#include <util/data/file/data_name.hpp>
#include <util/data/file/line_is_not_empty.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/specializations.hpp>
#include <boost/assign/list_of.hpp>

using namespace std;
using namespace util;

class line_filter_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( line_filter_tester );

	CPPUNIT_TEST( test );

	CPPUNIT_TEST_SUITE_END();

public:

	template <class Expected> void test_string(const std::string &string, data_name &data_name, const Expected &expected) {

		typedef line_range<const std::string> lines_type;
		typedef line_filter<line_is_not_empty, const lines_type> filtered_lines_type;
		filtered_lines_type lines(make_line_filter(line_is_not_empty(), make_line_range(string), data_name));

		CPPUNIT_ASSERT_EQUAL( 4, boost::distance(lines) );
		CPPUNIT_ASSERT_RANGES_EQUAL( expected, lines );
	}

	void test() {

		data_name has_data_name_indeterminate;
		data_name has_data_name(true);
		data_name has_no_data_name(false);

		try {

			test_string("1\n2\n3\n4", has_no_data_name, boost::assign::list_of("1")("2")("3")("4"));
			test_string("1\n2\n3\n4", has_data_name_indeterminate, boost::assign::list_of("1")("2")("3")("4"));
			CPPUNIT_ASSERT( !has_no_data_name.get_name() );
			CPPUNIT_ASSERT( !has_data_name_indeterminate.get_name() );

			test_string("a\n1\n2\n3\n4", has_data_name, boost::assign::list_of("1")("2")("3")("4"));
			CPPUNIT_ASSERT( has_data_name.get_name() );
			CPPUNIT_ASSERT_STRINGS_EQUAL( "a", *has_data_name.get_name() );

			test_string("e\n2\n3\n4", has_data_name_indeterminate, boost::assign::list_of("e")("2")("3")("4"));
			test_string("E\n2\n3\n4", has_data_name_indeterminate, boost::assign::list_of("E")("2")("3")("4"));
			CPPUNIT_ASSERT( !has_data_name_indeterminate.get_name() );

			test_string("a\n1\n2\n3\n4", has_data_name_indeterminate, boost::assign::list_of("1")("2")("3")("4"));
			CPPUNIT_ASSERT( has_data_name_indeterminate.get_name() );
			CPPUNIT_ASSERT_STRINGS_EQUAL( "a", *has_data_name_indeterminate.get_name() );

			test_string("\n \n1\n\t\n2\n  \n3\n4\n   ", has_no_data_name, boost::assign::list_of("1")("2")("3")("4"));

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( line_filter_tester );
