/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/data/dataset_collection.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <util/data/dataset/variant_storage.hpp>
#include <util/data/transform_loader.hpp>
#include <boost/mpl/vector.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/range/counting_range.hpp>
#include "../object/object.h"
#include "../object/base_object.h"

using namespace util;
using namespace boost;
using namespace boost::assign;

typedef boost::mpl::vector<int, object, base_object> sequence_type;
typedef boost::mpl::vector<int, object, base_object, sequence_type> collection_sequence_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class dataset_collection_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( dataset_collection_tester );

	CPPUNIT_TEST( test );
	CPPUNIT_TEST( test_sequence );

	CPPUNIT_TEST_SUITE_END();

	std::vector<int> v1;
	std::vector<int> v2;

public:

	dataset_collection_tester() {

		v1 = list_of(0)(1)(2)(3)(4);
		v2 = list_of(1)(2)(3)(4)(5);
	}

	template <class Collection> void test_copy(const Collection &collection) {

		Collection collection_copy(collection);
		CPPUNIT_ASSERT( collection_copy == collection );
	}

	template <class Collection> void test_assignment(const Collection &collection) {

		Collection collection_assign;
		CPPUNIT_ASSERT( !(collection_assign == collection) );
		collection_assign = collection;
		CPPUNIT_ASSERT( collection_assign == collection );
	}

	template <class Archive> void register_type(Archive &archive) {

		archive.register_type<data_loader<std::vector<int> > >();
		archive.register_type<data_loader<std::vector<int>, sequence_type> >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_object> > >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_derived_object> > >();
		archive.register_type<data_loader<transform_loader<std::vector<int>, make_derived_object>, base_object> >();
	}

	void test() {

		try {

			typedef dataset_collection<int> collection_type;

			collection_type datasets;
			datasets.create_dataset();
			datasets.create_dataset(list_of(v1)(v2));
			CPPUNIT_ASSERT_EQUAL( (size_t)2, datasets.size() );

			test_copy(datasets);
			test_assignment(datasets);

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}

	void test_sequence() {

		try {

			typedef dataset_collection<collection_sequence_type, variant_storage> collection_type;

			collection_type datasets;
			datasets.create_dataset<int>();
			datasets.create_dataset<int>(list_of(v1)(v2));
			datasets.create_dataset<object>();
			datasets.create_dataset<object>(list_of(make_transform_loader(v1, make_object()))(make_transform_loader(v2, make_object())));
			datasets.create_dataset<base_object>();
			datasets.create_dataset<base_object>(list_of(make_transform_loader(v1, make_derived_object()))(make_transform_loader(v2, make_derived_object())));
			datasets.create_dataset<sequence_type>(list_of(v1)(v2));
			CPPUNIT_ASSERT_EQUAL( (size_t)7, datasets.size() );

			test_copy(datasets);
			test_assignment(datasets);

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( dataset_collection_tester );
