/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/data/dataset_matrix_adaptor.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/specializations.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/range/counting_range.hpp>
#include <boost/range/algorithm.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/operations.hpp>
#include <boost/mpl/vector.hpp>

using namespace util;
using namespace boost;
using namespace boost::assign;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace boost {

struct int_equality : public static_visitor<bool> {

	int_equality(int value) : value(value) {}

	template <class T> bool operator() (const T &t) const { return value == t; }

	int value;
};

template <BOOST_VARIANT_ENUM_PARAMS(typename T)> bool operator== (int lhs, const variant<BOOST_VARIANT_ENUM_PARAMS(T)> &rhs) {
	return apply_visitor(int_equality(lhs), rhs);
}

} // namespace boost

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef boost::mpl::vector<int, double> sequence_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class dataset_matrix_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( dataset_matrix_tester );

	CPPUNIT_TEST( test_int );
	CPPUNIT_TEST( test_sequence );
	CPPUNIT_TEST( test_sequence_variant_storage );
	CPPUNIT_TEST( test_matrix );

	CPPUNIT_TEST_SUITE_END();

public:

	template <class Matrix, class Function> void for_each1(const Matrix &matrix, const Function &function) {

		typename Matrix::const_iterator1 iterator;
		CPPUNIT_ASSERT_EQUAL( 3, std::distance(matrix.begin1(), matrix.end1()) );
		for(iterator = matrix.begin1() ; iterator != matrix.end1() ; ++iterator)
			function(iterator);
	}

	struct test1 {

		template <class Range> void operator() (const Range &range) const {

			CPPUNIT_ASSERT_EQUAL( 5, std::distance(range.begin(), range.end()) );
			CPPUNIT_ASSERT_RANGES_EQUAL( counting_range((int)range.index1(), (int)range.index1() + 5), boost::make_iterator_range(range.begin(), range.end()) );
		}
	};

	template <class Matrix, class Function> void for_each2(const Matrix &matrix, const Function &function) {

		typename Matrix::const_iterator2 iterator;
		CPPUNIT_ASSERT_EQUAL( 5, std::distance(matrix.begin2(), matrix.end2()) );
		for(iterator = matrix.begin2() ; iterator != matrix.end2() ; ++iterator)
			function(iterator);
	}

	struct test2 {

		template <class Range> void operator() (const Range &range) const {

			CPPUNIT_ASSERT_EQUAL( 3, std::distance(range.begin(), range.end()) );
			CPPUNIT_ASSERT_RANGES_EQUAL( counting_range((int)range.index2(), (int)range.index2() + 3), boost::make_iterator_range(range.begin(), range.end()) );
		}
	};

	template <class Matrix> void test_transposed_matrix(const Matrix &matrix) {

		CPPUNIT_ASSERT_EQUAL( (size_t)3, matrix.size1() );
		CPPUNIT_ASSERT_EQUAL( (size_t)5, matrix.size2() );
		for_each1(matrix, test1());
		for_each2(matrix, test2());
	}

	template <class T, template <class U> class StoragePolicy> void test() {

		try {

			typename dataset<T, StoragePolicy>::auto_ptr datas = new_dataset_storage<T, StoragePolicy>(list_of(counting_range(0, 5))(counting_range(1, 6))(counting_range(2, 7)));
			test_transposed_matrix(numeric::ublas::trans(make_dataset_matrix_adaptor(*datas)));

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}

	void test_int() { test<int, raw_storage>(); }

	void test_sequence() { test<sequence_type, raw_storage>(); }

	void test_sequence_variant_storage() { test<sequence_type, variant_storage>(); }

	// test to simulate dataset with numeric::ublas::vector_of_vector
	void test_matrix() {

		try {

			typedef numeric::ublas::vector_of_vector<int> matrix_type;

			matrix_type m(5, 3);
			matrix_type::iterator2 column_it = m.begin2();
			boost::copy(counting_range(0, 5), column_it++.begin());
			boost::copy(counting_range(1, 6), column_it++.begin());
			boost::copy(counting_range(2, 7), column_it++.begin());

			test_transposed_matrix(numeric::ublas::trans(m));

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( dataset_matrix_tester );
