/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/data/transform_loader.hpp>
#include <util/data/data_loader.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/specializations.hpp>
#include <cppunit/specializations/data/data.hpp>
#include <boost/assign/list_of.hpp>
#include <boost/range/counting_range.hpp>
#include <vector>

using namespace util;

namespace {

class transformed_object {

public:
	transformed_object(int value) : value(value) {}
	transformed_object(const transformed_object &other) : value(other.value) {}

	friend bool operator== (const transformed_object &lhs, const transformed_object &rhs) { return lhs.value == rhs.value; }
	friend std::ostream &operator<< (std::ostream &os, const transformed_object &object) { return os << object.value; }

private:
	int value;
};

struct transform_function : public std::unary_function<int, transformed_object> {

	transformed_object operator() (int value) const { return transformed_object(value); }
};

struct range_loader {

	typedef boost::iterator_range<boost::counting_iterator<int> > iterator_range;

	iterator_range range() const { return boost::counting_range(0, 5); }

	friend bool operator== (const range_loader &, const range_loader &) { return true; }
};

range_loader::iterator_range get_data(const range_loader &loader) { return loader.range(); }

} // namespace

namespace util {

template <> struct loader_data_traits< ::range_loader> {

	typedef ::range_loader::iterator_range type;
};

} // namespace util

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class transform_loader_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( transform_loader_tester );

	CPPUNIT_TEST( test );
	CPPUNIT_TEST( test_range_loader );

	CPPUNIT_TEST_SUITE_END();

public:

	void test() {

		try {

			data<transformed_object>::unique_ptr d(new_data(make_transform_loader(boost::counting_range(0, 5), transform_function())));
			CPPUNIT_ASSERT_EQUAL( (size_t)5, d->size() );
			CPPUNIT_ASSERT_RANGES_EQUAL( boost::counting_range(0, 5), *d );

			// cloning
			data<transformed_object>::unique_ptr d_clone(d->clone());
			CPPUNIT_ASSERT_EQUAL( (size_t)5, d_clone->size() );
			CPPUNIT_ASSERT_RANGES_EQUAL( boost::counting_range(0, 5), *d_clone );

			// equality
			CPPUNIT_ASSERT_EQUAL( *d, *d_clone );

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}

	void test_range_loader() {

		try {

			data<transformed_object>::unique_ptr d(new_data(make_transform_loader( ::range_loader(), transform_function())));
			CPPUNIT_ASSERT_EQUAL( (size_t)5, d->size() );
			CPPUNIT_ASSERT_RANGES_EQUAL( boost::counting_range(0, 5), *d );

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( transform_loader_tester );

