/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#ifndef BASE_OBJECT_H_
#define BASE_OBJECT_H_

#include <cassert>
#include <typeinfo>
#include <boost/utility.hpp>
#include <util/object/collection/store_as_pointer.hpp>

namespace {

class base_object : public boost::noncopyable {

public:
	base_object() {}
	virtual ~base_object() {}

	base_object *clone() const {
		base_object *new_object = do_clone();
		assert(typeid(*new_object) == typeid(*this) && "do_clone incorrectly overridden");
		return new_object;
	}

	friend bool operator== (int lhs, const base_object &rhs) { return rhs.equal(lhs); }
	friend bool operator== (const base_object &lhs, const base_object &rhs) { return (typeid(lhs) == typeid(rhs)) && lhs.equal(rhs); }
	friend std::ostream &operator<< (std::ostream &os, const base_object &object) { return object.to_stream(os); }

protected:
	base_object(const base_object &other) {}

private:
	virtual base_object *do_clone() const = 0;
	virtual bool equal(int value) const = 0;
	virtual bool equal(const base_object &other) const = 0;
	virtual std::ostream &to_stream(std::ostream &os) const = 0;
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

inline base_object *new_clone(const base_object &object) { return object.clone(); }

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class derived_object : public base_object {

public:
	derived_object() : base_object(), value(0) {}
	derived_object(int value) : base_object(), value(value) {}
	derived_object(const derived_object &other) : base_object(other), value(other.value) {}

	derived_object &operator= (const derived_object &other) {
		value = other.value;
		return *this;
	}

	friend bool operator== (int lhs, const derived_object &rhs) { return rhs.equal(lhs); }
	friend bool operator== (const derived_object &lhs, const derived_object &rhs) { return lhs.equal(rhs); }

private:
	int value;

	base_object *do_clone() const { return new derived_object(*this); }
	bool equal(int value) const { return value == this->value; };
	bool equal(const base_object &other) const { return static_cast<const derived_object &>(other).equal(value); }
	std::ostream &to_stream(std::ostream &os) const { return os << value; }
};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct make_derived_object : public std::unary_function<int, derived_object> {

	derived_object operator() (int value) const { return derived_object(value); }
};

} // namespace

#endif /* BASE_OBJECT_H_ */
