/**
 * Util Native Library
 *
 * Copyright (C) 2014 IRSTEA
 *
 * Auteur/Author:
 *     Jean-luc Lablée - http://itap.irstea.fr/?page_id=2277
 *
 * =========================================================================
 *
 * Ce logiciel est régi par la licence CeCILL-C soumise au droit français et
 * respectant les principes de diffusion des logiciels libres. Vous pouvez
 * utiliser, modifier et/ou redistribuer ce programme sous les conditions
 * de la licence CeCILL-C telle que diffusée par le CEA, le CNRS et l'INRIA
 * sur le site "http://www.cecill.info".
 *
 * En contrepartie de l'accessibilité au code source et des droits de copie,
 * de modification et de redistribution accordés par cette licence, il n'est
 * offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
 * seule une responsabilité restreinte pèse sur l'auteur du programme,  le
 * titulaire des droits patrimoniaux et les concédants successifs.
 *
 * A cet égard  l'attention de l'utilisateur est attirée sur les risques
 * associés au chargement,  à l'utilisation,  à la modification et/ou au
 * développement et à la reproduction du logiciel par l'utilisateur étant
 * donné sa spécificité de logiciel libre, qui peut le rendre complexe à
 * manipuler et qui le réserve donc à des développeurs et des professionnels
 * avertis possédant  des  connaissances  informatiques approfondies.  Les
 * utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
 * logiciel à leurs besoins dans des conditions permettant d'assurer la
 * sécurité de leurs systèmes et ou de leurs données et, plus généralement,
 * à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.
 *
 * Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
 * pris connaissance de la licence CeCILL-C, et que vous en avez accepté les
 * termes.
 *
 * =========================================================================
 *
 * This software is governed by the CeCILL-C license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/ or redistribute the software under the terms of the CeCILL-C
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
 */
#include <util/object/object_collection.hpp>
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/specializations.hpp>
#include <cppunit/specializations/object/object_collection.hpp>
#include <boost/mpl/vector.hpp>
#include <boost/range/counting_range.hpp>
#include <boost/range/adaptor/transformed.hpp>
#include <boost/archive/xml_iarchive.hpp>
#include <boost/archive/xml_oarchive.hpp>
#include "../object/object.h"
#include "../object/base_object.h"

using namespace util;
using namespace boost;
using namespace boost::adaptors;

typedef boost::mpl::vector<int, object, base_object> sequence_type;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace boost {

struct int_equality : public static_visitor<bool> {

	int_equality(int value) : value(value) {}

	template <class T> bool operator() (const T &t) const { return value == t; }

	template <class T> bool operator() (const shared_ptr<T> &pointer) const { return value == *pointer; }

	int value;
};

template <BOOST_VARIANT_ENUM_PARAMS(typename T)> bool operator== (int lhs, const variant<BOOST_VARIANT_ENUM_PARAMS(T)> &rhs) {
	return apply_visitor(int_equality(lhs), rhs);
}

} // namespace boost

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

typedef iterator_range<counting_iterator<int> > counting_range_type;

template <class T> struct range_traits;

template <> struct range_traits<int> {

	typedef counting_range_type type;
};

template <> struct range_traits<object> {

	typedef transformed_range<make_object, const counting_range_type> type;
};

template <> struct range_traits<base_object> {

	typedef transformed_range<make_derived_object, const counting_range_type> type;
};

template <> struct range_traits<derived_object> {

	typedef transformed_range<make_derived_object, const counting_range_type> type;
};

template <class T> typename range_traits<T>::type make_range(const counting_range_type &);

template <> range_traits<int>::type make_range<int>(const counting_range_type &counting_range) { return counting_range; }

template <> range_traits<object>::type make_range<object>(const counting_range_type &counting_range) { return transform(counting_range, make_object()); }

template <> range_traits<base_object>::type make_range<base_object>(const counting_range_type &counting_range) { return transform(counting_range, make_derived_object()); }

template <> range_traits<derived_object>::type make_range<derived_object>(const counting_range_type &counting_range) { return transform(counting_range, make_derived_object()); }

template <class T> std::auto_ptr<T> new_object(int value) { return std::auto_ptr<T>(new T(6)); };

template <> std::auto_ptr<base_object> new_object(int value) { return std::auto_ptr<base_object>(new derived_object(6)); }

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

class object_collection_tester : public CppUnit::TestFixture {

	CPPUNIT_TEST_SUITE( object_collection_tester );

	CPPUNIT_TEST( test<int> );
	CPPUNIT_TEST( test<object> );
	CPPUNIT_TEST( test<base_object> );
	CPPUNIT_TEST( test<derived_object> );
	CPPUNIT_TEST( test_sequence<int> );
	CPPUNIT_TEST( test_sequence<object> );
	CPPUNIT_TEST( test_sequence<base_object> );
	CPPUNIT_TEST( test_sequence<derived_object> );

	CPPUNIT_TEST_SUITE_END();

public:

	template <class Collection, class T> void test_collection() {

		try {

			Collection objects(make_range<T>(counting_range(0, 5)));
			CPPUNIT_ASSERT_EQUAL( (size_t)5, objects.size() );
			CPPUNIT_ASSERT_RANGES_EQUAL( counting_range(0, 5), objects );

			// equality
			Collection objects2(make_range<T>(counting_range(0, 5)));
			CPPUNIT_ASSERT_EQUAL( objects, objects2 );

			// copy
			Collection objects_copy(objects);
			CPPUNIT_ASSERT_EQUAL( (size_t)5, objects_copy.size() );
			CPPUNIT_ASSERT_EQUAL( objects, objects_copy );

			// assignment
			objects.assign(make_range<T>(counting_range(1, 6)));
			CPPUNIT_ASSERT_EQUAL( (size_t)5, objects.size() );
			CPPUNIT_ASSERT_RANGES_EQUAL( counting_range(1, 6), objects );

			objects2 = objects;
			CPPUNIT_ASSERT_EQUAL( (size_t)5, objects2.size() );
			CPPUNIT_ASSERT_EQUAL( objects, objects2 );

			// push_back
			objects.push_back(std::auto_ptr<T>(new_object<T>(6)));
			CPPUNIT_ASSERT_EQUAL( (size_t)6, objects.size() );
			CPPUNIT_ASSERT( 6 == objects.back() );

		} catch(std::exception &e) { CPPUNIT_FAIL( e.what() ); }
	}

	template <class T> void test() {
		test_collection<object_collection<T>, T>();
	}

	template <class T> void test_sequence() {
		test_collection<object_collection<sequence_type>, T>();
	}
};

CPPUNIT_TEST_SUITE_REGISTRATION( object_collection_tester );

